require_dependency "login_system"
# Filters added to this controller will be run for all controllers in the application.
# Likewise, all the methods added will be available for all controllers.
class ApplicationController < ActionController::Base

  # include these helpers all the time
  helper  :layout, :display, :navigation, :autocomplete, :specimen, :taxon_name, :figure, :image, :confidence, :ref, :sensu, :content, :geog, :ontology_relationship, :ontology_class, :tag, :image_description

  include ExceptionNotifiable
  
  filter_parameter_logging "password" # likely a good idea
  filter_parameter_logging "person_password" 
  
  include LoginSystem
  
  before_filter :configure_site # forks to a public or private mode, calling the authentications if private
  before_filter :set_charset
  
  # built in Rails-ness
  def method_missing(methodname, *args)
    @methodname = methodname
    @args = args
    public_route_failure and return 
  end
    
  def set_charset
      headers["Content-Type"] = "text/html; charset=utf-8" 
  end
  
# # so that we don't get spammed, emails don't get sent for 404 errors or 'record not found'
# def rescue_action_in_public(exception)
#   if exception.class.name == "ActionController::RoutingError" || exception.class.name == "ActionController::UnknownAction"
#     render(:file => "#{RAILS_ROOT}/public/404.html", :status => "404 Not Found")
#   elsif exception.class.name == "ActiveRecord::RecordNotFound" && self.respond_to?("index") # check that this controller has an index method
#     flash[:notice] = "OOPS! We couldn't find the object you requested. Try again, and contact the administrators if this problem persists."
#     redirect_to(:action => "index") 
#   else
#     logger.info "Sending email for exception: #{exception.class.name}"
#     old_level = logger.level
#     logger.level = Logger::WARN
#     ErrorMailer.deliver_snapshot(exception, clean_backtrace(exception), session.instance_variable_get("@data"), params, request.env)
#     logger.level = old_level
#     render(:file => "#{RAILS_ROOT}/public/500.html", :status => "500 Error")
#   end
# end 
  
  def public_route_failure
    render(:file => "#{RAILS_ROOT}/public/404.html", :status => "404 Not Found")
  end
  
  protected
  
  # aside from this logic here, a site has:
  # - settings in the project table
  # - a layout
  # - style sheets, images, etc
  # - controllers/views (or it can share)
      
  def configure_site  # do we need to setup test.host? 
        
    if params[:unresolvable] # called when bad queries are thrown at the db (i.e routed from *anything)
      public_route_failure and return
    end
   
    # TODO: just make this test for RAILS_ENV=development? 
    # recognizes both http://www.foo.org and http://foo.org 
    if ["0.0.0.0", "127.0.0.1"].include?(request.env["REMOTE_ADDR"]) 
      @server_name = 'development'
    else
      @server_name = request.env["SERVER_NAME"].sub(/^www\./, "")
    end
    
    # pre integration tests
    #    unless (request.env["REMOTE_ADDR"] == "0.0.0.0")  # no server name is set in testing  
    #  @server_name = request.env["SERVER_NAME"].sub(/^www\./, "")
    #end
       
    # private 
    # see environment.rb for setting the HOME_SERVER
    if (@server_name == HOME_SERVER or @server_name == 'development') && (self.class.controller_path[0,7] != "public/") 
      if login_required # this is defined in lib/login_system
        proj_required # so is this - sets @proj
      end
      @public = false
      
    # we're in public, login is not required
    else
      @public = true
      
      # this handles the incoming URLS with nothing past the base URL, like 'http://foo.bar.com'
      if self.controller_name == "proj"   
        # redirect to the appropriate home controller
        unix_name = Proj.find_by_public_server_name(@server_name).unix_name
        redirect_to :action => 'index', :controller => "public/site/#{unix_name}/home" and return
      end
    
      # need to find the pertinent project here
      # the logic is: 1) use the id if given like /projects/id/foo
      # or if /projects is not there (e.g. public or other calls) 2) use the unix name to find the project
      # there is one last case- we're testing, in that case @server_name = testing and no project should be found
      if params[:proj_id] 
        @proj = Proj.find(params[:proj_id])      
      else
        @proj = Proj.find_by_public_server_name(@server_name)  # tests hits here, also sets proj
      end
        
      # if project not found, or class == public, or its not public controller return a 404     
      # if its a home controller then its ok as well!
      is_home_controller = (self.controller_name == "home")
      is_allowed_public_controller = (@proj && self.class.controller_path[0,7] == "public/" && @proj.public_controllers.include?(self.controller_name))
            
      if (!is_allowed_public_controller && !is_home_controller)
         public_route_failure and return
      end     
 
      # ADDED JAN 08 
      # we're in public, but we've been asked for a non-existant controller - MAY BE BAD
      if (!@proj)
         public_route_failure and return
      end     
      
      ## below may be unecessary, check        
      $proj_id = @proj.id # for non-public pages this gets set in # proj_required
  
    end 
    
    true # this method must return true or rails' filter chain halts
  end
  
  class BatchParseError < StandardError
  end
  
end



