class CeController < ApplicationController
  verify :method => :post, :only => [ :destroy, :create, :update ],
    :redirect_to => { :action => :list }
  
  def index
    list
    render :action => 'list'
  end

  def list_params
    @ce_pages, @ces = paginate :ce, :per_page => 25, :conditions => "(proj_id = #{@proj.id})"
  end
  
  def list
    list_params
     if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end

  def _show_params
    id = params[:ce][:id] if params[:ce] # for autocomplete/ajax picker use (must come first!)
    id ||= params[:id]
   
    @ce = Ce.find(id)
  end
  
  def show
    _show_params
    @no_right_col = false
    session['ce_view']  = 'show'
    @show = ('show_default')
  end

  def clone
    ce = Ce.find(params[:id])
    begin
      Ce.transaction do
        @ce = ce.clone
        @ce.locality ? @ce.locality += " [CLONE OF #{ce.id}]\n" : @ce.locality = " [CLONE OF #{ce.id}]\n"
        @ce.save
      end
    rescue
      flash[:notice] = "Failed to clone the Ce, contact an admin."
      redirect_to :action => :show, :id => ce.id and return
    end
    flash[:notice] = "Cloned! From #{ce.id}." 
    render :action => :edit
  end


  def show_material
    _show_params
    @lots = @ce.lots
    @specimens = @ce.specimens
    @no_right_col = true
    session['ce_view']  = 'show_material'
    @show = ('show_material')
    render :action => 'show'
  end
  
  def new
    @ce = Ce.new
  end

  def new_fast
     @target = 'fast'
     @ce = Ce.new
     render :action => 'new_fast'
  end
  
  def create
    @ce = Ce.new(params[:ce])
    if @ce.save
      flash[:notice] = 'Ce was successfully created.'
      if params['target'] == 'fast'
        redirect_to :action => 'new_fast'
      else
        redirect_to :action => :show, :id => @ce.id
      end
        
    else
      render :action => 'new'
    end
  end

  def edit
    @ce = Ce.find(params[:id])
  end

  def update
    @ce = Ce.find(params[:id])
    if @ce.update_attributes(params[:ce])
      flash[:notice] = 'Ce was successfully updated.'

      if params[:update_and_next]
        @id = Ce.find(:first, :conditions => ["proj_id = #{@proj.id} AND id > ?", @ce.id], :order => 'id ASC')
          if @id
            redirect_to(:action => 'edit', :id => @id) 
          else
            flash[:notice] = 'Last record reached.'
            redirect_to(:action => 'list')
          end
      else
        redirect_to :action => 'show', :id => @ce
      end
    else
      render :action => 'list'
    end
  end

  def destroy
    Ce.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
  
  def auto_complete_for_ce
    @tag_id_str = params[:tag_id]
    
    value = params[@tag_id_str.to_sym]

    @ces = Ce.find_for_auto_complete(value)
    render :inline => "<%= auto_complete_result_with_ids(@ces,
      'format_obj_for_auto_complete', @tag_id_str) %>"
  end
 
  # label related functions

  def labels
    @labels = @proj.ces.to_print
  end

  def labels_clear_to_zero
    for l in  @proj.ces.to_print 
       l.num_to_print = nil
       l.save
     end
      
    redirect_to :action => 'labels'
  end

  def labels_print_preview

    # need to make this smarter to count lines, not labels, at that point it should pretty much be set to render
    # perfectly

    # this will change to map more cleverly, its a hack for Jonathon
    if params[:lbl][:type] == '4 dram ETOH'
      @css = 'ce_label_OH'
    else
      @css = 'ce_label'
    end
    
    @lines_per_col = params[:lbl][:lines_per_col].to_i
    @columns_max = params[:lbl][:columns_max].to_i
    @labels = Ce.find(:all, :conditions => ["ces.num_to_print > 0 and proj_id = ?", @proj.id ])

    render :layout => false
  end


  def batch_load
  end

  def batch_verify
     if params[:temp_file][:file].blank?
       flash[:notice] = "Choose a text file with your labels in it before verifying!"
       redirect_to(:action => :batch_load) and return  
     end
    
    # read the contents of the uploaded file, split on pairs of newlines or '---', strip each one and add a newline
    @ces = params[:temp_file][:file].read.split(/\n{2,}|---/m).map {|x| x.strip + "\n"}
  end

  def batch_create
    @count = 0
    begin
      Ce.transaction do
        for ce in params[:ce].keys
          if params[:check][ce]
            c = Ce.new(:verbatim_label => params[:ce][ce], :print_label => params[:ce][ce])
            c.save!
            @count += 1
          end
        end
      end

    rescue
      flash[:notice] = "Something went wrong."
      redirect_to :action => :batch_load and return
    end
     
    flash[:notice] = "Successfully added #{@count} labels." 
    redirect_to :action => :batch_load
  end

  def new_from_gmap
  end
  
  def create_from_gmap
    ce = Ce.new(params[:m])
    if ce.save
        res={:success=>true,:content=>"<div><strong>ID: </strong>#{ce.id}</div><div><strong>Label: </strong>#{ce.label}</div><div><strong>Notes: </strong>#{ce.notes}</div><div><strong>Lat: </strong>#{ce.latitude}</div><div><strong>Long: </strong>#{ce.longitude}</div>"}
      else
        res={:success=>false,:content=>"Could not save the marker!"}
      end
    render :text=>res.to_json
  end
    
  def test_list
    render :text=>(Marker.find :all).to_json
  end

end
