class ImageController < ApplicationController
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
          
  auto_complete_for :image, :owner
         
  def index
    list
    render :action => 'list'
  end

  def list
   @image_description_pages, @descriptions = paginate :image_description, :per_page => 20, :include => [:image, :otu, :image_view], :order => "image_descriptions.image_id",
   :conditions => "image_descriptions.proj_id = #{@proj.id}"
    if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end

  def list_by_id
    @images = @proj.images 
  end
  
  def show
    id = params[:image][:id] if params[:image]
    id ||= params[:id]

    if params[:image] || params[:shared] # we are actually passed an image description id
      @image = (params[:shared] ? ImageDescription.find(params[:id]).image : ImageDescription.find(params[:image][:id]).image)
    else
      @image = Image.find(params[:id])    
    end
  
    @image_descriptions = @image.image_descriptions_by_project(@proj.id)

    if (@image_descriptions.size == 0) 
      flash[:notice] = "You don't have that image described for this project, try adding it first."
      redirect_to :action => :list and return
    end

    @image_description = ImageDescription.new
  end

  
  def new
    @image = Image.new
    @image_description = ImageDescription.new

    # needs some better checking
    @image_description.specimen = Specimen.find(params[:specimen_id]) if params[:specimen_id]
    @image_description.otu = Otu.find(params[:otu_id]) if params[:otu_id]
  end

  def create
    @image = Image.new(params[:image])
    @image_description = ImageDescription.new(params[:image_description])
    if params[:taxon_name]
      if params[:taxon_name][:id].to_i > 0  
        taxon_id = params[:taxon_name][:id]
       if Otu.find_by_taxon_name_id_and_proj_id(params[:taxon_name][:id], @proj.id)   
         @image_description.errors.add_to_base "There is already an OTU assciated with that taxon name. Use the existing OTU or manually create a new one."
         flash[:notice] = "Failed to create image."
         render :action => 'new' and return
       end
     end
   end

    begin
      Image.transaction do
        @image.save!
        @image_description.image_id = @image.id
        @otu = Otu.create!(:taxon_name_id => params[:taxon_name][:id]) if taxon_id
        @image_description.otu_id = @otu.id if @otu # must always be present?
        @image_description.save!
        flash[:notice] = 'Image was successfully created.'
        redirect_to :action => 'show', :id => @image
      end
    rescue ActiveRecord::RecordInvalid, ActiveRecord::RecordNotSaved
      flash[:notice] = 'Failed to create image.'
      render :action => 'new'
    end
  end

  def edit
    @image = Image.find(params[:id])

    # somewhat weak 
    if @image.image_descriptions_by_project(@proj.id).size == 0 
      flash[:notice] = "You don't have that image described for this project, try adding it first."
      redirect_to :action => :list and return
    end

  end

  def update
    @image = Image.find(params[:id])

    # somewhat weak 
    if @image.image_descriptions_by_project(@proj.id).size == 0 
      flash[:notice] = "You don't have that image described for this project, try adding it first."
      redirect_to :action => :list and return
    end

    if @image.update_attributes(params[:image])
      flash[:notice] = 'Image was successfully updated.'
      redirect_to :action => 'show', :id => @image
    else
      render :action => 'edit'
    end
  end

  def destroy
    if o = Image.find(params[:id])
      begin
        o.destroy 
        flash[:notice] =  "Image deleted."         
      rescue
        flash[:notice] =  "Can't delete image, you're likely using it in a figure, or it belongs to another project."         
      end
    else
      flash[:notice] =  "Can't find that image!" 
    end 

    redirect_to :action => 'list'
  end

  def auto_complete_for_image
    @tag_id_str = params[:tag_id]
    
    if @tag_id_str == nil
      redirect_to(:action => 'index', :controller => 'image') and return
    else
      value = params[@tag_id_str.to_sym].split.join('%') # hmm... perhaps should make this order-independent
      @id = ImageDescription.find(:all, :conditions => ["(images.id LIKE ? OR otus.name like ? or labels.name like ? or image_views.name like ? or images.user_file_name like ?) AND image_descriptions.proj_id = ?", value.gsub(/\%/, ""), "%#{value}%",  "%#{value}%", "%#{value}%", "%#{value}%", @proj.id], :include => [:image, :otu, :label, :image_view], :order => 'images.id' )
    end
    
    render :inline => "<%= auto_complete_result_with_ids(@id,
      'format_image_description_for_autocomplete', @tag_id_str) %>"
  end

  # testing only
  def input_draw
    @figure = Figure.find(2989) # 2990 
  end

end
