require 'ontology/ontology_methods'

class LabelController < ApplicationController

  verify :method => :post, :only => [ :destroy, :create ], # :update
    :redirect_to => { :action => :list }

  before_filter :_show_params, :only => [:show, :show_tags]

  def index
    list
    render :action => 'list'
  end

  def list_params
    @label_pages, @labels = paginate :label, :per_page => 25, :conditions => "(proj_id = #{@proj.id})"
  end

  def list
    list_params
    if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end

  def new
    @label = Label.new
    @label.plural_of_label_id = params[:id] if !params[:id].blank?
  end

  def create
    @label = Label.new(params[:label])
    if @label.save
      flash[:notice] = "Label was successfully created."
      redirect_to :action => :show, :id => @label
    else
      render :action => :new
    end
  end

  def destroy
    @label = Label.find(params[:id])
    if @label.destroy
      flash[:notice] = "Label destroyed."
      redirect_to :action => :list
    else
      flash[:notice] = "Can not destroy label, it is likely used with an ontology class that has a dbxref."
      redirect_to :action => :show, :id => @label
    end
  end

  def show
    session['label_view']  = 'show'
    @show = ('show_default')
    @no_right_col = true
    render :action => 'show'
  end

  def show_tags
    session['label_view']  = 'show_tags'
    @tags = @label.tags.group_by{|o| o.addressable_type}
    @show = ('show_tags')
    @no_right_col = true
    render :action => 'show'
  end

  def edit
    @label = Label.find(params[:id], :include => [:ontology_classes])
  end

  def update
    @label = Label.find(params[:id])
    if @label.update_attributes(params[:label])
      flash[:notice] = "Updated."
      redirect_to :action => :show, :id => @label
    else
      flash[:notice] = "Failed to update, did the proposed change result in a duplicate label?"
      redirect_to :action => :edit, :id => @label
    end
  end

  def list_by_keyword
    render :action => :list_by_keyword and return if params[:kw].blank? || params[:kw][:id].blank? 
    respond_to do |format|
      format.html { 
      }
      format.js { # hit on request 
        @keyword = Keyword.find(params[:kw][:id])
        render :update do |page|
          page.replace_html :results, :partial => 'list_edit_ontology_classes', :locals => {:keyword => @keyword, :labels => @proj.labels.tagged_with_keyword(@keyword)}
        end and return
      }
    end      
  end

  def list_latest
    @tile = "Latest edits"
    @labels = @proj.labels.recently_changed(1.weeks.ago).ordered_by_updated_on.ordered_by_name 
    render :file => 'label/generic_list', :use_full_path => true, :layout => true
  end

  def list_without_definitions
    @labels = @proj.labels.singular.without_ontology_classes.ordered_by_name
    @title = "Labels without associated ontology classes (#{@labels.size})"
    render :file => 'label/generic_list',:use_full_path => true, :layout => true
  end

  def list_homonyms
    @labels = OntologyMethods.homonyms(:proj_id => @proj.id)
    @title = "Homonymous labels (#{@labels.size})"
    render :file => 'label/generic_list_with_ontology_classes', :use_full_path => true, :layout => true
  end

  def list_alpha
    if params[:letter] == nil
      @labels = []
    else
      @labels = @proj.labels.with_first_letter(params[:letter]).singular.ordered_by_name
    end
  end

  def list_without_plural
    @labels = @proj.labels.singular.without_plural_forms.ordered_by_name[0..20]
  end

  def list_lbls_in_defs_wo_ontology_classes
    @result_hash = Label.without_ontology_classes_but_used_in_ontology_class_definitions(:proj_id => @proj)
  end

  def _show_params
    id = params[:label][:id] if params[:label]
    id ||= params[:id]
    @label = Label.find(id)
  end

  def auto_complete_for_label
    @labels = Label.auto_complete_search_result(params.merge!(:proj_id => @proj.id))
    render :inline => "<%= auto_complete_result_with_ids(@labels,
          'format_obj_for_auto_complete', @tag_id_str) %>"
  end

end
