class LotController < ApplicationController
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list } 
  
  layout 'layouts/application',  :except => :grand_summary_xsl
  
  def index
    list
    render :action => :list
  end

  def list  
    @lots = Lot.find(:all, :conditions => ["lots.proj_id = ?", @proj.id], :include => [:otu, :ce, :repository])
    @ids = LotIdentifier.find_all_by_lot_id(params[:id])
    @lot_pages, @lots = paginate :lots, :per_page => 30, :conditions => "(proj_id = #{@proj.id})"
    if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end

  def show
    id = params[:lot][:id] if params[:lot] # for ajax picker use
    id ||= params[:id]
  
    if @lot = Lot.find(id, :include => 'lot_identifiers')
    else
     flash[:notice] =  "Lot #{id} not found."  
     redirect_to :action => :list
   end
    
    session['lot_view']  = 'show'
    @show = ('show_default') 
  end

  # just set some params
  
  def new
    @lot = Lot.new
    _lot_params
  end

  def create
    @lot = Lot.new(params[:lot])

    if @lot.save 
      
      if !params[:lot_identifier][:namespace_id].blank? && !params[:lot_identifier][:identifier].blank?
        @lot.lot_identifiers << LotIdentifier.new(params[:lot_identifier])
      end

      flash[:notice] = 'Lot was successfully created.'
   
      # some crude redirect fu
      if params[:commit] == 'Create, clone and next'
        @lot = Lot.new
        _lot_params
        @lot_identifier = @lotID
        @lot_identifier.identifier = ''
        render :action => 'new' and return
      elsif params[:commit] == 'Create and next' 
        @lot = Lot.new
        @lot_identifier = LotIdentifier.new
        _lot_params
        render :action => :new and return
      else
        redirect_to :action => :show, :id => @lot
      end
    else # couldn't save
      render :action => :new
    end
  end

  def edit
    @lot = Lot.find(params[:id])
    _lot_params
  end

  def update
    @lot = Lot.find(params[:id])

    if @lot.update_attributes(params[:lot])
      flash[:notice] = 'Lot was successfully updated.' ## not really, see below
      unless (params[:lot_identifier][:namespace_id] == "") or (params[:lot_identifier][:identifier] == "")
        @lotID = LotIdentifier.new
        @lotID.identifier = params[:lot_identifier][:identifier]
        @lotID.lot_id = @lot.id # is the id from the just saved lot
        @lotID.namespace_id = params[:lot_identifier][:namespace_id]
        @lotID.save! ## NEED TO throw errors on this
      end
      redirect_to :action => :show, :id => @lot
    else
      render :action => :edit
    end
  end

  def destroy
    Lot.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
  
  def auto_complete_for_lot
    @tag_id_str = params[:tag_id]
    value = params[@tag_id_str.to_sym]
        
    @lots = Lot.find_for_auto_complete(value)
    render :inline => "<%= auto_complete_result_with_ids(@lots,
      'format_obj_for_auto_complete', @tag_id_str) %>"
  end

  # returns variables for grand_summary functions
  def gs
     @records = Lot.find_by_sql(["SELECT l.proj_id, l.notes, lis.identifier as i, o.id, Sum(l.key_specimens) AS nsk, Sum(l.value_specimens) AS nsv, sum(l.key_specimens + l.value_specimens) AS ns
                    FROM (lots AS l LEFT JOIN lot_identifiers AS lis ON l.id = lis.lot_id) LEFT JOIN otus AS o ON l.otu_id = o.id
                    GROUP BY lis.identifier, o.id
                    HAVING (l.proj_id = ?);", @proj.id])
     @tot_unique_ids = Lot.find_by_sql(["Select distinct lis.identifier FROM (lots AS l LEFT JOIN lot_identifiers AS lis ON l.id = lis.lot_id) Where (l.proj_id = ?);", @proj.id]).size
     @tot_specimens = @records.inject(0) do |sum, o| sum += o.ns.to_i end
   end
  
  def grand_summary
      gs
      render :partial => 'grand_summary'
  end

  def grand_summary_xsl
    # experimental, likely only works with Office 2003
    headers['Content-Type'] = "application/vnd.ms-excel" 
    headers['Content-Disposition'] = 'attachment; filename="lots_grand_summary.xls"'
    headers['Cache-Control'] = ''
    @columns = ['Identifier', 'Otu', '#specimens']
    gs
  end 


  def clone_to_specimen
    if @l = Lot.find(params[:id])
      begin
          Specimen.transaction do
            @s = Specimen.new
            @s.ce_id = @l.ce_id if @l.ce_id
            @s.notes = "Cloned from lot #{@l.id}." + (@l.notes.size > 0 ? " #{@l.notes}" : '')
            @s.repository_id = @l.repository_id if @l.repository_id
            @s.dna_usable = @l.dna_usable
            @s.temp_ce = @l.ce_labels if @l.ce_labels
            @s.save
            
            @d = SpecimenDetermination.new
            @d.otu_id = @l.otu_id  # if @l.otu_id
            @d.specimen = @s
            @d.save
          end
          flash[:notice] = 'Specimen cloned extracted from lot.'
          redirect_to(:action => 'edit', :controller => 'specimen', :id => @s) and return
      rescue
       flash[:notice] = "Something went wrong in clone" 
      end
    end
    redirect_to :action => 'show', :id => @l
  end

  
  def extract_specimen
      if @l = Lot.find(params[:id])
      begin
          Specimen.transaction do
            @s = Specimen.new
            @s.ce_id = @l.ce_id if @l.ce_id
            @s.notes = "Cloned from lot #{@l.id}.\n" + (@l.notes.size > 0 ? " #{@l.notes}" : '')
            @s.repository_id = @l.repository_id if @l.repository_id
            @s.dna_usable = @l.dna_usable
            @s.temp_ce = @l.ce_labels if @l.ce_labels
            @s.save
            
            @d = SpecimenDetermination.new
            @d.otu_id = @l.otu_id  # if @l.otu_id
            @d.specimen = @s
            @d.save
            
            if params[:remove_from] == 'value'
              @l.value_specimens = @l.value_specimens - 1
            elsif params[:remove_from] == 'key'
              @l.key_specimens = @l.key_specimens - 1
            else
              raise "something went wrong"
            end
            
            @l.save
          end
          flash[:notice] = 'Specimen successfully extracted from lot.'
          redirect_to(:action => 'edit', :controller => 'specimen', :id => @s) and return
      rescue
       flash[:notice] = "Something went wrong in clone" 
      end
    end
    redirect_to :action => 'show', :id => @l
  end

  private
 
  def _lot_params
    @ids = @lot.lot_identifiers 
    @lot_identifier = LotIdentifier.new
  end
  

end
