class MxController < ApplicationController
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  # TODO: this should be elsewhere?
  include ActionView::Helpers::TextHelper

  layout "layouts/application",  :except => [:tnt_as_file, :nexus_as_file]
  # layout "layouts/application",  :except => :nexus_as_file

  def index
    list
    render :action => 'list'
  end

  def list
    list_params
     if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end

  def show
    id = params[:mx][:id] if params[:mx] # for autocomplete/ajax picker use (must come first!)
    id ||= params[:id] 
    @mx = Mx.find(id)
   
    respond_to do |format|
      format.html {
        session['mx_view']  = 'show'
        @show = ('show_default') # not redundant with above- @show necessary for multiple display of items 
      }
      format.xml {
        render :action => 'nexml/nexml', :layout => false
      }
      format.rdf {
        @transform = true
        render :action => 'nexml/nexml', :layout => false
      }
    end
  end

  def new
    @mx = Mx.new
  end

  def create
    @mx = Mx.new(params[:mx])
    if @mx.save
      flash[:notice] = 'Mx was successfully created.'
      redirect_to :action => 'show', :id => @mx.id
    else
      render :action => :edit
    end
  end

  def edit
    @mx = Mx.find(params[:id])
  end

  def update
    @mx = Mx.find(params[:id])
    if @mx.update_attributes(params[:mx])
      flash[:notice] = 'Mx was successfully updated.'
      redirect_to :action => 'show', :id => @mx.id
    else
      render :action => :edit
    end
  end

  def destroy
    @mx = Mx.find(params[:id])
    if @mx.destroy 
      flash[:notice] = 'Successfully destroyed.'
    else
      flash[:notice] = 'Error destroying matrix.'
    end
    redirect_to :action => :list
  end

  def show_batch_code
    @mx = Mx.find(params[:id])  
    @chrs = @mx.chrs
    @otus = @mx.otus

    @no_right_col = true

    session['mx_view']  = 'show_batch_code'
    @show = ('show_batch_code')
    render :action => :show
  end

  def show_data_sources
    @mx = Mx.find(params[:id], :include => :data_sources)  
    @no_right_col = true

    session['mx_view']  = 'show_data_sources'
    @show = ('show_data_sources')
    render :action => :show
  end

  def show_nexus
    @mx = Mx.find(params[:id])  
    @chrs = @mx.chrs
    @otus = @mx.otus
    # @codings = @mx.codings
    @codings_mx = @mx.codings_mx

    @no_right_col = true
    session['mx_view']  = 'show_nexus'
    @show = ('show_nexus') 
    render :action => :show
  end

  def show_tnt
    @mx = Mx.find(params[:id])  
    @chrs = @mx.chrs
    @otus = @mx.otus
    # @codings = @mx.codings
    @codings_mx = @mx.codings_mx

    @no_right_col = true

    session['mx_view']  = 'show_tnt'
    @show = ('show_tnt') 
    render :action => :show
  end

  def show_ascii
    @mx = Mx.find(params[:id])  
    @chrs = @mx.chrs
    @otus = @mx.otus
    @codings_mx = @mx.codings_mx
    render :action => :show_ascii, :layout => false
  end


  def tnt_as_file
    @mx = Mx.find(params[:id])  
    @chrs = @mx.chrs
    @otus = @mx.otus
    @codings_mx = @mx.codings_mx

    f = render_to_string(:partial => "tnt", :layout => false)
    send_data(f, :filename => 'mx_matrix.tnt', :type => "application/rtf", :disposition => "attachment")
    # DO NOT USE REDIRECT/RENDER HERE
  end
 
  def show_trees
    @mx = Mx.find(params[:id]) 
    @trees = @mx.trees
    @no_right_col = true
    session['mx_view']  = 'show_trees'
    @show = ('show_trees') # not redundant with above- @show necessary for multiple display of items 
    render :action => :show
  end

  def show_otus
   @mx = Mx.find(params[:id])  
    _set_otus
    
    @chr = @mx.chrs.first # first chr to point 'code' at 
    @no_right_col = true
    session['mx_view']  = 'show_otus'
    @show = ('show_otus')
    render :action => :show
  end

  def show_chrs
    @mx = Mx.find(params[:id])  
    _set_chrs
    
    @otu = @mx.otus.first # first otu to 'code' at
    @no_right_col = true
    session['mx_view']  = 'show_chrs'
    @show = ('show_chrs')
    render :action => :show
  end

  def show_grid_coding
    @mx = Mx.find(params[:id])  
    grid_coding_params

    @time = Benchmark.measure{
       @codings_mx = @mx.codings_mx
    }.to_s

    @no_right_col = true
    session['mx_view']  = 'show_grid_coding'
    @show = ('show_grid_coding') 
    render :action => :show
  end


  def show_grid_coding2
    @mx = Mx.find(params[:id])  
    grid_coding_params

      @time = Benchmark.measure{
       @codings_mx =  @mx.codings_in_grid({})
     }.to_s

    @no_right_col = true
    session['mx_view']  = 'show_grid_coding2'
    @show = ('show_grid_coding2') 
    render :action => :show
  end

  # sloooow
  def show_grid_tags
    @mx = Mx.find(params[:id])  
    grid_coding_params

    @codings_mx = @mx.codings_mx
    @no_right_col = true
    session['mx_view']  = 'show_grid_tags'
    @show = ('show_grid_tags') 
    render :action => :show
  end

  # TODO: move logic to model
  # This method provides one-click coding, iterating through either chrs or OTUs
  # It handles both the post and show aspects.
  def fast_code
      # regardless of whether we navigate with ajax, or by post, we need these: 
      @mx = Mx.find(params[:id])
      @confidences = @proj.confidences
      @mode = params[:mode] # 'row' or 'col', depending on the direction we're coding
 
      @present_position = params[:position].to_i 

      # general setup
      @tag = Tag.new
      @no_right_col = true

      @otus = @mx.otus
      @chrs = @mx.chrs

      # Pull up the OTU and Chr 
      # and the other based on :present_position
      if @mode == 'row'
        # we need the full list of @chrs for positioning
        unless @chrs.length > @present_position # these check for Ajax, the checks below check for POST
          flash[:notice] = "You've finished one-click coding for that OTU." 
          redirect_to :action => :show_otus, :id => @mx.id and return
        end

        # WHAT IS THIS?!
        @otu = Otu.find(:first, :conditions => {:proj_id => @proj.id, :id => params[:otu_id]})
        @chr = @chrs[@present_position]

      elsif @mode == 'col'
        unless @otus.length > @present_position # these check for Ajax, the checks below check for POST
          flash[:notice] = "You've finished one-click coding for that character." 
          redirect_to :action =>'show_chrs', :id => @mx.id and return
        end       
        # and WHY THIS?!
        @chr = Chr.find(:first, :conditions => {:proj_id => @proj.id, :id => params[:chr_id]})
        @otu = @otus[@present_position]
      else
        redirect_to :action => :index and return # illegal mode
      end

      # add/delete actions
      if params[:nuke] == 'true'   # nuke states for this combination
        Coding.destroy_by_otu_and_chr(Otu.find(params[:otu_id]), Chr.find(params[:chr_id]))
      
      elsif request.post?  
        if !params[:chr_state_id].blank?
          chr_state = ChrState.find(params[:chr_state_id])
          
          # from form: otu_id, chr_id, chr_state_id
          # handled by model: chr_state_state, chr_state_name
          # use find_or_create here?
          @coding = Coding.find(:first, :conditions => ["otu_id = ? AND chr_id = ? AND chr_state_id = ?", @otu.id, @chr.id, chr_state.id])
          if @coding # existing coding has been clicked, delete it
            @coding.destroy
          else
            coding = Coding.new(:otu_id => @otu.id, 
                                :chr_id => @chr.id,
                                :chr_state_id => chr_state.id)

            coding.confidence_id =  params[:coding][:confidence_id] if params[:coding] && params[:coding][:confidence_id]
		      	coding.save
            @coding = coding
          end

          @present_position = @present_position + 1

          # include a tag if requested
          if !params[:tag].blank? and !params[:tag][:keyword_id].blank? 
            t = Tag.new(params[:tag])
            t.addressable_id = @coding.id
            t.addressable_type = 'Coding'
            t.save
          end
        end
      end

      # A lot of this code is repeated from above, but that avoids the need to call this action 
      # twice per coding, which improves performance a lot
      if @mode == 'row'
        @chr = @chrs[@present_position]
        unless @chrs.length > @present_position # these check for POST, the checks above check for AJAX
          flash[:notice] = "You've finished one-click coding for that OTU." 
          redirect_to :action =>'show_otus', :id => @mx.id and return
        end      
      elsif @mode == 'col'
        unless @otus.length > @present_position # these check for POST, the checks above check for AJAX
          flash[:notice] = "You've finished one-click coding for that character." 
          redirect_to :action =>'show_chrs', :id => @mx.id and return
        end
        @otu = @otus[@present_position]
      end

      @last_otu = (@mode == 'row' ? @otu : @otus[@present_position - 1])
      @last_chr = (@mode == 'col' ? @chr : @chrs[@present_position - 1]) 
      @previous_position ||= @present_position
      
      # render the updates
      respond_to do |format|
        format.html {      
          session['mx_view']  = 'fast_coding'
          @show = ('fast_coding') 
          render :action => :show
        }
        format.js { # AJAX
          render :update do |page|
            page.replace_html :notice, flash[:notice]
            page.replace_html :fast_coding_form, :partial => 'fc'
            flash.discard
        end and return }
      end   
  end

  def show_code
    @mx = Mx.find(params[:id])
    @otu = Otu.find(params[:otu_id])
    @chr = Chr.find(params[:chr_id])
    @confidences = @proj.confidences

    codings = []
    # move logic to model? 
    if request.post?
      @codings = Coding.by_chr(@chr).by_otu(@otu)
      params[:state].each_pair {|chr_state_id, coded|
        chr_state = ChrState.find(chr_state_id.to_i)
        if (coding = @codings.detect {|c| c.chr_state_id == chr_state.id}) # coding exists? 
          if coded == "0"
            coding.destroy
          else # exists, but confidence might have changed
            coding.update_attributes(:confidence_id => ((params[:confidence] && params[:confidence][chr_state.id.to_s]) ? params[:confidence][chr_state.id.to_s] : nil) )
            codings.push coding
          end
        else # coding doesn't exist 
          if coded == "1"
            coding = Coding.create(
              "otu_id" => @otu.id,
              "chr_id" => @chr.id,
              "chr_state_id" => chr_state.id,
              # "chr_state_state" => chr_state.state, # set on before_filter
              # "chr_state_name" => chr_state.name,
              :confidence_id => (params[:confidence] ? params[:confidence][chr_state.id.to_s] : nil),
              "proj_id" => @proj.id 
            ) 
            codings.push coding
          end
        end        
      }
      flash[:notice] = "Updated."
    end
  
    if params[:from_grid_coding]
      # should make these locals 
      @x = params[:x]
      @y = params[:y]
      cell_type = session["#{$person_id}_mx_overlay"] if not session["#{$person_id}_mx_overlay"].blank?
      cell_type ||= 'none' 
      render :update do |page|
        page.replace_html :cell_zoom, :partial => 'grid_cell_zoom' 
        page.replace_html "cell_#{params[:x]}_#{params[:y]}", :partial => "cell_#{cell_type}", :locals => {:i => params[:x], :j => params[:y], :o => @otu, :c => @chr, :mx_id => @mx.id, :codings => codings} 
      end and return
    else

    @adjacent_cells = @mx.adjacent_cells(:otu_id => @otu.id, :chr_id => @chr.id)
    @no_right_col = true
    session['mx_view']  = 'show_grid_coding'
    @show = ('show_code') 
      render :action => :show, :id => @mx.id, :otu_id => @otu.id, :chr_id => @chr.id and return
    end
  end 
 
  # managing chrs

  def add_chr
    @mx = Mx.find(params[:mx][:id])
    if !params[:chr_group_id].blank?
      begin
       @mx.add_group(ChrGroup.find(params[:chr_group_id]))
        flash[:notice] = "Added a character group."
      rescue
        flash[:notice] = "Problem adding character group, is it already present?"
      end
    end
    
    if params[:mx_chr] && !params[:mx_chr][:plus_id].blank?
      c = Chr.find(params[:mx_chr][:plus_id])
      @mx.chrs_plus << c if c
      @mx.save! 
    end
    
  if params[:mx_chr] && !params[:mx_chr][:minus_id].blank?
      c = Chr.find(params[:mx_chr][:minus_id])
      @mx.chrs_minus << c if c 
    end
    
    redirect_to :action => :show_chrs, :id => @mx.id    
  end

  def remove_chr
    @mx = Mx.find(params[:id])
    if @mx
      @mx.remove_group(ChrGroup.find(params[:chr_group_id])) if params[:chr_group_id]
      @mx.remove_from_plus(Chr.find(params[:chr_id])) if params[:chr_id]
      @mx.remove_from_minus(Chr.find(params[:minus_chr_id])) if params[:minus_chr_id]
    else
      render :action => :list and return
    end
    redirect_to :action => :show_chrs, :id => @mx.id    
  end

  # managing OTUs
  
  def add_otu
    redirect_to :action => :list and return if params[:mx].blank?
    @mx = Mx.find(params[:mx][:id])
    if !params[:otu_group_id].blank?
      begin
       @mx.add_group(OtuGroup.find(params[:otu_group_id]))
        flash[:notice] = "Added a character group."
      rescue
        flash[:notice] = "Problem adding character group, is it already present?"
      end
    end
    
     
   if params[:add_otu_minus]
      if params[:otu_minus] && !params[:otu_minus][:id].blank? 
         o = Otu.find(params[:otu_minus][:id]) 
         @mx.otus_minus << o if o 
      end 
    elsif params[:add_otu_plus]
      if params[:otu_plus] && !params[:otu_plus][:id].blank? 
        o = Otu.find(params[:otu_plus][:id])
        @mx.otus_plus << o if o
      end 
    end

    @mx.save! 
    
    redirect_to :action => :show_otus, :id => @mx.id    
  end
 
  def remove_otu
    @mx = Mx.find(params[:id])
    if @mx 
      @mx.remove_group(OtuGroup.find(params[:otu_group_id])) if params[:otu_group_id]
      @mx.remove_from_plus(Otu.find(params[:otu_id])) if params[:otu_id]
      @mx.remove_from_minus(Otu.find(params[:minus_otu_id])) if params[:minus_otu_id]
    else
      render :action => :list and return
    end
    redirect_to :action => :show_otus, :id => @mx.id    
  end
  
  # otu sorting

  def show_sort_otus
    @mx = Mx.find(params[:id])
    @mxes_otus = @mx.mxes_otus
    session['mx_view']  = 'show_sort_otus'
    @show = ('show_sort_otus') 
    @no_right_col = true
    render :action => :show, :id => @mx.id and return
  end


  def reset_otu_positions
   if @mx = Mx.find(params[:id])
      @mx.reset_otu_positions
      flash[:notice] = 'order reset'
    else
      redirect_to :action => :list
      flash[:notice] = "Can't find matrix with id #{params[:id]}."
    end
    redirect_to :action => :show_sort_otus, :id => @mx.id
  end


  def sort_otus
    params[:otus].each_with_index do |id, index|
      MxesOtu.update_all(['position=?', index+1], ['id=?', id])
    end
    render :nothing => true
  end


  # character sorting
  
  def show_sort_chrs
    @mx = Mx.find(params[:id])
    @chrs_mxes = @mx.chrs_mxes
    session['mx_view']  = 'show_sort_chrs'
    @show = ('show_sort_chrs') 
    @no_right_col = true
    render :action => :show, :id => @mx.id and return
  end

  def reset_chr_positions
    if @mx = Mx.find(params[:id])
      @mx.reset_chr_positions
      flash[:notice] = 'order reset'
    else
      redirect_to :action => :list
      flash[:notice] = "Can't find matrix with id #{params[:id]}."
    end
    redirect_to :action => :show_sort_chrs, :id => @mx
  end

  def sort_chrs
    params[:chrs].each_with_index do |id, index|
      ChrsMx.update_all(['position=?', index+1], ['id=?', id])
    end
    render :nothing => true
  end

  def invalid_codings
    @invalid_codings = Coding.invalid(@proj.id)
  end

  # misc
  def auto_complete_for_mx
      @tag_id_str = params[:tag_id]
      
      if @tag_id_str == nil
        redirect_to(:action => 'list', :controller => 'mx') and return
      else
        value = params[@tag_id_str.to_sym].split.join('%') # hmm... perhaps should make this order-independent
        @mxes = Mx.find(:all, :conditions => ["(name LIKE ? OR id = ?) AND proj_id=?", "%#{value}%", value.gsub(/\%/, ""), @proj.id], :order => "name")
      end
      
      render :inline => "<%= auto_complete_result_with_ids(@mxes, 'format_obj_for_auto_complete', @tag_id_str) %>"
    end

  def browse
    @matrix = Mx.find(params[:id])
    @total_chrs = @matrix.chrs.count
    @total_otus = @matrix.otus.count
 
    if @total_chrs == 0 || @total_otus == 0
      flash[:notice] = "Populate your matrix with some characters or OTUs before browsing it."
      redirect_to :action => :show, :id => @matrix and return
    end 

    @cell_type = session["#{$person_id}_mx_overlay"] if !session["#{$person_id}_mx_overlay"].blank?
    @cell_type ||= 'none' 

    person = Person.find($person_id)

    respond_to do |format|
		  format.html {} # default .rhtml
        @window = {:chr_start => 1, :otu_start => 1, :chr_end => person.pref_mx_display_width, :otu_end => person.pref_mx_display_height} 
        @mx = @matrix.codings_in_grid(@window) 
        
        # simplify several calculations for the view
        @oes = @window[:otu_end] - @window[:otu_start];  @ces = @window[:chr_end] - @window[:chr_start]; 
      format.js { 
        _get_window_params 
        render :update do |page|
          page.replace_html :window_to_update, :partial => 'window'
          page.replace_html :cell_zoom, :text => nil
          # page.visual_effect :fade, "tl_#{@obj.class.to_s}_#{@obj.id}"
          # page.insert_html :bottom, "t_#{@obj.class.to_s}_#{@obj.id}", :partial => 'popup_form'
        end and return 
      }
		end
  end

  # TODO protect
  def _get_window_params
   @window = @matrix.slide_window(params) 
   @oes = @window[:otu_end] - @window[:otu_start];  @ces = @window[:chr_end] - @window[:chr_start]; 
   @mx = @matrix.codings_in_grid(@window)
  end

  def _cell_zoom
    @x = params[:x]
    @y = params[:y]
    @otu = Otu.find(params[:otu_id])
    @chr = Chr.find(params[:chr_id])
    @confidences = @proj.confidences
    @mx = Mx.find(params[:mx_id])
    render :update do |page|
     page.replace_html :cell_zoom, :partial => 'grid_cell_zoom'
    end
  end

  def _otu_zoom
    @matrix = Mx.find(params[:id])
    @otu = Otu.find(params[:otu_id])
    @unique_codings = @otu.unique_codings 
    render :update do |page|
      page.replace_html :cell_zoom, :partial => 'otu_zoom'
    end
  end

  def _set_overlay_preference
    @matrix = Mx.find(params[:id])
    session["#{$person_id}_mx_overlay"]  =  params[:overlay]
    _get_window_params
    render :update do |page|
      page.replace_html :window, :partial => 'grid_window', :locals => {:codings_in_grid => @mx, :mx_id => @matrix.id, :cell_type => params[:overlay] }
    end and return
  end

  private
  
  def list_params
    @mx_pages, @mxes = paginate :mx, :per_page => 20, :order_by => 'name', :conditions => ['proj_id = (?)', @proj.id]
  end

  # before_filter this
  def _set_otus
    @otus = @mx.otus
    @otus_plus = @mx.otus_plus 
    @otus_minus = @mx.otus_minus
    @otu_groups_in = @mx.otu_groups
    @otu_groups_out = @proj.otu_groups - @otu_groups_in

    @hash_heat = @mx.percent_coded_by_otu
  end

  # before_filter this
  def _set_chrs
    @chrs = @mx.chrs

    @chrs_plus = @mx.chrs_plus 
    # @chrs_plus_out = @proj.chrs - @chrs  

    @chrs_minus = @mx.chrs_minus
    # @chrs_minus_out = @proj.chrs - @chrs_minus

    @chr_groups_in = @mx.chr_groups
    @chr_groups_out = @proj.chr_groups - @chr_groups_in

    @hash_heat = @mx.percent_coded_by_chr
  end

  # before_filter this
  def grid_coding_params
    @chrs = @mx.chrs
    @otus = @mx.otus
 
    session[:interleave_size] = params[:interleave_size] if params[:interleave_size]
    session[:interleave_size] ||= 20
  end

end
