class OntologyClassController < ApplicationController
  in_place_edit_for :ontology_class, :definition
  
  verify :method => :post, :only => [ :destroy, :create, :generate_dbxref ], # , :update
  :redirect_to => { :action => :list }

  before_filter :_show_params, :only => [:show, :show_tags, :show_figures]

  def index
    list
    render :action => 'list'
  end

  def list_params
    @ontology_class_pages, @ontology_classes = paginate :ontology_classes, :per_page => 25, :conditions => "(proj_id = #{@proj.id})"
  end

  def list
    list_params
    if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end

  def new
    @ontology_class = OntologyClass.new
  end

  def create
    @ontology_class = OntologyClass.new(params[:ontology_class])
    if @ontology_class.save
      flash[:notice] = "Ontology class was successfully created."
      redirect_to :action => :show, :id => @ontology_class
    else
      render :action => :new
    end
  end

  def show
    session['ontology_class_view']  = 'show'
    @show = ('show_default')
    @no_right_col = true
    render :action => 'show'
  end

  def show_next_without
    # filter options :without_sensus, :without_definitions, :without_relationships
    @withouts =  @proj.ontology_classes.send(params[:filter]).excluding_id(params[:id]).ordered_by_id
    if @withouts.size == 0
      flash[:notice] = "There are no ontology classes #{params[:filter]}."
      redirect_to :back
    else
      redirect_to :action => :show, :id => @withouts.first.id
    end
  end

  def show_tags
    session['ontology_class_view']  = 'show_tags'
    @tags = @ontology_class.tags.group_by{|o| o.addressable_type}
    @show = ('show_tags')
    @no_right_col = true
    render :action => 'show'
  end

  def show_figures
    session['ontology_class_view']  = 'show_figures'
    @show = ('show_figures') 
    @no_right_col = true
    render :action => :show
  end

  def edit
    @ontology_class = OntologyClass.find(params[:id])
  end

  def update
    @ontology_class = OntologyClass.find(params[:id])
    @ontology_class.update_attributes(params[:ontology_class])
    flash[:notice] = "Updated."
    redirect_to :action => :show, :id => @ontology_class
  end

  def destroy
    @ontology_class = OntologyClass.find(params[:id])
    if @ontology_class.destroy
      redirect_to :action => :list
    else
      flash[:notice] = "Can't destroy this ontology class."
      render :action => :show
    end
  end

  def show_visualize_newick
    id = params[:ontology_class][:id] if params[:ontology_class] ## MUST COME FIRST ##
    id ||= params[:id]
    if id == ""
      flash[:notice] = "Can't find a ontology class with that id, make sure to select the ontology class from the dropdown."
      render :action => :index and return 
    end 
    
    if @ontology_class = OntologyClass.find(id)
      _show_params
      session['ontology_class_view'] = 'show_visualize_newick'
      @show = ('ontology/visualize/show_visualize_newick') # not redundant with above- @show necessary for multiple display of items 
      render :action => :show
    else
      flash[:notice] = "Can't find a ontology class with that id, make sure to select the ontology class from the dropdown."
      render :action => :index
    end

    @no_right_col = true  
    
  end

  def _render_newick
   root = OntologyClass.find(params[:id])
   @object_relationship = ObjectRelationship.find(params[:object_relationship_id]) 
    
   @tree_root = @proj.default_ontology_class #Part.find(:first, :conditions => {:name => 'anatomical surface'}) #  @proj.default_ontology_term
 
   render :update do |page|
      page.replace_html :tree, :partial => 'ontology/visualize/newick_tree', :locals => {
        :root => root,
        :relationship_type => [@object_relationship.id],
        :max_depth => params[:max_depth].to_i,
        :hilight_depth => params[:hilight_depth].to_i,
        :color => params[:color],
        :annotate_value => params[:annotate_value],
        :annotate_index => params[:annotate_index],
        :annotate_branches => params[:annotate_branches],
        :annotate_clades => params[:annotate_clades]}
        # :relationships => @relationships}
    end and return
  end










  def generate_dbxref
    @ontology_class = OntologyClass.find(params[:id])
    begin
      if OntologyClass.generate_xrefs(:ontology_classes => [@ontology_class], :proj_id => @proj.id, :prefix => @proj.ontology_namespace)
        flash[:notice] = "Dbxref generated."
      else
        raise
      end
    rescue
      flash[:notice] = "Problem generating dbxref, does your project have a ontology namespace?  See settings.."
    end
    redirect_to :action => :show, :id => @ontology_class  
  end

  def _show_params
    id = params[:ontology_class][:id] if params[:ontology_class]
    id ||= params[:id]
    @ontology_class = OntologyClass.find(id, :include => [:labels, :sensus, :figures, :tags])
    @class_parents = @ontology_class.logical_relatives(:direction => :parents)
  end

  def auto_complete_for_ontology_class
    @ontology_classes = OntologyClass.auto_complete_search_result(params.merge!(:proj_id => @proj.id))
    render :inline => "<%= auto_complete_result_with_ids(@ontology_classes,
    'format_obj_for_auto_complete', @tag_id_str) %>" 
  end

  def _populate_consituent_parts
    @cop = OntologyClass.find(params[:id]).logical_relatives(:direction => :children)       
    @cop ||= []
    render :update do |page|
      page.replace_html :constituent_parts, :partial => 'logical_ontology_class_list', :locals => {:children_of_part => @cop}
    end and return
  end

  # TODO: dirty updates reverts here?
  def _in_place_definition_update
    t = OntologyClass.find(params[:id])
    t.definition = params[:value]
    if t.save
      render :text => t.definition    
    else
      render :text => '<span style="color: red;">Validation failed, record not updated.</span>'
    end
  end

end
