
# this controller is for methods that span the ontology-related classes, there is no Ontology model

require 'ontology/ontology_methods'
require 'ontology/batch_load/simple'
require 'ontology/batch_load/obo'
require 'ontology/obo2mx'

class OntologyController < ApplicationController

  verify :method => :post, :only => [:create], :redirect_to => { :action => :index }

  def index
  end

  def search
    if redirect = OntologyMethods.search_redirect(params)
      redirect_to redirect
    else
      redirect_to(:action => 'index', :controller => 'ontology') and return
    end
  end

  def proofer
    respond_to do |format|
      format.html {
        @text = nil
        if !params[:ref_id].blank? 
          # need to expand to any kind of incoming text
          if @ref = Ref.find(params[:ref_id])
            if @ref.ocr_text.blank?
              @text = "No text available."  
            else
              @text =  @ref.ocr_text # have to set this vs. partial rendering
            end 
            @l = Linker.new(:proj_id => @proj.id, :incoming_text => @text, :exclude_common_words => true, :adjacent_words_to_fuse => 5, :minimum_word_size => 3)
            @unmatched = @l.unmatched(:proj_id => @proj.id, :minimum_word_size => 3).reject{|w| w =~ /\d\d/} 
          end
        end
        render :action => 'ontology/proofer/index'
      }
      format.js {
        render :update do |page|
        @l = Linker.new(:proj_id => @proj.id, :adjacent_words_to_fuse => 5, :incoming_text => params[:txt].to_s, :exclude_common_words => (params[:exclude_common_words] || false))
        @unmatched = @l.unmatched(:proj_id => @proj.id, :minimum_word_size => 3).reject{|w| w =~ /\d\d/}
        @bioportal_id = params[:bioportal_id] 
        page.replace_html :proofed_text, :partial => 'ontology/proofer/proofed' 
        end and return
      }
    end
    @bioportal_id = params[:bioportal_id]            
  end

  def proofer_batch_create
    begin
      @count = OntologyMethods.proofer_batch_create(params.merge(:proj_id => @proj.id))
    rescue Exception => e
      flash[:notice] = "Something went wrong: #{e}"
      render :action => 'ontology/proofer/index' and return
    end
    redirect_to :action => :list_latest, :controller => :label
  end

 # Simple text-file import
 # def batch_simple
 # end

 #def batch_verify_simple
 #  if @result = Ontology::BatchLoad::Simple.batch_verify(:params => params, :proj => @proj)
 #    flash[:notice] = "Successfully parsed the file."
 #  else
 #    flash[:notice] = "Problem reading/parsing the input file."
 #    redirect_to :action => :batch_simple and return
 #  end      

 #  # TODO - move this to the result
 #  if !params[:tag][:keyword_id].blank? && @tag_keyword = Keyword.find(params[:tag][:keyword_id])
 #    @tag_ref = Ref.find(params[:tag][:ref_id]) if params[:tag] && !params[:tag][:ref_id].blank?
 #    @tag_notes = params[:tag][:notes]
 #    @tag_referenced_object = params[:tag][:referenced_object]
 #  end
 #end

 #def batch_create_simple
 #  if @count = Part.batch_create_simple(:params => params, :proj_id => @proj.id)
 #    flash[:notice] = "Successfully added #{@count} terms."
 #  else
 #    flash[:notice] = "Something went wrong during creation (often this due to a duplicate term), no terms added."
 #  end
 #  redirect_to :action => :list_latest
 #end

  def auto_complete_for_ontology
    if @result = OntologyMethods.auto_complete_search_result(params.merge!(:proj_id => @proj.id))
      render :inline => "<%= auto_complete_result_with_ids_and_class(@result,
        'format_ontology_result_for_auto_complete', params[:tag_id]) %>" 
    else
      redirect_to(:action => 'index', :controller => 'ontology') and return
    end
  end

  # tree based navigation, TODO: move this to a separate controller?
  # it's recursive, and we keep track of a lot of metadata, so there is lots of :locals use
  def tree
    @treetop = @proj.default_ontology_class
    @proj.ontology_classes.first if !@treetop
    redirect_to :action => :new, :controller => :ontology_class and return if !@treetop 
    @colored_object_relationships = @proj.object_relationships.with_color_set
    @all_object_relationships = @proj.object_relationships 
    render :action => 'ontology/tree/index'
  end

  def _tree_set_root
    render :layout => false, :nothing => true and return if !params[:ontology_class]
    @treetop = OntologyClass.find(params[:ontology_class][:id])
    render :layout => false, :partial => 'ontology/tree/tree_index', :locals => { :relationship_type => ((params[:object_relationship] && params[:object_relationship][:id] && !params[:object_relationship][:id].blank?) ? params[:object_relationship][:id]  : 'all' ), :max_depth => (params[:max_depth].to_i || 2)}
  end 

  # open and close a node
  def _tree_navigate_through_child
    @ontology_class = OntologyClass.find_by_id(params[:id])
    
    @object_relationship = ObjectRelationship.find(params[:parent_relationship]) if !params[:parent_relationship].blank?
    @object_relationship == 'root' if !@object_relationship 
   
    # close 1 1;  open 0 1; open false (close); true (open) 
    render :update do |page|
      page.replace "level_#{@ontology_class.id}", :partial => 'ontology/tree/tree_recurser', :locals => {
        :level => (params[:close] ? 1 : 0),
        :max_depth => 1,
        :parent_node => @ontology_class,
        :open => (params[:close] ? false : true),
        :relationship => @object_relationship,
        :relationship_type => params[:relationship_type] }
    end and return        
  end

  def _tree_populate_target
    @ontology_class = OntologyClass.find_by_id(params[:id])
    render :update do |page|
       page.replace_html "ontology_tree_info_target", :partial => 'ontology_class/oc', :object => @ontology_class # :text => 'foo!' #:partial => "filter_small_form_item", :object => @ontology_class 
     #  page.replace_html "existing_relationships", :text => "<i>#{@part.relationships.size} existing relationships<i>" 
     #  page.replace_html "relationship_cart", :partial => 'relationship_cart', :locals => {:part2_id => @part.id, :isa => ((params[:relationship_type] == 'all' || params[:relationship_type].blank?) ?  @proj.isas.first :  Isa.find(params[:relationship_type])) }
     #  page.replace_html "unattached_parts", :partial => '/part/draggable_part', :collection => @proj.parts.without_relationships.ordered_by_updated_on
    end and return       
  end

  # targeting OBO v1.2 
  def show_OBO_file
    @time = Time.now()
    @relationships = @proj.object_relationships.reject{|r| OntologyMethods::OBO_TYPEDEFS.include?(r.interaction)}  
    @xref_keywords = @proj.keywords.that_are_dbxrefs
    
    # a little check
    if @proj.ontology_namespace.blank?
      flash[:notice] = "Project not fully configured to dump OBO files.  Check that ontology namespace is set."
      redirect_to :controller => :proj, :id => @proj.id, :action => :edit and return 
    end

    @terms = @proj.ontology_classes.with_dbxref_namespace(@proj.ontology_namespace).with_obo_label.ordered_by_dbxref # sort{|a,b| a.obo_dbxref <=> b.obo_dbxref}
    render :file => 'ontology/obo/show_OBO_file', :use_full_path => true, :layout => false
  end

end
