class RepositoryController < ApplicationController
    verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def index
    list
    render :action => 'list'
  end

  def list
    @repository_pages, @repositories = paginate :repository, :per_page => 30, :order_by => "coden, name"
     if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end

  def show
    id = params[:id]
    id ||= params[:repository][:id] if params[:repository] # for autocomplete/ajax picker use
    
    if @repository = Repository.find(:first, :conditions => ["id = ?", id])
      return @repository
    else
      flash[:notice] = "Couldn't find a repository with that id."
      redirect_to :action => 'list'
    end
    
  end

  def new
    @repository = Repository.new
  end

  def create
    @repository = Repository.new(params[:repository])

    if @repository.save
      flash[:notice] = 'Repository was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @repository = Repository.find(params[:id])
  end

  def update
    @repository = Repository.find(params[:id])
    if @repository.update_attributes(params[:repository])
      flash[:notice] = 'Repository was successfully updated.'
      redirect_to :action => 'show', :id => @repository
    else
      render :action => 'edit'
    end
  end

  def destroy
    Repository.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
  
  def auto_complete_for_repository
    @tag_id_str = params[:tag_id]
    value = params[@tag_id_str.to_sym]
    
    @reps = Repository.find(:all, :conditions => ["name LIKE ? OR coden LIKE ?", "%#{value}%", "%#{value}%"], :limit => 10,
      :order => 'coden')
    render :inline => "<%= auto_complete_result_with_ids(@reps,
      'format_repository_for_auto_complete', @tag_id_str) %>"
  end
  
end
