class SeqController < ApplicationController
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
  
  layout "layouts/application",  :except => :seqs_as_fasta_file
  layout "layouts/application",  :except => :seqs_as_oneline_file
  
  def index
    list
    render :action => 'list'
  end

  def list  
    @seq_pages, @seqs = paginate :seqs, :per_page => 30, :conditions => ["proj_id = (?)", @proj.id], :order_by => 'otu_id'
    @target = 'list' # hmm- why is this here?
    if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end

  def show
    id = params[:seq][:id] if params[:seq] # for autocomplete/ajax picker use 
    id ||= params[:id]
    session['seq_view']  = 'show'
    @seq = Seq.find(id)
    @chromatograms = @seq.chromatograms
    @show = ('show_default') # not redundant with above- @show necessary for display of multiple of items 
  end

  def new
    @seq = Seq.new
  end

  def new_from_table
    @seq = Seq.new()
    @seq.gene_id = params[:gene_id]
    @seq.otu_id = params[:otu_id]
    @seq.save
    flash[:notice] = "Created new sequence."
    render :action => :edit
  end

  # POST only
  def create  
    @seq = Seq.new(params[:seq])
    if @seq.save
      flash[:notice] = 'Seq was successfully created.'
         redirect_to(:action => :show, :id => @seq)
    else
      flash[:notice] = 'Not saved!'
      render :action => 'new'
    end
  end

  def create_multiple # trys to add seqs for all members of an otu_group (run elsewhere?)
    @p, @f = Seq.create_multiple(params[:multi_seq])
    if @p
      flash[:notice] = 'Added ' + @p.to_s + ", and failed to add (most likely the otu/gene combo already exists) " + @f.to_s + ', sequences.' 
    else 
      flash[:notice] = "None added, possible error with passed parameters."
    end
    redirect_to :action => :list
  end
  
  def edit
    @seq = Seq.find(params[:id])
  end

  def update
    @seq = Seq.find(params[:id])
    if @seq.update_attributes(params[:seq])
      flash[:notice] = 'Seq was successfully updated.'
      redirect_to :action => 'show', :id => @seq
    else
      render :action => :edit
    end
  end

  def destroy
    Seq.find(params[:id]).destroy
    redirect_to :action => :list
  end

  def show_seqs_for_otu
    @seqs = Seq.find_all_by_otu_id_and_proj_id(params[:otu_id], @proj.id)
    @otu = Otu.find(params[:otu_id])
    render :action => :seqs_for_otu
  end
 
  def show_otus_with_seqs
    @otus = @proj.otus.with_seqs_not_through_specimens
    render :action => :otus_with_seqs
  end

  def views
    render :action => :views
  end
  
  def view_query  ## call summarize or some such???
    # check that both a groups have been selected
    if (params[:view][:all_otus].empty? && params[:view][:otu_group_id].empty?) || params[:view][:gene_group_id].empty?  
      flash[:notice] = 'Include both a gene group and a otu group or select All OTUs'
      @target = ''
      render :action => :views and return
    end

    if params[:view][:all_otus] == '1'
      @otus = @proj.otus
    else
      @otugroup = OtuGroup.find(params[:view][:otu_group_id])
      @otus = @otugroup.otus 
    end

    @genegroup = GeneGroup.find(params[:view][:gene_group_id])
    @genes = @genegroup.genes
  
    if @otus.empty? or @genes.empty?
      flash[:notice] = 'Hmm... one (or both) of your groups is empty!'
      @target = ''
      render :action => :views and return
    end
      
    # return only otus with sequences initiated if checked
    if (params[:view][:data_only] == '1')
       @s = "seqs.gene_id = " +  @genes.collect{|o| o.id}.join(' OR seqs.gene_id = ')
       @otus = (@otus & Otu.find_by_sql("SELECT otus.* from otus INNER JOIN seqs ON otus.id = seqs.otu_id WHERE ( (otus.proj_id = #{@proj.id}) AND (#{@s}));")  )
    end
    
    @report_type = params[:report][:type] if params[:report]
    @report_type ||= 'report_grid_summary'
    
    render :action => :views, :otu_group_id => params[:view][:otu_group_id] 
  end

  ## lots of redundancy for file output, should just parameterize
  def seqs_as_fasta_file
    @f = Seq.fasta_file(params)
    if @f
      send_data(@f, :filename => 'mx_matrix.fas', :type => "application/rtf", :disposition => "attachment")
    else
      flash[:notice] = "Couldn't generate the file, make sure you select valid options."
      render :action => :views and return
    end
  end

  def seqs_as_oneline
    @f = Seq.one_line_file(params)
    if @f
      send_data(@f, :filename => 'mx_matrix.txt', :type => "application/rtf", :disposition => "attachment")
    else
      flash[:notice] = "Couldn't generate the file, make sure you select valid options."
      render :action => :views and return
    end
  end

  def seqs_as_nexus
     @f = Seq.nexus_file(params)
    if @f
      send_data(@f, :filename => 'mx_matrix.nex', :type => "application/rtf", :disposition => "attachment")
    else
      flash[:notice] = "Couldn't generate the file, make sure you select valid options."
      render :action => :views and return
    end 
  end

  def seqs_from_FASTA
  end

  def verify_seqs_from_FASTA
    @seqs = Seq.batch_load_FASTA(:file => params[:file])
    if @seqs.nil?
      flash[:notice] = "An error occured. Make sure to select a FASTA formatted file. Check the format of the line endings in your file, they must be unix or dos format, not Mac." 
      render :action => :seqs_from_FASTA and return 
    end 
    flash[:notice] = "Succesfully read file."
  end

  def _batch_add_FASTA 
   if r =  Seq.add_FASTA_after_verify(params)
    flash[:notice] = "Added #{r.size} sequences."
    redirect_to :action => :list, :controller => :seq and return
   else
    flash[:notice] = "Problem adding sequences!"
    redirect_to :action =>  :verify_seqs_from_FASTA and return
   end
  end

  def auto_complete_for_seq
    @tag_id_str = params[:tag_id]
    if @tag_id_str == nil
      redirect_to(:action => 'index', :controller => 'ontology') and return
    else
      @seqs = Seq.find_for_auto_complete(params[@tag_id_str.to_sym], @proj.id)
    end
    render :inline => "<%= auto_complete_result_with_ids(@seqs, 'format_obj_for_auto_complete', @tag_id_str) %>"
  end

  # def blast
    # remote_blast_factory = Bio::Blast.remote('blastn', 'SWISS', '-e 0.0001', 'genomenet')
    
    # report = remote_blast_factory.query(params[:seq])
    # render :update do |page|
    #  page.replace_html :result, debug(report)
    # end and return
  # end

end
