class SpecimenController < ApplicationController
   verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def index
    list
    render :action => :list
  end

  def list_params
    @specimen_pages = Paginator.new self, @proj.specimens.count, 30, params[:page] 
    @specimens = Specimen.find(:all, :conditions => "proj_id = #{@proj.id}", :offset => @specimen_pages.current.offset, :limit => 20) 
    @inc_actions = true # the specimen table is used various places 
  end

  def list 
    list_params
    if request.xml_http_request?
      render(:layout => false, :partial => 'ajax_list')
    end
  end
   
  def search_by_identifier
    if params[:search].blank? or params[:search][:string].blank?
      flash[:notice] = "No search string provided to search by identifier."
      redirect_to :action => 'list' and return
    end
    
    s = params[:search][:string].split
    sql = s.inject([]) {|sum, s| sum << "(specimen_identifiers.identifier LIKE \"%#{s}%\")"}.join(" OR ")
    
    @specimens = Specimen.find(:all, :conditions => "(" + sql + ") and specimens.proj_id = #{@proj.id}", :include => 'specimen_identifiers')
    @target = 'all'
    render :action => 'list'
  end

  def list_all
    @specimens = @proj.specimens
    @target = 'all'
    render :action => 'list'
  end

  def list_by_current_user
    # TODO: by identifier should be a named_scope
    @specimens = @proj.specimens(:include => :specimen_identifiers).created_by(session[:person].id)
    @target = 'all'
    render :action => 'list'
  end
 
 def list_by_creator
    @specimens = @proj.specimens. Specimen.by_identifier(@proj.id)
    @target = 'all'
    render :action => 'list'
  end
 

  def show
    id = params[:specimen][:id] if params[:specimen] # for ajax picker use
    id ||= params[:id]
  
    @specimen = Specimen.find(id)
    @ids = SpecimenIdentifier.find_all_by_specimen_id(params[:id])
    @dets = SpecimenDetermination.find_all_by_specimen_id(params[:id])

    session['specimen_view']  = 'show'
    @show = ('show_default') # not redundant with above- @show necessary for multiple display of item
  end

  def params_for_new_specimen
    @namespaces = Namespace.find(:all).collect {|t| [ t.display_for_select, t.id ] } #.collect{|o| [nil, o.name]}
    if @proj.default_specimen_identifier_namespace
      @specimen_identifier = SpecimenIdentifier.new(:namespace => @proj.default_specimen_identifier_namespace)
    end

    @specimen.repository_id = session[:person].pref_default_repository_id

    @type_assignments = @specimen.type_assignments
  end
  
  def new
    @specimen = Specimen.new
    @target = 'edit'
    params_for_new_specimen
  end

  def create
    @specimen = Specimen.new(params[:specimen])  
   
    begin
      Specimen.transaction do
        @specimen.save!

        if !params[:specimen_identifier][:namespace_id].blank? or ( !params[:specimen_identifier][:identifier].blank?) 
          @specimen_identifier = SpecimenIdentifier.new(params[:specimen_identifier])
          @specimen.specimen_identifiers << @specimen_identifier
        end
   
        unless (params[:specimen_determination][:otu_id].blank?) and (params[:specimen_determination][:name].blank?) 
           @specimen_determination = SpecimenDetermination.new(params[:specimen_determination])
           @specimen.specimen_determinations << @specimen_determination 
        end

        # make this specimen a type specimen
        unless not params[:type_specimen] or (params[:type_specimen][:taxon_name_id].blank? and params[:type_specimen][:type_type].blank?)
          @ts = TypeSpecimen.new(params[:type_specimen])
          @ts.specimen_id = @specimen.id
          @ts.save!
        end  
     end

    rescue  Exception => e # ActiveRecord::RecordInvalid, ActiveRecord::RecordNotSaved
      flash[:notice] = e.message # 'Failed to create specimen.'

      respond_to do |format|
        format.html { 
          params_for_new_specimen
          #  flash[:notice] = "Failed to add specimen (non-AJAX)."
          render(:action => :new) and return

        }
        format.js {
          #   flash[:notice] = "Failed to add specimen (AJAX)." 

          render :update do |page|
           page.replace_html :notice, flash[:notice]
           flash.discard
          end and return        

          #flash[:notice] = "BLEEP BLEEP BLEEP"
        }
      end
      
    end
   
    respond_to do |format|
      format.html { 
        case params[:commit]
        when 'Create'
          flash[:notice] = "Specimen (mxID #{@specimen.id}) was successfully created."
          redirect_to :action => 'show', :id => @specimen and return
        when 'Create and next'
          flash[:notice] = "Specimen (mxID #{@specimen.id}) was successfully created."
          redirect_to :action => :new and return
        when  'Create, clone and next'
          flash[:notice] = "Specimen (mxID #{@specimen.id}) was successfully created, this is the new clone."
          params_for_new_specimen
          render :action => :new and return
        else
          redirect_to :action => :list
        end
      }

      format.js {
        flash[:notice] = "Successfully created specimen with mx id #{@specimen.id}."
        params_for_new_specimen

        render :update do |page|
          page.insert_html :top, :specimens, :partial => 's', :object => @specimen
          if params[:clear_form]
            page[:specimen_form].reset
          end

          page.replace_html :notice, flash[:notice]
          page[:specimen_total].replace_html "<i>incremented post load.</i>"
          flash.discard
        end
      }
    end
  end

  def quick_create

     @namespaces = Namespace.find(:all).collect {|t| [ t.display_for_select, t.id ] }
     @specimen = Specimen.new(params[:specimen]) 
     @ce = Ce.new(params[:ce])
     @identifier = SpecimenIdentifier.new(params[:specimen_identifier])

    if request.post?
      begin 
        Specimen.transaction do 
          @ce.save
          @specimen.ce = @ce
          @specimen.save
          @specimen.specimen_identifiers << @identifier 
          @specimen.save

          flash[:notice] = "Created"
          redirect_to :action => :show, :id => @specimen and return 
        end
      rescue
        flash[:notice] = "Something went wrong."
      end 

    else 
    end
  
  end


  def edit
    @specimen = Specimen.find(params[:id], :include => [:specimen_identifiers, :specimen_determinations, :type_assignments])
    # need to goto taxon name to edit type specimen information
    #  @type_assignments = @specimen.type_assignments
    
    @namespaces = Namespace.find(:all).collect {|t| [ t.display_for_select, t.id ] }
  end

  def update
    @specimen = Specimen.find(params[:id])
  
    begin
      Specimen.transaction do
        @specimen.update_attributes(params[:specimen])
        
      # below redundant with create
       if !params[:specimen_identifier][:namespace_id].blank? || ( !params[:specimen_identifier][:identifier].blank?)     
        @specimen_identifier = SpecimenIdentifier.new(params[:specimen_identifier])
        @specimen.specimen_identifiers << @specimen_identifier
       end

      unless (params[:specimen_determination][:otu_id].blank?) && (params[:specimen_determination][:name].blank?) 
         @specimen_determination = SpecimenDetermination.new(params[:specimen_determination])
         @specimen.specimen_determinations << @specimen_determination 
      end
 
    end
     
    rescue
      flash[:notice] = 'Failed to update the record.'
      redirect_to :back and return # :action  => :edit, :id => @specimen and return
    end
    
      flash[:notice] = 'Updated the record.'
      redirect_to :action => :show, :id => @specimen
  end

  def destroy
    Specimen.find(params[:id]).destroy
    redirect_to :action => 'list'
  end

  def destroy_type_assignment
  end

  ## ! these two need post protection
  def destroy_identifier
      SpecimenIdentifier.find(params[:id]).destroy
      redirect_to :back
  end
  
  
  def destroy_determination
      SpecimenDetermination.find(params[:id]).destroy
      redirect_to :back
  end
  
  def auto_complete_for_specimen
    @tag_id_str = params[:tag_id]
    value = params[@tag_id_str.to_sym]
        
    @specimens = Specimen.find_for_auto_complete(value)
    render :inline => "<%= auto_complete_result_with_ids(@specimens,
      'format_obj_for_auto_complete', @tag_id_str) %>"
  end
 
  # tests/hacking

  def test
    @specimens = @proj.specimens
    @specimen = Specimen.new 
    params_for_new_specimen
  
  end

  def clone
    s = Specimen.find(params[:id])
    @specimen = s.make_clone
    flash[:notice] = "Editing newly cloned specimen."
    redirect_to :action => :edit, :id => @specimen 
  end

  def accordion
    @specimen = Specimen.find(:first, :conditions => "proj_id = #{@proj.id}")
  end 
 
  def batch_load
  end

  def batch_verify_or_create
    if params[:file].blank?
      flash[:notice] = "Provide a file."
      redirect_to :action => :batch_load and return 
    end

    @file =  params[:file].read
       md5 = Digest::MD5.hexdigest(@file)

    begin
      if params[:verify]
        @result = Specimen.read_from_batch(:file => @file, :proj_id => @proj.id)
        session["#{$person_id}_spec_ld_md5_#{md5}"] = md5 # nothing has been thrown, verify this file in the session
        @operation = "verifying"
        render :action => :batch_verify
      elsif params[:create]
        if session["#{$person_id}_spec_ld_md5_#{md5}"] == md5 # then this person verified the this file
          @result = Specimen.read_from_batch(:file => @file, :proj_id => @proj.id, :save => true)
          @operation = "created"
          render :action => :batch_verify
        else
          raise "This file has not yet been verified, please do so first."
        end
      end
    rescue StandardError => e
     flash[:notice] = "Failed to parse: #{e}"
     redirect_to :action => :batch_load and return
    end
  end

end
