# Methods added to this helper will be available to all templates in the application.
module DisplayHelper

  # now that we sanitize data before it is stored in the db
  # this just turns off the h method
  # see also acts_as_santized
  #  ... meh, should remove h() throughout

  # TODO: need to revist, if you want to use this use the alias html_escape 
  def h(t)
    t
  end

  # same as textilize, except sanitizes text first
  def htmlize(text)
    text.blank? ? "" : RedCloth.new(sanitize(text)).to_html 
  end

  def hilight_string_in_text(options = {})
    opts = {:string => '', :text => '', :class => 'passed', :word_bounds => true}.merge!(options)
    return '' if opts[:string].size == 0 || opts[:text].size == 0
    b = (opts[:word_bounds] ? '\b' : '')
    opts[:text].gsub(/#{b}#{opts[:string]}#{b}/, "<span class=\"#{opts[:class]}\" style=\"padding:2px;\">#{opts[:string]}</span>")
  end

  # TODO: revist the bold/header style
  def show_header_tag(obj)
    return "no display_name method" if !obj.respond_to?('display_name') 
    content_tag('div', :style => 'display:inline;') do 
      obj.display_name + "&nbsp;" + content_tag('span', "(id: #{obj.id})", :class => 'small_grey')
    end     
  end

  def is_public_tag(obj)
    return "no is_public method" if !obj.respond_to?('display_name') 
    obj.is_public ? content_tag('span', 'yes', :class=> 'passed') : content_tag('span', 'no', :class => 'failed')

  end

  # renders a quick view of an object
  def render_show(obj, show_links = true, side_nav = true) 
    @obj = obj
    @show_links = show_links
    @side_nav = side_nav
    render(:partial => '/shared/show')
  end

  def display_record_ccuu(obj)
    '<b>Created by: </b>' + obj.c_by.to_s + '<br/>' +
    '<b>Updated by: </b>' + obj.m_by.to_s + '<br/>' +
    '<b>Created on: </b>' + obj.c_on.to_s + '<br/>' +
    '<b>Updated on: </b>' + obj.m_on.to_s + '<br/>' 
  end

  # maybe a Table helper?
  # renders a table row of column headers 
  # if you pass a hash you pass 'name' and 'class' and /or 'style' (include ";" in your style!)
  # like {'name' => 'foo', 'style' => 'bar: 0px;', 'class' => 'blorf'}
  def t_col_heads(heads)
        "<tr>" + heads.collect{|c| c.class.to_s == 'Hash' ? 
          ("<th " +
     (c['class'] ? (" class=\"" + c['class'] + "\"") : "") +
     (c['style'] ? (" style=\"" + c['style'] + "\"") : "") + ">" +
     c['name'] + "</th>") :
     ("<th>#{h c}</th>")}.join("") + "</tr>"
  end

  def t_row(params)           # renders a table row, requires 'obj' as a key if inc_actions ==  true
    opt = { 'inc_actions' => true, # include show/edit/destory/tag as options
      'tr_css' => "",               # a css CLASS for the row, if and integer is passed assumes it's a stripe
      'open_tr' => true,            # true - include <tr>  (useful if you want to append or prefix unescaped data to row)
      'no_show' => false,           # don't show the 'show' action link
      'close_tr' => true,           # true - include </tr>
      'cell_data' => []}            # the cell data
    opt.merge!(params) 
    r = ''

    opt["tr_css"].to_s.to_i == opt["tr_css"] && opt["tr_css"] = ( opt["tr_css"] % 2 == 0 ? 'stripe' : "")
    opt['open_tr'] && ( r << "<tr class=\"" + opt["tr_css"]  + "\">")
    r << opt["cell_data"].collect{|c| c.class.to_s == 'Hash' ? ("<td class=\"" + c.values[0] + "\">#{h c.keys[0]}</td>") : "<td>#{h c}</td>" }.join("") ## IS THIS SECURE??
    if opt['inc_actions']
      r << self.t_cell_obj_actions(opt["obj"])
    end  
    opt["close_tr"] and (r << "</tr>")
    r
  end

  def t_cell_obj_actions(o) # return show/edit/destroy tables cells for object o 
    r = ''
    klass = ActiveSupport::Inflector.underscore(o.class.to_s)
    if o.taggable 
      r << "<td class='list_action'>" + render( :partial => "tag/tag_link", :locals => { :tag_obj_id => o.id, :link_text => "Tag",:tag_obj_class => o.class.to_s, :msg => ''}) + "</td>"
    end 
    r << "<td class='list_action'>" + link_to('Show', :action => 'show', :controller => klass ,  :id => o.id, :target =>'show') + "</td>"
    r << "<td class='list_action'>" + link_to('Edit', :action => 'edit', :controller => klass, :id => o.id) + "</td>"
    r << "<td class='list_action'>" + link_to('Destroy', {:action => 'destroy', :controller => klass, :id => o.id}, :method => "post", :confirm => ([Ref, Otu].include?(o.class) ? "WARNING! You are about to delete EVERYTHING in this project that is directly tied to this #{o.class}.  Are you sure you want to do this?" : "Are you sure?")) + "</td>"
  end

  def quick_table(heads, objs, include_actions)
    t = "<table>" + t_col_heads(heads.collect {|h| h.keys.first })
    for o in objs
      t << "<tr>"
      heads.collect{|h| h.values.first}.each do |m| inc_actions
      t << "<td>#{h o.send(m)}</td>"
      end

      if include_actions
        t << t_cell_obj_actions(o)
      end
      t << "</tr>\n"    
    end
    t << "</table>"    
  end

  def spinner_tag(id)
    image_tag('/images/spinner.gif', :alt => 'Loading', :id => id, :style => "display: none; vertical-align:middle;"  ) 
  end

  def expandable_caption(t, id, truncate_length = 100)
    str = ''
    xml = Builder::XmlMarkup.new(:indent=> 2, :target => str)
    # id is any unique id for the div element
    # t is the text

    trunc = truncate_length - 6 # account for truncation marks, and arrows
    if t.size > 0
      if t.size > trunc
        xml.span( 'id' => "cp_#{id}_short", 'style' => 'display: inline; z-index:300;') do 
          # strip all html before displaying the truncated version # stripped = html_escape(t) # just escape all the html here
           
          stripped = htmlize(t).gsub(/\<.?\>/, '' )

          # xml <<  ( htmlize(t).gsub(/\<.?p\>/, '') )
          xml << stripped[0..trunc] + "... "  
        end
       
        xml.span( 'id' =>  "cp_#{id}_more", "style" => "display: inline; display: none; z-index:300;") do 
          xml <<  ( htmlize(t).gsub(/\<.?p\>/, '') )
        end
        
        xml.a("style" => 'display:inline', "onclick" => "Element.toggle('cp_#{id}_more'); Element.toggle('#{id}_minus'); Element.toggle('#{id}_plus'); Element.toggle('cp_#{id}_short')" ) do |a|
          xml.span( 'id' => "#{id}_minus", "style" => "display:none") do 
            xml <<  "&#8624;"
          end
          xml.span( 'id' => "#{id}_plus", "style" => "display:inline") do
            xml << "&#8628;"
          end
        end
      else
        xml << t        
      end
    end

    str 
  end

end
