module OntologyHelper
  
  def bioportal_link_tag(ontology_class = nil,bioportal_id = nil)
    return content_tag(:em, "no dbxref for this class") if ontology_class.nil? || bioportal_id.nil? || ontology_class.dbxref.blank?
    "http://bioportal.bioontology.org/visconcepts/#{bioportal_id.to_s}/?id=#{ontology_class.dbxref.to_s}"
  end

  # pass an instance of OntologyClass
  # renders the def: line
  def ontology_class_definition_tag_for_OBO(ontology_class)
    "def: \"#{ontology_class.definition.gsub(/\n/,'').strip}\" #{xrefs_for_written_by_tag_for_OBO(ontology_class.written_by, @proj)}<br />" 
  end 

  # renders the synonym: lines
  def synonyms_tag_for_OBO(ontology_class)
    syns = ontology_class.obo_label.synonyms_by_ontology_class(ontology_class)
    if syns.size > 0 
      syns.collect{|l| "synonym: \"#{l.name}\" []"}.join("<br />") + '<br />'
    end 
  end

  # render the collective relationship lines 
  def relationship_tag_for_OBO(ontology_class)
    rels = ontology_class.primary_relationships.where_both_ontology_classes_have_dbxrefs
    if rels.size > 0
      rels.collect{|o| display_relationship_tag_for_OBO(o)}.join('<br />')  + "<br />"
    end
  end 

  # render a single relationship: line
  def display_relationship_tag_for_OBO(ontology_relationship)
    if ontology_relationship.object_relationship.interaction == "is_a"
      "is_a: #{ontology_relationship.ontology_class2.dbxref} ! #{ontology_relationship.ontology_class2.obo_label.display_name}" 
    else
      "relationship: #{ontology_relationship.object_relationship.interaction} #{ontology_relationship.ontology_class2.dbxref} ! #{ontology_relationship.ontology_class2.obo_label.display_name}" 
    end
  end





  # renders the obsolete: line if needed 
  def obsolete_tag_for_OBO(ontology_class)
    'is_obsolete: true<br />' if ontology_class.is_obsolete
  end

  # renders the xref: lines taken from Tags
  def xrefs_for_ontology_class_tag_for_OBO(ontology_class)
    xrefs = ontology_class.dbxrefs_from_tags
    if xrefs.size > 0
      xrefs.collect{|x| "xref: #{x}"}.join("<br />") + '<br />'
    end
  end

  # renders the dbxrefs for a reference, taking into account the Ontology namespace
  # may expand the namespace vs. a URI for a given installation of mx ultimately
  # TODO: remove if check on ref when data updated
  def xrefs_for_written_by_tag_for_OBO(ref = nil, proj = nil)
    return "[ERROR:ONTOLOGY_CLASS_WITHOUT_WRITTEN_BY]" if ref.nil?
    xrefs = ["#{proj.ontology_namespace}:ref #{ref.display_name(:type => :dbxref)}"] 
    xrefs += ref.alternate_dbxrefs   
    return "[" + xrefs.join(", ") + "]"  # there has to be at least one
  end

  # renders the collective [Typdef] tags 
  def typedefs_tag_for_OBO(proj)
    typedefs = proj.object_relationships.not_isa
    if typedefs.size > 0
    typedefs.collect{|i| typedef_tag_for_OBO(i)}.join("<br />") 
    end
  end

  # renders a single [Typedef] tag
  def typedef_tag_for_OBO(object_relationship)
   s =  '[Typedef]<br />' 
   s += "id: #{object_relationship.interaction}<br />"
   s += "name: #{object_relationship.interaction}<br />"
   s += "is_transitive: true<br />" if object_relationship.is_transitive
   s += "is_reflexive: true<br />" if object_relationship.is_reflexive
   s += "is_anti_symmetric: true<br />" if object_relationship.is_anti_symmetric
   s += "xref: #{object_relationship.xref}<br />" if !object_relationship.xref.blank?
  end

end
