# == Schema Information
# Schema version: 20090930163041
#
# Table name: ces
#
#  id                :integer(4)      not null, primary key
#  proj_id           :integer(4)      not null
#  namespace_id      :integer(4)
#  external_id       :integer(4)
#  verbatim_label    :text
#  num_to_print      :integer(4)
#  collectors        :text
#  locality          :text
#  geography         :text
#  geog_id           :integer(4)
#  mthd              :string(255)
#  sd_d              :string(2)
#  sd_m              :string(4)
#  sd_y              :string(4)
#  ed_d              :string(2)
#  ed_m              :string(4)
#  ed_y              :string(4)
#  latitude          :float
#  longitude         :float
#  lat_lon_error_m   :integer(4)
#  elev_min          :float
#  elev_max          :float
#  elev_unit         :string(6)
#  label_lat         :string(48)
#  label_lon         :string(48)
#  undet_ll          :boolean(1)      not null
#  notes             :text
#  trip_code         :string(255)
#  trip_namespace_id :integer(4)
#  doc_label         :text
#  verbatim_method   :string(255)
#  host_genus        :string(255)
#  host_species      :string(255)
#  err_label         :boolean(1)      not null
#  err_entry         :boolean(1)      not null
#  err_checked       :boolean(1)      not null
#  undetgeog         :boolean(1)      not null
#  updated_on        :timestamp       not null
#  created_on        :timestamp       not null
#  creator_id        :integer(4)      not null
#  updator_id        :integer(4)      not null
#  print_label       :text
#  depth_min         :float
#  depth_max         :float
#

class Ce < ActiveRecord::Base
  include ActionView::Helpers::TextHelper 
  
  include ModelExtensions::Taggable
  include ModelExtensions::Figurable
  include ModelExtensions::DefaultNamedScopes
  include ModelExtensions::MiscMethods
 
  has_standard_fields
  
  belongs_to :geog
  belongs_to :namespace
  belongs_to :trip_namespace, :class_name => 'Namespace', :foreign_key => 'trip_namespace_id'
  
  has_many :public_tags, :as => :addressable, :class_name => "Tag", :include => [:keyword, :ref], :order => 'refs.cached_display_name ASC', :conditions => 'keywords.is_public = true'
  has_many :lots, :dependent => :nullify
  has_many :specimens, :dependent => :nullify
   
  validates_numericality_of :latitude, :longitude, :allow_nil => true

  named_scope :to_print, {:conditions => "ces.num_to_print > 0"}

  def display_name(options = {})
     @opt = {
      :type => :line # :list, :head, :select, :sub_select
     }.merge!(options.symbolize_keys)
     s = ''

     case @opt[:type]
     when :material_examined_verbose
      s << [self.locality, self.verbatim_method, self.date_range, self.collectors].compact.join(", ")
     else
       tmp_str = [geography, locality, lat_long, elevation, date_range, collectors, mthd, "#{trp_namespace} #{trip_code}"].reject(&:blank?).collect{|t| truncate(t, :length => 45, :omission =>  '...')}.join(", ") 
       s << "<div class=\"dn_#{@opt[:type].to_s}\">" 
       s << "<div class=\"dnsid\">id: #{id}</div>"
       # the label content
          s << '<span class="hd">ce: </span>' if @opt[:type] == :sub_select
          # case views for :list, :head, etc. here 
          # case @opt[:type]
          # when :list 
          s << '<div class="small_grey">'
          if tmp_str.blank? && verbatim_label.blank?
           s << "<span style=\"color: red;\">stub only</span>"
          elsif !verbatim_label.blank? && tmp_str.blank? 
            s << verbatim_label
          else
           s << "#{tmp_str}"
          end
          s << "</div>"
          # end 

        s << "</div>"
     end
    return nil if s == ""
    s 
  end
  
  #TODO: deprectate 
  def display_summary
    # attempts to show the generated label first, then the verbatim
    s = [geography, locality, lat_long, elevation, date_range, collectors, mthd, "#{trp_namespace} #{trip_code}", "mx id:#{id}"].reject(&:blank?).join("<br/>")
    return ('<span style="color: red;">' + verbatim_label + '</span>') if s.blank?
    s
  end

  #TODO: deprecate
  def display_for_select ## super ugly
    #s = [geography, locality, lat_long, elevation, date_range, collectors, mthd, "#{trp_namespace} #{trip_code}", "mx id:#{id}"].reject(&:blank?).collect{|t| truncate(t. 20, '...')}.join("<br/>")
   # return ('<span style="color: red;">' + label + '</span>') if s.blank?
   # s
    s = ''
    s << truncate(geography, :length => 15)  if (geography != nil )
    s << " " + truncate(locality, :length => 15) if (locality != nil)
    s << " " + date_range
    s << " " + truncate(collectors, :length => 15) if (collectors != nil ) 
    s << truncate(verbatim_label, :length => 45)  if ((s == '') && (verbatim_label != nil)) # if there is nothing in the label use verbatim label field
    s <<  " [id:" + id.to_s + "]"
  end

  #TODO: deprecate
  # hmm- not used likely
  def display_for_list
    s = [geography, locality, lat_long, elevation, date_range, collectors, mthd, "#{trp_namespace} #{trip_code}", "mx id:#{id}"].reject(&:blank?).collect{|t| truncate(t, :length => 20, :omission =>  '...')}.join(", ")
    return ('<span style="color: red;">' + verbatim_label + '</span>') if s.blank? && verbatim_label.blank?
    return verbatim_label if !verbatim_label.blank? && s.blank? 
    s 
  end

  def trp_namespace
    self.trip_namespace.display_name if self.trip_namespace
  end

  def start_date
    [sd_d, sd_m, sd_y].reject(&:blank?).join(".")
  end

  def end_date
    [ed_d, ed_m, ed_y].reject(&:blank?).join(".")
  end

  def date_range
    the_start = start_date
    the_end = end_date
    [the_start, the_end].reject(&:blank?).join("-")
  end
  
  def lat_long
    [latitude, longitude].reject(&:blank?).join(",")
  end
  
  # formats elevation values for display
  def elevation
    range = [elev_min, elev_max].reject(&:blank?).join("-")
    range.blank? ? "" : range + " #{elev_unit}"
  end
  
  def self.find_for_auto_complete(value)
    conditions = value.split.collect { |t| 
      sanitize_sql(["((c.verbatim_label like ?) OR (c.geography like ?) OR (c.locality like ?) OR (c.trip_code like ?) OR (g.name like ?) OR (c.id = ?) OR c.collectors LIKE ?)",
        "%#{t}%", "%#{t}%", "%#{t}%", "%#{t}%", "#{t}%", "#{t}", "%#{t}%"])
    }.join " AND "    
        
    find_by_sql("SELECT c.* FROM ces AS c LEFT JOIN geogs g ON c.geog_id = g.id WHERE c.proj_id = #{$proj_id} AND #{conditions} LIMIT 30") 
  end
 
  def mappable
    return true if !latitude.blank? && !longitude.blank?
    false
  end
  
  def gmap_hash
    if self.mappable
      # name should be description, to match up 
      
      {:latitude => self.latitude, :longitude => self.longitude, :name => self.display_summary.gsub(/[\r\n|\n|\r]/, '<br/>')} # cross platform newlines STILL SUCK, even in Ruby AFAIKT
    else
      false
    end
  end

  # puts the hash in an array, so we can use it as a single marker
  def gmap_array
    [gmap_hash]
  end


  def geog_array
    return [nil, nil, nil, nil] if self.geog.blank?
    [self.geog.country_string, self.geog.state_string, self.geog.county_string, self.locality]
  end



  protected
    def validate
      if latitude
        errors.add("latitude", "must be within -90 to 90") unless (-90 <= latitude.to_f and latitude.to_f <= 90)
      end
      if longitude
        errors.add("longitude", "must be within -180 to 180") unless (-180 <= longitude.to_f and longitude.to_f <= 180)
      end
    end
  
    # batch uploading

    def path_to_file
    end
    
end

