# == Schema Information
# Schema version: 20090930163041
#
# Table name: chromatograms
#
#  id                 :integer(4)      not null, primary key
#  pcr_id             :integer(4)
#  primer_id          :integer(4)
#  protocol_id        :integer(4)
#  done_by            :string(255)
#  chromatograph_file :string(255)
#  result             :string(24)
#  seq                :text
#  notes              :text
#  proj_id            :integer(4)      not null
#  creator_id         :integer(4)      not null
#  updator_id         :integer(4)      not null
#  updated_on         :timestamp       not null
#  created_on         :timestamp       not null
#
class Chromatogram < ActiveRecord::Base
  has_standard_fields
  
  belongs_to :pcr  
  belongs_to :primer
  belongs_to :protocol

  has_many :seqs, :through => :pcr

  validates_presence_of :primer, :pcr
  
  def before_destroy
     delete_file
  end
  
  def display_name(options = {})
    "foo"
  end

  def display_for_select
    primer.display_name + " " + pcr.display_name
  end

  # file uploads (heavily) modified from code at
  # http://www.albert.bagasie.com/RailsTips/FileUpload taken on July 28 / 05

  def file=(fn) # this is an accesor, the magic is that you need to have the accesor name (in this case 'file') listed as an attr_accessible!!! wow did that take some time to figure out 
 
   if fn.size > 0
    @file = fn   

    original_filename = fn.original_filename
    logger.info("Original filename is '#{original_filename}'")
      
    @clean_filename = base_part_of(original_filename) 

    self.chromatograph_file = @clean_filename
    
    logger.info("Sanitized filename is '#{self.chromatograph_file}'")
    # self.content_type = file_field.content_type.strip
    end
  end

  def write_file(id)
    filename = CHROMATOGRAPH_FILE_PATH + id.to_s + "_" + @clean_filename
      logger.info("Uploading file to '#{filename}'")
    @file.rewind
     File.open(filename, "wb") do |f|
         f.write(@file.read)
         end  
    true
  end
  
  def delete_file
    File.delete(chromatograph_path) if chromatograph_file
  end

  def detach_chromatograph_file
    delete_file
    write_attribute(chromatograph_file, nil)
  end

  def save
    super
    write_file(self.id) if @file
    true
  end

  def chromatograph_path
    CHROMATOGRAPH_FILE_PATH + self.chromatograph_filename
  end
  
  def chromatograph_filename
    self.id.to_s + "_" + self.chromatograph_file
  end
  
  def chromatograph_download_path
    "/files/chromatographs/" + chromatograph_filename
  end
  
  private

  def base_part_of(filename)
    filename = File.basename(filename.strip)
    # remove leading period, whitespace and \ / : * ? " ' < > |
    filename = filename.gsub(%r{^\.|[\s/\\\*\:\?'"<>\|]}, '_')
  end

end
