# == Schema Information
# Schema version: 20090930163041
#
# Table name: content_templates
#
#  id         :integer(4)      not null, primary key
#  name       :string(255)     not null
#  is_default :boolean(1)      not null
#  is_public  :boolean(1)      not null
#  proj_id    :integer(4)      not null
#  creator_id :integer(4)      not null
#  updator_id :integer(4)      not null
#  updated_on :timestamp       not null
#  created_on :timestamp       not null
#

class ContentTemplate < ActiveRecord::Base
  has_standard_fields

  # has_and_belongs_to_many :content_types
  has_many :content_types, :through => :content_templates_content_types, :order => 'content_templates_content_types.position'
  has_many :content_templates_content_types, :order => 'position', :dependent => :destroy
  has_many :public_content_types, :through => :content_templates_content_types, :source => 'content_type', :conditions => 'content_types.is_public = true', :order => 'position'
  has_many :mx_content_types, :through => :content_templates_content_types, :source => 'content_type', :conditions => 'content_types.sti_type != "TextContent"', :order => 'position'
  has_many :text_content_types, :through => :content_templates_content_types, :source => 'content_type', :conditions => 'content_types.sti_type = "TextContent"', :order => 'position'

  validates_uniqueness_of :name, :scope => 'proj_id'
  validates_presence_of :name

  # DEPRECATED
  # return the text contents for an Otu in an array
  def contents(otu)
    ts = self.text_content_types 
    otu.contents.inject([]) do |memo, o|
      ts.include?(o.content_type) ? memo << o : memo 
    end
  end

  # returns a hash with the ContentType pointing to the Content
  def text_content(otu, is_public = false)
    ts = self.text_content_types 
    # rewrite to a hash 
    otu.contents.inject({}) do |memo, o|
      ts.include?(o.content_type) && o.is_public ? memo.update({o.content_type.id => o}) : memo 
    end
  end

  def display_name(options = {})
    name
  end

  def publish(otu)
    return false if not otu
    p = self.public_content_types.collect{|o| o.id}

    for c in self.contents(otu)
      c.publish if p.include?(c.content_type_id) 
    end

    true
  end

  # returns ContentTypes
  def available_text_content_types # available to add
    ContentType.find(:all, :conditions => {:proj_id => self.proj_id, :sti_type => 'TextContent'}) -  self.content_types 
  end

  # returns Array of Strings
  def available_mx_content_types2 # not presently included in the template
    (ContentType.custom_types - self.mx_content_types.collect{|s| s.sti_type})  # .collect{|c| c.gsub(/ContentType::/, "")}
  end

  # returns an Array of ContentType subclasses
  def available_mx_content_types # not presently included in the template
    (ContentType.custom_types - self.mx_content_types.collect{|s| s.sti_type}).collect{|ct| "ContentType::#{ct}".constantize}  # .collect{|c| c.gsub(/ContentType::/, "")}
  end

  def add_or_remove_content_type(params)
    if params[:out]
      if ct_out = ContentTemplatesContentType.find(:first, :conditions => ["content_template_id = ? AND content_type_id = ?",  self.id, params[:content_type_id]])
        self.content_templates_content_types.delete(ct_out)
      else
        return false
      end
    else
      begin
        c = ContentTemplatesContentType.new(:content_template => self)
        if params[:content_type_id]
          ctype = ContentType.find(params[:content_type_id])
        elsif params[:mx_content_type]
          ctype = ContentType.create_if_needed(params[:mx_content_type], self.proj_id)
        else
          raise
        end
        c.content_type_id = ctype.id
        c.save!
      rescue Exception => e
        return false
      end
    end
    true
  end

  def transfer_to_otu(otu_in, otu_out, delete_from_incoming = true)
    self.text_content_types.each do |ct|
      if c = Content.find(:first, :conditions => {:otu_id => otu_in.id, :content_type_id => ct.id})
        c.transfer_to_otu(otu_out, false)
      end
    end
  end

  def taxon_treatment(options = {})
    opt = { :target => ''}.merge!(options)

    doc = Builder::XmlMarkup.new(:indent => 2, :target => opt[:target])
    doc.instruct!(:xml, :version => "1.0", :encoding => "ISO-8859-1")

    doc.body do |body|
      self.content_types.each do |ct|
        ct.tp_xml(:target => opt[:target])
      end
    end
    return opt[:target]
  end

end
