# == Schema Information
# Schema version: 20090930163041
#
# Table name: distributions
#
#  id            :integer(4)      not null, primary key
#  geog_id       :integer(4)
#  otu_id        :integer(4)
#  ref_id        :integer(4)
#  confidence_id :integer(4)
#  verbatim_geog :string(255)
#  introduced    :boolean(1)
#  num_specimens :integer(4)      not null
#  notes         :text
#  proj_id       :integer(4)      not null
#  creator_id    :integer(4)      not null
#  updator_id    :integer(4)      not null
#  updated_on    :timestamp       not null
#  created_on    :timestamp       not null
#


# Records extracted from the literature, these data are here not tied to specimens or collecting events
class Distribution < ActiveRecord::Base

  include ModelExtensions::DefaultNamedScopes
  # introduced -> 0:native; 1:introduced; null:unknown

  has_standard_fields
  belongs_to :otu
  belongs_to :ref
  belongs_to :confidence
  belongs_to :geog

  validates_presence_of :otu, :ref, :geog
 
  named_scope :native, :include => :geog, :order => 'geogs.name, distributions.verbatim_geog', :conditions => 'introduced = 0'
  named_scope :introduced, :include => :geog, :order => 'geogs.name, distributions.verbatim_geog', :conditions => 'introduced = 1'
  named_scope :unclassified, :include => :geog, :order => 'geogs.name, distributions.verbatim_geog', :conditions => 'introduced = null'
  named_scope :ordered_by_geog_name, :include => :geog, :order => 'geogs.name, distributions.verbatim_geog'
  named_scope :using_geogs, :conditions => 'geog_id is not null'

  def display_name(options = {})
    otu.display_name + " / " + ref.authors_year + " / " + geog.display_name
  end

  def display_introduced
    case introduce
    when 0
      "native"
    when 1
      "introduced"
    else
      "unknown"
    end
  end

end
