# == Schema Information
# Schema version: 20090930163041
#
# Table name: figures
#
#  id                      :integer(4)      not null, primary key
#  addressable_id          :integer(4)
#  addressable_type        :string(64)
#  image_id                :integer(4)      not null
#  position                :integer(1)
#  caption                 :text
#  updated_on              :timestamp       not null
#  created_on              :timestamp       not null
#  creator_id              :integer(4)      not null
#  updator_id              :integer(4)      not null
#  proj_id                 :integer(4)      not null
#  morphbank_annotation_id :integer(4)    TODO: DEPRECATED
#  svg_txt                 :text
#

class Figure < ActiveRecord::Base
  has_standard_fields 

  include ModelExtensions::DefaultNamedScopes

  belongs_to :addressable, :polymorphic => true
  belongs_to :image

  acts_as_list :scope => 'addressable_id = #{addressable_id} AND addressable_type = \"#{addressable_type}\"'

  has_many :figure_markers, :dependent => :destroy

  # polymorphic associations
  validates_presence_of :image_id
  validates_presence_of :addressable_id
  validates_presence_of :addressable_type

  after_update :save_figure_markers

  # railscasts thnx
  # Not presently implemented in the app
  def figure_marker_attributes=(figure_marker_attributes)
    figure_marker_attributes.each do |attributes|
      next if attributes == {} 
      if attributes[:id].blank?
        figure_markers.build(attributes)
      else
        a = figure_markers.detect { |t| t.id == attributes[:id].to_i } 
        a.attributes = attributes
      end
    end
    true
  end
 
  def save_figure_markers
    figure_markers.each do |a|
      a.save(true) # passing false ignores validation -- ugh!
    end
  end

  def display_name(options = {})
       opt = {:type => :inline 
    }.merge!(options.symbolize_keys)
    case opt[:type]
    when :on
      figured_obj ? 
      figured_obj.display_name(:type => :figure) :
      "<strong style='color:red;'> ORPHANED #{addressable_type}: #{addressable_id} </strong>"
    else

    end
  end

  # pass :obj => ObjectToFigure along with params for Figure.new
  def self.create_new(options = {})
    opt = {
      :addressable_id => options[:obj].id,
      :addressable_type => options[:obj].class.to_s
    }.merge!(options).to_options!
    opt.delete(:obj)
    t = Figure.new(opt)
    t.save 
    t 
  end  

  # returns the object that the figure is attached to
  def figured_obj
    begin
      ActiveRecord::const_get(self.addressable_type).find(self.addressable_id) 
    rescue
      return false 
    end
  end

  # return the number of characters for a pixel width of w, font_size of 8(units?)
  def text_width_chrs(font_size = 5.5)
    self.image.width_for_size(:thumb).to_f / font_size 
  end

  def self.create_all_for_content_by_otu(content_id, otu_id)
    ## TODO should be abstracted to allow attachment to any object type
    @o = Otu.find(otu_id) or return false
    @c = Content.find(content_id) or return false
    begin
      Figure.transaction do
        @o.images.each do |i|
          if !Figure.find(:first, :conditions => {:addressable_id => @c.id, :addressable_type => @c.class.to_s, :image_id => i.id})
            f = Figure.new(:addressable_id => @c.id, :addressable_type => @c.class.to_s, :image_id => i.id)
            f.save!
          end
        end
      end
    rescue
      return false
    end
    true
  end

  def svgObjRoot_params(size = :medium)
    x = self.image.width_for_size(size)
    y = self.image.height_for_size(size) 
    ["figure_#{self.id}_img", "fig_svg_root_#{self.id}", self.svg(:width => x, :height => y, :size => size), x, y]
  end

  # renders an SVG representation of the figure
  def svg(options = {})
    opt = {
      :target => "", 
      :scale => nil,
      :size => :medium                   # :thumb, :medium, :big, :original
    }.merge!(options.symbolize_keys)
  
     xml = Builder::XmlMarkup.new(:indent=> 2, :target => opt[:target])

     xml.svg(:id => "fig_svg_#{self.id}",
             :width => self.image.width_for_size(opt[:size]),
             :height => self.image.height_for_size(opt[:size]),
             :display => 'inline', # was block
             :xmlns => 'http://www.w3.org/2000/svg',
             'xmlns:xlink' => "http://www.w3.org/1999/xlink") {

       xml.image( :x => 0,  
                  :y => 0,
                 'width' => self.image.width_for_size(opt[:size]),
                 'height' => self.image.height_for_size(opt[:size]),  
                 'id' => 'someid',
                 'xlink:href' => self.image.path_for(:size => opt[:size])
        )

        xml.g(:id => "markers_for_fig_#{self.id}", :transform => "scale(#{self.image.width_scale_for_size(opt[:size])})", 'stroke-width' => self.image.stroke_width_for_size(opt[:size]) ) {  # to 6 decimal places     
          figure_markers.ordered_by_position.each do |fm|
            xml << fm.render 
          end
        }

     }
   
    opt[:target] 
  end


end
