class FigureMarker < ActiveRecord::Base
  has_standard_fields

  include ModelExtensions::DefaultNamedScopes

  belongs_to :figure
  acts_as_list :scope => :figure # first is topmost

  validates_presence_of [:svg, :figure]

  ALLOWED_BASE_ATTRIBUTES = []
  FIGURE_MARKER_TYPES = [:point, :line, :area, :volume]
  DEFAULT_OPACITY = 0.4

  # return a valid svg element including stroke and fill
  def render(options = {})
    opt = {
      :id => "marker_#{self.id}",
      :stroke => '#14eddd',
      :fill => '#14eddd',
      :opacity => DEFAULT_OPACITY, 
      :display => 'inline',
    }.merge!(options)

    target = ''

    xml = Builder::XmlMarkup.new(:indent=> 2, :target => target)
    xml.g(opt) {
      xml << self.svg
    }
    
    target 
  end

  before_save :strip_unused_attributes_from_svg

  protected

  def strip_unused_attributes_from_svg
    begin
      # do some cleanup on the incoming string
      txt = self.svg
      txt.strip!
      txt.gsub!(/\n/, '') # compress whitespace
      txt.gsub!(/\n+\s*/, "\s") # compress whitespace
      txt.gsub!(/\t/, '')       # strip tabs

      self.logger.info "incoming figure marker text: #{txt}" + txt

      s = REXML::Document.new(txt)

      return false if s.elements.size == 0  # FIX THIS
      REXML::XPath.match(s, "//").each do |e|
        # strip the unecessary Text nodes that might have been picked up
        if e.class == REXML::Text
          e.parent.delete(e) 
          next
        end 
        e.attributes.keys.each do |k|
          # we likely don't need this, it's cleaned in sanitize!
          if !ALLOWED_SVG_ATTRIBUTES.include?(k) # see environment.rb
            e.delete_attribute(k) 
            next
          end
          # and this is likely necessary either
          e.attribute(k).value.gsub!(/\n+\s*/, "\s") # compress somewhitespace
        end
      end

    rescue REXML::ParseException => e
      return false
    end

    self.logger.info "parsed: #{s}"
    self.logger.info "parsed.to_s: #{s.to_s}"

    # have to loop the elements here and check each one individually
    #  return false if !s.elements.first.has_attributes?
    self.svg = s.to_s
  end

end
