# == Schema Information
# Schema version: 20090930163041
#
# Table name: geogs
#
#  id                         :integer(4)      not null, primary key
#  name                       :string(255)     not null
#  abbreviation               :string(64)
#  fips_code                  :integer(4)
#  sort_NS                    :integer(4)
#  sort_WE                    :integer(4)
#  center_lat                 :string(64)
#  center_long                :string(64)
#  geog_type_id               :integer(4)
#  inclusive_biogeo_region_id :integer(4)
#  country_id                 :integer(4)
#  state_id                   :integer(4)
#  county_id                  :integer(4)
#  continent_ocean_id         :integer(4)
#  namespace_id               :integer(4)
#  external_id                :integer(4)
#  creator_id                 :integer(4)      not null
#  updator_id                 :integer(4)      not null
#  updated_on                 :timestamp       not null
#  created_on                 :timestamp       not null
#

class Geog < ActiveRecord::Base
  has_standard_fields
  belongs_to :geog_type
  belongs_to :country,  :class_name => "Geog", :foreign_key => "country_id"  
  belongs_to :state, :class_name => "Geog", :foreign_key => "state_id"  
  belongs_to :county, :class_name => "Geog",:foreign_key => "county_id"  
  belongs_to :continent_ocean, :class_name => "Geog", :foreign_key => "continent_ocean_id"  
  belongs_to :biogeo_region, :class_name => "Geog",:foreign_key => "inclusive_biogeo_region_id"    

  has_many :ces
  has_many :tags, :as => :addressable, :dependent => :destroy, :include => [:keyword, :ref], :order => 'refs.cached_display_name ASC'  
  has_many :distributions
  has_many :otus, :through => :distributions

  validates_presence_of :geog_type_id
  validates_presence_of :name 


  def display_name(options = {})
    type_name = (self.geog_type ? self.geog_type.name : 'none')
    n = ''
    n << "#{self.country.name}: " if (self.country and type_name != 'country')
    n << "#{self.state.name}: " if (self.state_id and self.id != self.state_id) # may have a 'state', but type_name may be 'province'
    n << (type_name == 'county' ? self.name + " Co." : self.name)
  end

  # TODO: deprecate
  def display_for_select
    n = name
    n << " [#{self.geog_type.name}]" if self.geog_type
    n << " #{self.country.name}" if (self.country and not self.geog_type.name == 'country')
    n
  end

  # TODO: deprecate
  def display_for_list
     display_name[0..25]
  end

  def country_string
    self.country ? self.country.name : 'not provided'
  end

  def state_string
    self.state ? self.state.name : 'not provided'
  end

  def county_string
    self.county ? self.county.name : 'not provided'
  end

  # sorting f(n)
  def country?
     country ? country.name : "AAAA#{name}" 
  end

end
