# == Schema Information
# Schema version: 20090930163041
#
# Table name: image_descriptions
#
#  id            :integer(4)      not null, primary key
#  otu_id        :integer(4)      not null
#  proj_id       :integer(4)      not null
#  image_id      :integer(4)   # ALLOW NU
#  image_view_id :integer(4)
#  part_id       :integer(4) # DEPRECATED FOR LABEL
#  stage         :string(32)
#  sex           :string(32)
#  specimen_id   :integer(4)
#  svg_txt       :text
#  notes         :text
#  is_public     :boolean(1)      not null
#  priority      :string(6)
#  requestor_id  :integer(4)
#  contractor_id :integer(4)
#  request_notes :text
#  status        :string(64)
#  updated_on    :timestamp       not null
#  created_on    :timestamp       not null
#  creator_id    :integer(4)      not null
#  updator_id    :integer(4)      not null
#  magnification :string(255)
#

class ImageDescription < ActiveRecord::Base
  has_standard_fields
  
  include ModelExtensions::DefaultNamedScopes
  
  belongs_to :image  # We don't require a image because ImageDescripion is mocked to be used for requests/to do lists as well
  belongs_to :image_view
  belongs_to :otu
  belongs_to :label 
  belongs_to :specimen

  validates_presence_of :image

  def validate
    errors.add(:otu_id, "Either an OTU or a specimen is required for this image description.") if otu.blank? && specimen.blank?
  end
 
  named_scope :public, :conditions => 'is_public is true'

  def display_name(options = {})
      s = image.id.to_s + " " + otu.display_name
      s << " / " + label.display_name if label
      s << " / " + image_view.display_name if image_view and label
      s << '<img src="' + image.path_for(:size => :thumb) + '"  width=40 />'
  end

  # TODO: deprecate
  def display_for_ajax_dropdown
      s =  '<div style="border: 1px solid silver; padding: 1px; margin: 1px; width: 100%; font-size: smaller;">' 
      s << '<img src="' + self.image.path_for(:size => :thumb) + '" width=80 align=center style="padding-left: 2px;" />'
      s <<  "&nbsp; #{self.image_id.to_s}<br/>#{self.otu.display_name}"
      s << " / " + self.label.display_name if self.label
      s << " / " + self.image_view.display_name if self.image_view && self.label
      s << '</div>'
  end
 
  # TODO: deprecate 
  def label_view
      s = ''
      s << label.display_name if label
      s << " / " + image_view.display_name if image_view
      s
  end
  
  def self.random_set(proj_id, num = 1, public = true)
    id = ImageDescription.find(:all, :conditions => ["proj_id = ? and is_public = ?", proj_id, public ? 1 : 0])
    ids = []
    id.size < num and num = id.size
    for i in (1..num)
      ids << id[rand(id.size)]
    end
    
    ids
  end    
end
