# == Schema Information
# Schema version: 20090930163041
#
# Table name: lots
#
#  id              :integer(4)      not null, primary key
#  otu_id          :integer(4)      not null
#  key_specimens   :integer(4)      default(0), not null
#  value_specimens :integer(4)      default(0), not null
#  ce_id           :integer(4)
#  ce_labels       :text
#  rarity          :string(16)
#  source_quality  :string(16)
#  notes           :text
#  repository_id   :integer(4)
#  dna_usable      :boolean(1)      default(TRUE)
#  mixed_lot       :boolean(1)
#  sex             :string(64)
#  proj_id         :integer(4)      not null
#  creator_id      :integer(4)      not null
#  updator_id      :integer(4)      not null
#  updated_on      :timestamp       not null
#  created_on      :timestamp       not null
#  stage           :string(255)
#

class Lot < ActiveRecord::Base
  has_standard_fields

  include ModelExtensions::Taggable
  include ModelExtensions::DefaultNamedScopes
  # should be Figurable? 
  include ModelExtensions::MiscMethods

  has_many :extracts, :dependent => :destroy
  has_many :lot_identifiers, :dependent => :delete_all
  
  belongs_to :otu
  belongs_to :repository
  belongs_to :ce 
  has_and_belongs_to_many :lot_groups

  named_scope :with_usable_dna, :conditions => {:dna_usable => true} 
  named_scope :determined_as_otu, lambda {|*args| {:conditions => ["lots.otu_id = ?", (args.first || -1)] }}
  named_scope :with_value_specimens, :conditions => 'lots.value_specimens > 0'

  validates_presence_of :otu
  validates_length_of :rarity, :maximum => 16, :allow_nil => true, :message=>"must be %d or fewer characters"
  validates_length_of :source_quality, :maximum => 16, :allow_nil => true, :message=>"must be %d or fewer characters"

  def validate_on_create
    if (key_specimens == 0) and (value_specimens == 0)
      errors.add(:key_specimens, "- now that's a whole LOT of nothing!")
    end
  end
  
  def display_name(options = {})
     opt = {
      :type => :selected 
     }.merge!(options.symbolize_keys)
    case opt[:type]
      when :selected 
        (display_identifiers == 'none' ? "#{self.id}" : "#{display_identifiers}") + ": #{otu.display_name(:type => :multi_name)}"
      else
        (display_identifiers == 'none' ? "#{self.id}" : "#{display_identifiers}") + ": #{otu.display_name(:type => :multi_name)}"
    end
  end
  
  def display_for_select
    "#{display_identifiers} / #{otu.display_for_select} (#{id})"
  end

  def display_for_list
    "#{otu.display_name}" + (display_identifiers == 'none' ? '' : "{#{display_identifiers}}") + " [#{id}]"
  end
  
  def display_identifiers
    @ids = self.lot_identifiers
    return 'none' if @ids.empty?
    @strs = @ids.map{|i| "#{i.identifier}"}.join("; ")
    # @strs = @ids.map{|i| "#{i.identifier}: #{i.namespace.id}"}
    #"[" + @strs.join("; ") + "]" 
  end

  def display_determination
    return "#{self.otu.display_name}"
  end
  
  def self.find_for_auto_complete(value)
    find_by_sql [
    "SELECT l.* FROM lots l 
    LEFT JOIN lot_identifiers i ON l.id = i.lot_id
    LEFT JOIN otus o ON l.otu_id = o.id 
    LEFT JOIN taxon_names t ON o.taxon_name_id = t.id 
    WHERE o.proj_id = #{$proj_id} 
    AND (o.name LIKE ? OR o.matrix_name LIKE ? OR t.name LIKE ? OR i.identifier LIKE ? or l.id = ?) LIMIT 30",
     "#{value.downcase}%", "#{value.downcase}%", "#{value.downcase}%", "%#{value.downcase}%", "#{value}"]
  end
 
  def total_specimens
    key_specimens + value_specimens
  end
  
end
