# == Schema Information
# Schema version: 20090930163041
#
# Table name: specimen_identifiers
#
#  id           :integer(4)      not null, primary key
#  specimen_id  :integer(4)      not null
#  identifier   :string(64)      not null
#  namespace_id :integer(4)      not null
#  creator_id   :integer(4)      not null
#  updator_id   :integer(4)      not null
#  updated_on   :timestamp       not null
#  created_on   :timestamp       not null
#  cached_display_name :string(255)
#

class SpecimenIdentifier < ActiveRecord::Base
  has_standard_fields

  belongs_to :specimen
  belongs_to :namespace

  validates_presence_of :specimen, :namespace, :identifier
  validates_uniqueness_of :identifier, :scope => [:specimen_id, :namespace_id]

  # see Specimen for validation

  def display_name(options = {})
    @opt = {
      :type => :line # :list, :head, :select, :sub_select
    }.merge!(options.symbolize_keys)
    s = ''
    case @opt[:type]
    when :head
        "#{self.identifier}<span class=\"small_grey\">&nbsp;#{self.namespace.name}</span>"
    else
      cached_display_name
    end
  end

  # called through has_standard_fields automagically
  def update_cached_display_name
    self.cached_display_name = "#{self.namespace.name} #{self.identifier}"
    true # continue the chain
  end

end
