# == Schema Information
# Schema version: 20090930163041
#
# Table name: standard_views
#
#  id            :integer(4)      not null, primary key
#  name          :string(64)
#  part_id       :integer(4)      not null DEPRECATED
#  image_view_id :integer(4)      not null
#  stage         :string(32)
#  sex           :string(32)
#  notes         :text
#  proj_id       :integer(4)      not null
#  updated_on    :timestamp       not null
#  created_on    :timestamp       not null
#  creator_id    :integer(4)      not null
#  updator_id    :integer(4)      not null
#  identifier    :string(255)
#  namespace_id  :integer(4)
#

class StandardView < ActiveRecord::Base
  has_standard_fields
  has_and_belongs_to_many :standard_view_groups
  
  belongs_to :image_view
  belongs_to :ontology_class

  has_many :continuous_characters, :class_name => 'Chr'

  validates_uniqueness_of :name, :scope => 'proj_id'
  validates_presence_of :image_view, :name

  def validate
    if name.blank? && ontology_class_id.blank?
      errors.add(:name, 'You must either give this view a name, or tie it to an ontology class')
    end
  end

  def image_descriptions # return all the images for a given standard view
    sql = "((part_id = #{part_id}) and (image_view_id = #{image_view_id})" 
    # unless sex (sql << "and (sex = #{sex})" )
    # unless stage (sql << "and (stage = #{stage})" )  not working
    sql << ')'
   
    return ImageDescription.find(:all, :conditions => sql) # shouldn't be a problem with Proj because parts belong to them
  end

  # named_scope
  def image_descriptions_by_otu(otu_id)
    sql = "((part_id = #{part_id}) and (image_view_id = #{image_view_id}) and (otu_id = #{otu_id})" 
    # unless sex (sql << "and (sex = #{sex})" )
    # unless stage (sql << "and (stage = #{stage})" )  not working
    sql << ')'
   
    return ImageDescription.find(:all, :conditions => sql)
  end

  def display_for_select
    name
  end

  def display_name(options = {})
    name
  end

  def self.find_for_auto_complete(params)
    tag_id_str = params[:tag_id]
    return [] if tag_id_str == nil

    value = params[tag_id_str.to_sym].split.join('%') # hmm... perhaps should make this order-independent

    lim = case params[tag_id_str.to_sym].length
          when 1..2 then  10
          when 3..4 then  25
          else lim = false # no limits
          end 

    StandardView.find(:all, :conditions => ["(standard_views.name LIKE ? OR standard_views.id = ?) AND standard_views.proj_id=?", "%#{value}%",  value.gsub(/\%/, ""), params[:proj_id]], :order => "standard_views.name", :limit => lim)
  end


end
