# !!! Be sure to restart your web server after you modify this file. !!!
# Make sure your apache/www/etc. group has read access to this file, it determines what the RoR app runs as.

### See config/local_config.rb for local configuration ###

# Specifies gem version of Rails to use when vendor/rails is not present
RAILS_GEM_VERSION = '2.2.2' unless defined? RAILS_GEM_VERSION
APPLICATION_VERSION = '0.2.n'

# Uncomment below to force Rails into production mode when 
# you don't control web/app server and can't set it the proper way
ENV['RAILS_ENV'] ||= 'production'

# added by kpd for clarity when running rake tasks, etc
# this isn't really true at this point all the time, some ENV rake tasks defaults set after this, e.g. rake test
puts "Using #{ENV['RAILS_ENV']} environment"

# Bootstrap the Rails environment, frameworks, and default configuration
require File.join(File.dirname(__FILE__), 'boot')

Rails::Initializer.run do |config|
  # Settings in config/environments/* take precedence those specified here
  
  # Skip frameworks you're not going to use
  # config.frameworks -= [ :action_web_service, :action_mailer ]

  # Add additional load paths for your own custom dirs
  config.load_paths += %W(#{RAILS_ROOT}/lib/model_extensions #{RAILS_ROOT}/lib/ontology #{RAILS_ROOT}/lib/ontology/batch_load #{RAILS_ROOT}/lib/ontology/visualize)

  # Force all environments to use the same logger level 
  # (by default production uses :info, the others :debug)
  # config.log_level = :debug

  # Use the database for sessions instead of the file system
  # (create the session table with 'rake create_sessions_table')
  # config.action_controller.session_store = :active_record_store

  # Enable page/fragment caching by setting a file-based store
  # (remember to create the caching directory and make it readable to the application)
  # config.action_controller.fragment_cache_store = :file_store, "#{RAILS_ROOT}/cache"

  # Activate observers that should always be running
  # config.active_record.observers = :cacher, :garbage_collector

  # Make Active Record use UTC-base instead of local time
  # config.active_record.default_timezone = :utc
  
  # Use Active Record's schema dumper instead of SQL when creating the test database
  # (enables use of different database adapters for development and test environments)
  # config.active_record.schema_format = :ruby

  # See Rails::Configuration for more options
  # TODO: add other dependencies here (check case) 
  # rake gems:install
  # rake gems:build

  require 'json/add/rails' # gets rid of some problems with later versions of json_pure and ascii_only? calls

  config.gem "fastercsv", :version => "1.5.3" 
  config.gem "ruby-debug"

  # gems that have spawned from mx!
  config.gem "obo_parser", :version => "0.2.0"
  config.gem "nexus_parser", :version => "1.1.3"
  config.gem "rubyMorphbank", :version => "0.2.1"

  config.gem "haml"
  config.gem "bioruby", :lib => "bio"
  config.gem "echoe"
  config.gem "RedCloth"
  config.gem "mysql"
  config.gem "ruby-graphviz", :lib => "graphviz"
  config.gem 'cmess' # character encoding guessing

  # Mx requires MySQL at present

  # TODO: migrate to another non :p_store
  # we can NOT use cookies because our sessions are large (multikey)
  config.action_controller.session_store=(:p_store)
  
  # config.action_controller.session = {
  #  :session_key => '_mx_session',
  #  :secret      => 'really long sekret here'
  #} 
  
  # most of these from InputDraw tags, note that all tags are downcased first, so for the 'viewBox' tag of InputDraw we need to upcase it as a hack in the view

  ALLOWED_SVG_TAGS        = ['g', 'path', 'line']
  ALLOWED_SVG_ATTRIBUTES  = ['d', 'x1', 'x2', 'y1', 'y2', 'stroke-dasharray', 'stroke-linecap', 'stroke-linejoin'] 
  
  config.action_view.sanitized_allowed_tags = ['ref', 'otu']
  config.action_view.sanitized_allowed_tags += ALLOWED_SVG_TAGS

  config.action_view.sanitized_allowed_attributes = ['id'] # 'version', 'xmlns'
  config.action_view.sanitized_allowed_attributes += ALLOWED_SVG_ATTRIBUTES

end


