CREATE TABLE `association_parts` (
  `id` int(11) NOT NULL auto_increment,
  `association_id` int(11) NOT NULL,
  `position` int(11) NOT NULL,
  `isa_id` int(11) default NULL,
  `isa_complement` tinyint(1) default NULL,
  `otu_id` int(11) NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `association_id_ind` (`association_id`),
  KEY `position_ind` (`position`),
  KEY `isa_id_ind` (`isa_id`),
  KEY `otu_id_ind` (`otu_id`),
  CONSTRAINT `association_parts_ibfk_1` FOREIGN KEY (`association_id`) REFERENCES `associations` (`id`),
  CONSTRAINT `association_parts_ibfk_2` FOREIGN KEY (`isa_id`) REFERENCES `isas` (`id`),
  CONSTRAINT `association_parts_ibfk_3` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1248 DEFAULT CHARSET=latin1;

CREATE TABLE `association_supports` (
  `id` int(11) NOT NULL auto_increment,
  `association_id` int(11) NOT NULL,
  `confidence_id` int(11) NOT NULL,
  `type` varchar(32) default NULL,
  `ref_id` int(11) default NULL,
  `voucher_lot_id` int(11) default NULL,
  `specimen_id` int(11) default NULL,
  `temp_ref` text,
  `temp_ref_mjy_id` int(11) default NULL,
  `setting` varchar(32) default NULL,
  `notes` text,
  `negative` tinyint(1) default '0',
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `association_id_ind` (`association_id`),
  KEY `confidence_id_ind` (`confidence_id`),
  KEY `negative_ind` (`negative`),
  CONSTRAINT `association_supports_ibfk_1` FOREIGN KEY (`confidence_id`) REFERENCES `confidences` (`id`),
  CONSTRAINT `association_supports_ibfk_2` FOREIGN KEY (`association_id`) REFERENCES `associations` (`id`),
  CONSTRAINT `association_supports_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `association_supports_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=554 DEFAULT CHARSET=latin1;

CREATE TABLE `associations` (
  `id` int(11) NOT NULL auto_increment,
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `associations_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `associations_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `associations_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=515 DEFAULT CHARSET=latin1;

CREATE TABLE `authors` (
  `id` int(11) NOT NULL auto_increment,
  `ref_id` int(11) NOT NULL,
  `position` int(11) default NULL,
  `last_name` varchar(255) NOT NULL,
  `first_name` varchar(255) default NULL,
  `title` varchar(255) default NULL,
  `initials` varchar(8) default NULL,
  `auth_is` varchar(16) NOT NULL default 'author',
  `use_initials` tinyint(1) default NULL,
  `name_with_init` varchar(255) default NULL,
  `join_name` varchar(255) default NULL,
  `namespace_id` int(11) default NULL,
  `external_id` int(11) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `ref_id` (`ref_id`),
  KEY `namespace_id` (`namespace_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `authors_ibfk_1` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `authors_ibfk_2` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `authors_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `authors_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=91877 DEFAULT CHARSET=utf8;

CREATE TABLE `berkeley_mapper_results` (
  `id` int(11) NOT NULL auto_increment,
  `tabfile` mediumtext,
  `proj_id` int(11) NOT NULL,
  `created_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  CONSTRAINT `berkeley_mapper_results_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1656 DEFAULT CHARSET=latin1;

CREATE TABLE `ces` (
  `id` int(11) NOT NULL auto_increment,
  `proj_id` int(11) NOT NULL,
  `namespace_id` int(11) default NULL,
  `external_id` int(11) default NULL,
  `verbatim_label` text,
  `num_to_print` int(11) default NULL,
  `collectors` text,
  `locality` text,
  `geography` text,
  `geog_id` int(11) default NULL,
  `mthd` varchar(255) default NULL,
  `sd_d` varchar(2) default NULL,
  `sd_m` varchar(4) default NULL,
  `sd_y` varchar(4) default NULL,
  `ed_d` varchar(2) default NULL,
  `ed_m` varchar(4) default NULL,
  `ed_y` varchar(4) default NULL,
  `latitude` double default NULL,
  `longitude` double default NULL,
  `lat_lon_error_m` int(11) default NULL COMMENT 'error in meters',
  `elev_min` double default NULL,
  `elev_max` double default NULL,
  `elev_unit` varchar(6) default NULL,
  `label_lat` varchar(48) default NULL,
  `label_lon` varchar(48) default NULL,
  `undet_ll` tinyint(1) NOT NULL default '0' COMMENT 'the label does not have a ll determinable beyond county level',
  `notes` text,
  `trip_code` varchar(255) default NULL,
  `trip_namespace_id` int(11) default NULL,
  `doc_label` text COMMENT 'an expanded version of the label for use in docuements/publications',
  `verbatim_method` varchar(255) default NULL,
  `host_genus` varchar(255) default NULL,
  `host_species` varchar(255) default NULL,
  `err_label` tinyint(1) NOT NULL default '0' COMMENT 'there is an error written on the label',
  `err_entry` tinyint(1) NOT NULL default '0' COMMENT 'there is an error in the field due to incorrect data entry',
  `err_checked` tinyint(1) NOT NULL default '0' COMMENT 'this label has been checked for errors, CElabel is verbatim label when checked',
  `undetgeog` tinyint(1) NOT NULL default '0' COMMENT 'the label does not contain a determinable geog',
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `print_label` text,
  `depth_min` float default NULL,
  `depth_max` float default NULL,
  PRIMARY KEY  (`id`),
  KEY `creator_id_ind` (`creator_id`),
  KEY `updator_id_ind` (`updator_id`),
  KEY `geog_id_ind` (`geog_id`),
  KEY `proj_id_ind` (`proj_id`),
  KEY `trip_namespace_id_ind` (`trip_namespace_id`),
  KEY `namespace_id_ind` (`namespace_id`),
  CONSTRAINT `ces_ibfk_1` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `ces_ibfk_2` FOREIGN KEY (`trip_namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `ces_ibfk_3` FOREIGN KEY (`geog_id`) REFERENCES `geogs` (`id`),
  CONSTRAINT `ces_ibfk_4` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `ces_ibfk_5` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `ces_ibfk_6` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=4956 DEFAULT CHARSET=latin1;

CREATE TABLE `chr_groups` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) default NULL,
  `notes` text,
  `position` int(11) default NULL,
  `content_type_id` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `content_type_id` (`content_type_id`),
  CONSTRAINT `chr_groups_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `chr_groups_ibfk_2` FOREIGN KEY (`content_type_id`) REFERENCES `content_types` (`id`),
  CONSTRAINT `chr_groups_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `chr_groups_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=175 DEFAULT CHARSET=latin1;

CREATE TABLE `chr_groups_chrs` (
  `chr_group_id` int(11) NOT NULL,
  `chr_id` int(11) NOT NULL,
  `position` int(11) default NULL,
  `id` int(11) NOT NULL auto_increment,
  PRIMARY KEY  (`id`),
  KEY `chr_group_id_ind` (`chr_group_id`),
  KEY `chr_id_ind` (`chr_id`),
  CONSTRAINT `chr_groups_chrs_ibfk_1` FOREIGN KEY (`chr_group_id`) REFERENCES `chr_groups` (`id`),
  CONSTRAINT `chr_groups_chrs_ibfk_2` FOREIGN KEY (`chr_id`) REFERENCES `chrs` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=2522 DEFAULT CHARSET=latin1;

CREATE TABLE `chr_groups_mxes` (
  `mx_id` int(11) NOT NULL,
  `chr_group_id` int(11) NOT NULL,
  PRIMARY KEY  (`mx_id`,`chr_group_id`),
  KEY `mx_id_ind` (`mx_id`),
  KEY `chr_group_id_ind` (`chr_group_id`),
  CONSTRAINT `chr_groups_mxes_ibfk_1` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`),
  CONSTRAINT `chr_groups_mxes_ibfk_2` FOREIGN KEY (`chr_group_id`) REFERENCES `chr_groups` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `chr_states` (
  `id` int(11) NOT NULL auto_increment,
  `chr_id` int(11) NOT NULL,
  `state` varchar(8) NOT NULL,
  `name` varchar(255) default NULL,
  `cited_polarity` varchar(15) default 'none',
  `hh_id` int(11) default NULL,
  `revision_history` text,
  `notes` text,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `id_state` (`chr_id`,`state`),
  KEY `id` (`id`,`state`,`name`),
  KEY `state` (`state`),
  KEY `name` (`name`),
  KEY `chr_id_ind` (`chr_id`),
  KEY `state_ind` (`state`),
  KEY `hh_id_ind` (`hh_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `chr_states_ibfk_1` FOREIGN KEY (`chr_id`) REFERENCES `chrs` (`id`),
  CONSTRAINT `chr_states_ibfk_2` FOREIGN KEY (`hh_id`) REFERENCES `hhs` (`id`),
  CONSTRAINT `chr_states_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `chr_states_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=6837 DEFAULT CHARSET=latin1;

CREATE TABLE `chromatograms` (
  `id` int(11) NOT NULL auto_increment,
  `pcr_id` int(11) default NULL,
  `primer_id` int(11) default NULL,
  `protocol_id` int(11) default NULL,
  `done_by` varchar(255) default NULL,
  `chromatograph_file` varchar(255) default NULL,
  `result` varchar(24) default NULL,
  `seq` text,
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `pcr_id` (`pcr_id`),
  KEY `primer_id` (`primer_id`),
  KEY `protocol_id` (`protocol_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `chromatograms_ibfk_1` FOREIGN KEY (`pcr_id`) REFERENCES `pcrs` (`id`),
  CONSTRAINT `chromatograms_ibfk_2` FOREIGN KEY (`primer_id`) REFERENCES `primers` (`id`),
  CONSTRAINT `chromatograms_ibfk_3` FOREIGN KEY (`protocol_id`) REFERENCES `protocols` (`id`),
  CONSTRAINT `chromatograms_ibfk_4` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `chromatograms_ibfk_5` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `chromatograms_ibfk_6` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `chrs` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL,
  `cited_in` int(11) default NULL,
  `cited_page` varchar(64) default NULL,
  `cited_char_no` varchar(4) default NULL,
  `revision_history` text COMMENT 'as in notes field for image records',
  `syn_with` int(11) default NULL COMMENT 'points to a chrs.id',
  `doc_char_code` varchar(4) default NULL COMMENT 'character code (generally number in cited work), or number to use in manuscript if uncited)',
  `doc_char_descr` text COMMENT 'textual description in citation, or to be used in citation if uncited',
  `short_name` varchar(6) default NULL,
  `notes` text,
  `continuous` tinyint(1) default '0',
  `ordered` tinyint(1) default '0',
  `position` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `cited_in_ind` (`cited_in`),
  KEY `syn_with_ind` (`syn_with`),
  CONSTRAINT `chrs_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `chrs_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `chrs_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `chrs_ibfk_4` FOREIGN KEY (`cited_in`) REFERENCES `refs` (`id`),
  CONSTRAINT `chrs_ibfk_5` FOREIGN KEY (`syn_with`) REFERENCES `chrs` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=2306 DEFAULT CHARSET=latin1;

CREATE TABLE `chrs_mxes` (
  `id` int(11) NOT NULL auto_increment,
  `chr_id` int(11) NOT NULL,
  `mx_id` int(11) NOT NULL,
  `position` int(11) default NULL,
  `notes` text,
  `creator_id` int(11) default NULL,
  `updator_id` int(11) default NULL,
  `updated_on` datetime default NULL,
  `created_on` datetime default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `mx_id` (`mx_id`,`chr_id`),
  KEY `chr_id_ind` (`chr_id`),
  KEY `mx_id_ind` (`mx_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `chrs_mxes_ibfk_1` FOREIGN KEY (`chr_id`) REFERENCES `chrs` (`id`),
  CONSTRAINT `chrs_mxes_ibfk_2` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`),
  CONSTRAINT `chrs_mxes_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `chrs_mxes_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=15108 DEFAULT CHARSET=latin1;

CREATE TABLE `claves` (
  `id` int(11) NOT NULL auto_increment,
  `parent_id` int(11) default NULL,
  `otu_id` int(11) default NULL,
  `couplet_text` text,
  `position` int(3) default NULL,
  `link_out` text,
  `link_out_text` varchar(1024) default NULL,
  `edit_annotation` text,
  `pub_annotation` text,
  `head_annotation` text,
  `manual_id` varchar(7) default NULL,
  `ref_id` int(11) default NULL,
  `l` int(11) default NULL,
  `r` int(11) default NULL,
  `is_public` tinyint(1) NOT NULL default '0',
  `redirect_id` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `redirect_id` (`redirect_id`),
  KEY `parent_id` (`parent_id`),
  KEY `otu_id` (`otu_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id_2` (`creator_id`),
  KEY `updator_id_2` (`updator_id`),
  KEY `ref_id` (`ref_id`),
  CONSTRAINT `claves_ibfk_1` FOREIGN KEY (`redirect_id`) REFERENCES `claves` (`id`),
  CONSTRAINT `claves_ibfk_2` FOREIGN KEY (`parent_id`) REFERENCES `claves` (`id`),
  CONSTRAINT `claves_ibfk_3` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `claves_ibfk_4` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `claves_ibfk_5` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `claves_ibfk_6` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `claves_ibfk_7` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1432 DEFAULT CHARSET=utf8;

CREATE TABLE `codings` (
  `id` int(11) NOT NULL auto_increment,
  `otu_id` int(11) NOT NULL,
  `chr_id` int(11) NOT NULL,
  `chr_state_id` int(11) NOT NULL,
  `continuous_state` double default NULL,
  `cited_in` int(11) default NULL,
  `notes` text,
  `chr_state_state` varchar(8) NOT NULL,
  `chr_state_name` varchar(255) default NULL,
  `qualifier` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `confidence_id` int(11) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `chr_state_id_otu_id` (`chr_state_id`,`otu_id`),
  KEY `coding_speed` (`otu_id`,`chr_id`,`chr_state_id`),
  KEY `cited_in_ind` (`cited_in`),
  KEY `otu_id_ind` (`otu_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `chr_id` (`chr_id`),
  KEY `chr_state_id` (`chr_state_id`),
  KEY `chr_state_id_2` (`chr_state_id`,`chr_state_state`,`chr_state_name`),
  KEY `codings_confidence_fk` (`confidence_id`),
  CONSTRAINT `codings_confidence_fk` FOREIGN KEY (`confidence_id`) REFERENCES `confidences` (`id`),
  CONSTRAINT `codings_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `codings_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `codings_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `codings_ibfk_4` FOREIGN KEY (`chr_id`) REFERENCES `chrs` (`id`),
  CONSTRAINT `codings_ibfk_5` FOREIGN KEY (`chr_state_id`, `chr_state_state`, `chr_state_name`) REFERENCES `chr_states` (`id`, `state`, `name`) ON UPDATE CASCADE,
  CONSTRAINT `codings_ibfk_6` FOREIGN KEY (`cited_in`) REFERENCES `refs` (`id`),
  CONSTRAINT `codings_ibfk_7` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `codings_ibfk_8` FOREIGN KEY (`confidence_id`) REFERENCES `confidences` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=101362 DEFAULT CHARSET=latin1;

CREATE TABLE `confidences` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(128) default NULL,
  `position` int(11) default NULL,
  `short_name` varchar(4) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `html_color` varchar(8) default NULL,
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `position_ind` (`position`),
  KEY `short_name` (`short_name`),
  KEY `short_name_proj` (`short_name`,`proj_id`),
  CONSTRAINT `confidences_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `confidences_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `confidences_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=32 DEFAULT CHARSET=latin1;

CREATE TABLE `content_templates` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL,
  `is_default` tinyint(1) NOT NULL default '0',
  `is_public` tinyint(1) NOT NULL default '0',
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `name` (`name`,`proj_id`),
  KEY `proj_id_ind` (`proj_id`),
  KEY `name_ind` (`name`),
  KEY `creator_id_ind` (`creator_id`),
  KEY `updator_id_ind` (`updator_id`),
  CONSTRAINT `content_templates_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `content_templates_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=47 DEFAULT CHARSET=latin1;

CREATE TABLE `content_templates_content_types` (
  `foo_id` int(11) NOT NULL auto_increment,
  `content_type_id` int(11) NOT NULL,
  `content_template_id` int(11) NOT NULL,
  `position` tinyint(4) default NULL,
  PRIMARY KEY  (`foo_id`),
  UNIQUE KEY `content_type_id` (`content_type_id`,`content_template_id`),
  KEY `content_type_id_ind` (`content_type_id`),
  KEY `content_template_id_ind` (`content_template_id`),
  CONSTRAINT `content_templates_content_types_ibfk_1` FOREIGN KEY (`content_type_id`) REFERENCES `content_types` (`id`),
  CONSTRAINT `content_templates_content_types_ibfk_2` FOREIGN KEY (`content_template_id`) REFERENCES `content_templates` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=254 DEFAULT CHARSET=latin1;

CREATE TABLE `content_types` (
  `id` int(11) NOT NULL auto_increment,
  `sti_type` varchar(255) default NULL,
  `is_public` tinyint(1) default '0',
  `name` varchar(255) default NULL,
  `can_markup` tinyint(1) default '1',
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `name` (`name`,`proj_id`),
  KEY `name_ind` (`name`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `content_types_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `content_types_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `content_types_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=199 DEFAULT CHARSET=latin1;

CREATE TABLE `contents` (
  `id` int(11) NOT NULL auto_increment,
  `otu_id` int(11) default NULL,
  `content_type_id` int(11) default NULL,
  `text` text,
  `is_public` tinyint(1) NOT NULL default '1',
  `pub_content_id` int(11) default NULL,
  `revision` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `otu_id_ind` (`otu_id`),
  KEY `content_type_id_ind` (`content_type_id`),
  KEY `pub_content_id_ind` (`pub_content_id`),
  CONSTRAINT `contents_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `contents_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `contents_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `contents_ibfk_4` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `contents_ibfk_5` FOREIGN KEY (`pub_content_id`) REFERENCES `contents` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=2023 DEFAULT CHARSET=utf8;

CREATE TABLE `data_sources` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL,
  `mx_id` int(11) default NULL,
  `dataset_id` int(11) default NULL,
  `notes` text,
  `ref_id` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `ref_id` (`ref_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `mx_id` (`mx_id`),
  CONSTRAINT `data_sources_ibfk_1` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `data_sources_ibfk_2` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`),
  CONSTRAINT `data_sources_ibfk_3` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `data_sources_ibfk_4` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `data_sources_ibfk_5` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=25 DEFAULT CHARSET=latin1;

CREATE TABLE `datasets` (
  `id` int(11) NOT NULL auto_increment,
  `parent_id` int(11) default NULL,
  `content_type` varchar(255) default NULL,
  `filename` varchar(1024) default NULL,
  `size` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `datasets_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `datasets_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `datasets_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=26 DEFAULT CHARSET=latin1;

CREATE TABLE `distributions` (
  `id` int(11) NOT NULL auto_increment,
  `geog_id` int(11) default NULL,
  `otu_id` int(11) default NULL,
  `ref_id` int(11) default NULL,
  `confidence_id` int(11) default NULL,
  `verbatim_geog` varchar(255) default NULL,
  `introduced` tinyint(1) default NULL,
  `num_specimens` int(11) NOT NULL,
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `distributions_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `distributions_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `distributions_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=9171 DEFAULT CHARSET=latin1;

CREATE TABLE `extracts` (
  `id` int(11) NOT NULL auto_increment,
  `lot_id` int(11) default NULL,
  `specimen_id` int(11) default NULL,
  `protocol_id` int(11) default NULL,
  `parts_extracted_from` text,
  `quality` varchar(12) default NULL,
  `notes` text,
  `extracted_on` date default NULL,
  `extracted_by` varchar(128) default NULL,
  `other_extract_identifier` varchar(128) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `lot_id_ind` (`lot_id`),
  KEY `specimen_id_ind` (`specimen_id`),
  KEY `protocol_id_ind` (`protocol_id`),
  CONSTRAINT `extracts_ibfk_1` FOREIGN KEY (`protocol_id`) REFERENCES `protocols` (`id`),
  CONSTRAINT `extracts_ibfk_2` FOREIGN KEY (`specimen_id`) REFERENCES `specimens` (`id`),
  CONSTRAINT `extracts_ibfk_3` FOREIGN KEY (`lot_id`) REFERENCES `lots` (`id`),
  CONSTRAINT `extracts_ibfk_4` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `extracts_ibfk_5` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `extracts_ibfk_6` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=250 DEFAULT CHARSET=latin1;

CREATE TABLE `figures` (
  `id` int(11) NOT NULL auto_increment,
  `addressable_id` int(11) default NULL,
  `addressable_type` varchar(64) default NULL,
  `image_id` int(11) NOT NULL,
  `position` tinyint(4) default NULL,
  `caption` text,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `proj_id` int(11) NOT NULL,
  `morphbank_annotation_id` int(10) default NULL,
  `svg_txt` text,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `addressable_id_3` (`addressable_id`,`addressable_type`,`image_id`,`proj_id`),
  KEY `image_id` (`image_id`),
  KEY `addressable_type` (`addressable_type`),
  KEY `addressable_id` (`addressable_id`),
  KEY `addressable_id_2` (`addressable_id`,`addressable_type`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `proj_id` (`proj_id`),
  CONSTRAINT `figures_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `figures_ibfk_2` FOREIGN KEY (`image_id`) REFERENCES `images` (`id`),
  CONSTRAINT `figures_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `figures_ibfk_4` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=5907 DEFAULT CHARSET=latin1;

CREATE TABLE `gel_images` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) default NULL,
  `file_name` varchar(64) NOT NULL,
  `user_file_name` varchar(255) default NULL,
  `file_md5` char(32) NOT NULL COMMENT 'Stores the md5 checksum of the image file.',
  `file_type` char(4) default NULL,
  `file_size` int(11) default NULL COMMENT 'In bytes.',
  `width` mediumint(9) default NULL COMMENT 'In pixels.',
  `height` mediumint(9) default NULL COMMENT 'In pixels.',
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `gel_images_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `gel_images_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `gel_images_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `gene_groups` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) default NULL,
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `gene_groups_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `gene_groups_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `gene_groups_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=23 DEFAULT CHARSET=latin1;

CREATE TABLE `gene_groups_genes` (
  `gene_group_id` int(11) NOT NULL,
  `gene_id` int(11) NOT NULL,
  `sort` tinyint(4) default NULL,
  KEY `gene_group_id` (`gene_group_id`),
  KEY `gene_id` (`gene_id`),
  CONSTRAINT `gene_groups_genes_ibfk_1` FOREIGN KEY (`gene_group_id`) REFERENCES `gene_groups` (`id`),
  CONSTRAINT `gene_groups_genes_ibfk_2` FOREIGN KEY (`gene_id`) REFERENCES `genes` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `genes` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) default NULL,
  `notes` text,
  `position` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `genes_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `genes_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `genes_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=88 DEFAULT CHARSET=latin1;

CREATE TABLE `geog_types` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) default NULL,
  `feature_class` int(11) default NULL,
  PRIMARY KEY  (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=327 DEFAULT CHARSET=latin1;

CREATE TABLE `geogs` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL,
  `abbreviation` varchar(64) default NULL,
  `fips_code` int(11) default NULL,
  `sort_NS` int(11) default NULL,
  `sort_WE` int(11) default NULL,
  `center_lat` varchar(64) default NULL,
  `center_long` varchar(64) default NULL,
  `geog_type_id` int(11) default NULL,
  `inclusive_biogeo_region_id` int(11) default NULL,
  `country_id` int(11) default NULL,
  `state_id` int(11) default NULL,
  `county_id` int(11) default NULL,
  `continent_ocean_id` int(11) default NULL,
  `namespace_id` int(11) default NULL,
  `external_id` int(11) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `namespace_id` (`namespace_id`,`external_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `name` (`name`(8)),
  KEY `namespace_id_ind` (`namespace_id`),
  KEY `geog_type_id_ind` (`geog_type_id`),
  KEY `inclusive_biogeo_region_id_ind` (`inclusive_biogeo_region_id`),
  KEY `country_id_ind` (`country_id`),
  KEY `state_id_ind` (`state_id`),
  KEY `county_id_ind` (`county_id`),
  KEY `continent_ocean_id_ind` (`continent_ocean_id`),
  CONSTRAINT `geogs_ibfk_1` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `geogs_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `geogs_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `geogs_ibfk_4` FOREIGN KEY (`geog_type_id`) REFERENCES `geog_types` (`id`),
  CONSTRAINT `geogs_ibfk_5` FOREIGN KEY (`inclusive_biogeo_region_id`) REFERENCES `geogs` (`id`),
  CONSTRAINT `geogs_ibfk_6` FOREIGN KEY (`country_id`) REFERENCES `geogs` (`id`),
  CONSTRAINT `geogs_ibfk_7` FOREIGN KEY (`state_id`) REFERENCES `geogs` (`id`),
  CONSTRAINT `geogs_ibfk_8` FOREIGN KEY (`county_id`) REFERENCES `geogs` (`id`),
  CONSTRAINT `geogs_ibfk_9` FOREIGN KEY (`continent_ocean_id`) REFERENCES `geogs` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=4520 DEFAULT CHARSET=latin1;

CREATE TABLE `hhs` (
  `id` int(11) NOT NULL auto_increment,
  `working_name` varchar(255) default NULL,
  `description` text,
  `revision_history` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `hhs_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `hhs_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `hhs_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `hhs_statements` (
  `hh_id` int(11) NOT NULL,
  `statement_id` int(11) NOT NULL auto_increment,
  `qualifier` varchar(64) default NULL COMMENT 'not implmented yet?',
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`hh_id`,`statement_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `statement_id_ind` (`statement_id`),
  CONSTRAINT `hhs_statements_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `hhs_statements_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `hhs_statements_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `hhs_statements_ibfk_4` FOREIGN KEY (`hh_id`) REFERENCES `hhs` (`id`),
  CONSTRAINT `hhs_statements_ibfk_5` FOREIGN KEY (`statement_id`) REFERENCES `statements` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `image_descriptions` (
  `id` int(11) NOT NULL auto_increment,
  `otu_id` int(11) NOT NULL,
  `proj_id` int(11) NOT NULL,
  `image_id` int(11) default NULL,
  `image_view_id` int(11) default NULL,
  `part_id` int(11) default NULL,
  `stage` varchar(32) default NULL,
  `sex` varchar(32) default NULL,
  `specimen_id` int(11) default NULL,
  `svg_txt` text,
  `notes` text,
  `is_public` tinyint(1) NOT NULL default '0',
  `priority` varchar(6) default NULL,
  `requestor_id` int(11) default NULL,
  `contractor_id` int(11) default NULL,
  `request_notes` text,
  `status` varchar(64) default NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `magnification` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  KEY `otu_id` (`otu_id`),
  KEY `proj_id` (`proj_id`),
  KEY `image_id` (`image_id`),
  KEY `image_view_id` (`image_view_id`),
  KEY `part_id` (`part_id`),
  KEY `specimen_id` (`specimen_id`),
  KEY `requestor_id` (`requestor_id`),
  KEY `contractor_id` (`contractor_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `image_descriptions_ibfk_1` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `image_descriptions_ibfk_10` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `image_descriptions_ibfk_2` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `image_descriptions_ibfk_3` FOREIGN KEY (`image_id`) REFERENCES `images` (`id`),
  CONSTRAINT `image_descriptions_ibfk_4` FOREIGN KEY (`image_view_id`) REFERENCES `image_views` (`id`),
  CONSTRAINT `image_descriptions_ibfk_5` FOREIGN KEY (`part_id`) REFERENCES `parts` (`id`),
  CONSTRAINT `image_descriptions_ibfk_6` FOREIGN KEY (`specimen_id`) REFERENCES `specimens` (`id`),
  CONSTRAINT `image_descriptions_ibfk_7` FOREIGN KEY (`requestor_id`) REFERENCES `people` (`id`),
  CONSTRAINT `image_descriptions_ibfk_8` FOREIGN KEY (`contractor_id`) REFERENCES `people` (`id`),
  CONSTRAINT `image_descriptions_ibfk_9` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=6663 DEFAULT CHARSET=latin1;

CREATE TABLE `image_views` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(64) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `image_views_ibfk_1` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `image_views_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=17 DEFAULT CHARSET=latin1;

CREATE TABLE `images` (
  `id` int(11) NOT NULL auto_increment,
  `file_name` varchar(64) default NULL,
  `file_md5` char(32) default NULL COMMENT 'Stores the md5 checksum of the image file.',
  `file_type` char(4) default NULL,
  `file_size` int(11) default NULL COMMENT 'In bytes.',
  `width` mediumint(9) default NULL COMMENT 'In pixels.',
  `height` mediumint(9) default NULL COMMENT 'In pixels.',
  `user_file_name` varchar(64) default NULL COMMENT 'The name the file had when the user uploaded it.',
  `taken_on_year` smallint(6) default NULL COMMENT 'Date the original image was taken. If only the year is known, that is ok.',
  `taken_on_month` tinyint(4) default NULL COMMENT 'Date the original image was taken. If only the year is known, that is ok.',
  `taken_on_day` tinyint(4) default NULL COMMENT 'Date the original image was taken. If only the year is known, that is ok.',
  `owner` varchar(255) default NULL COMMENT 'i.e. the copyright holder.',
  `ref_id` int(11) default NULL,
  `technique` varchar(12) default NULL,
  `mb_id` int(11) default NULL,
  `notes` text COMMENT 'On backend, add date and user to each comment- an append-only log file.',
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `proj_id` int(11) NOT NULL,
  `copyright_holder` varchar(255) default NULL,
  `contributor` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  KEY `ref_id` (`ref_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `images_ibfk_1` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `images_ibfk_2` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `images_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `images_ibfk_4` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=6702 DEFAULT CHARSET=latin1;

CREATE TABLE `isas` (
  `id` int(11) NOT NULL auto_increment,
  `interaction` varchar(255) default NULL,
  `complement` varchar(255) default NULL,
  `notes` text,
  `position` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `html_color` varchar(6) default NULL,
  `is_transitive` tinyint(1) default NULL,
  `is_reflexive` tinyint(1) default NULL,
  `is_anti_symmetric` tinyint(1) default NULL,
  `xref` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `interaction` (`interaction`,`complement`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `position_ind` (`position`),
  CONSTRAINT `isas_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `isas_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `isas_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=52 DEFAULT CHARSET=latin1;

CREATE TABLE `keywords` (
  `id` int(11) NOT NULL auto_increment,
  `keyword` varchar(255) NOT NULL,
  `shortform` varchar(6) default NULL,
  `explanation` text,
  `is_public` tinyint(1) default '0',
  `html_color` varchar(6) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `keyword` (`keyword`(4)),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `keywords_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `keywords_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `keywords_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=242 DEFAULT CHARSET=latin1;

CREATE TABLE `languages` (
  `id` int(11) NOT NULL auto_increment,
  `ltype` varchar(128) default NULL,
  `subtag` varchar(4) default NULL,
  `description` varchar(1024) default NULL,
  `suppress_script` varchar(256) default NULL,
  `preferred_value` varchar(4) default NULL,
  `tag` varchar(64) default NULL,
  `prfx` varchar(255) default NULL,
  `added` date default NULL,
  `deprecated` date default NULL,
  `comments` text,
  PRIMARY KEY  (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1028 DEFAULT CHARSET=latin1;

CREATE TABLE `lot_groups` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL,
  `notes` text,
  `is_loan` tinyint(1) NOT NULL default '0',
  `outgoing_loan` tinyint(1) NOT NULL default '0',
  `incoming_transaction_code` varchar(255) default NULL,
  `repository_id` int(11) default NULL,
  `material_requested` text,
  `date_requested` date default NULL,
  `date_recieved` date default NULL,
  `total_specimens_recieved` int(11) default NULL,
  `loan_start_date` date default NULL,
  `loan_end_date` date default NULL,
  `specimens_returned_date` date default NULL,
  `loan_closed` tinyint(1) NOT NULL default '0',
  `contact_name` varchar(255) default NULL,
  `contact_email` varchar(255) default NULL,
  `policy_page_url` varchar(255) default NULL,
  `loan_notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `repository_id` (`repository_id`),
  CONSTRAINT `lot_groups_ibfk_1` FOREIGN KEY (`repository_id`) REFERENCES `repositories` (`id`),
  CONSTRAINT `lot_groups_ibfk_2` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `lot_groups_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `lot_groups_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=120 DEFAULT CHARSET=latin1;

CREATE TABLE `lot_groups_lots` (
  `lot_id` int(11) NOT NULL,
  `lot_group_id` int(11) NOT NULL,
  `notes` text,
  PRIMARY KEY  (`lot_id`,`lot_group_id`),
  KEY `lot_id` (`lot_id`),
  KEY `lot_group_id` (`lot_group_id`),
  CONSTRAINT `lot_groups_lots_ibfk_1` FOREIGN KEY (`lot_id`) REFERENCES `lots` (`id`),
  CONSTRAINT `lot_groups_lots_ibfk_2` FOREIGN KEY (`lot_group_id`) REFERENCES `lot_groups` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `lot_identifiers` (
  `id` int(11) NOT NULL auto_increment,
  `lot_id` int(11) NOT NULL,
  `identifier` varchar(64) default NULL,
  `namespace_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `lot_id` (`lot_id`,`identifier`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `lot_id_ind` (`lot_id`),
  KEY `namespace_id_ind` (`namespace_id`),
  CONSTRAINT `lot_identifiers_ibfk_1` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `lot_identifiers_ibfk_2` FOREIGN KEY (`lot_id`) REFERENCES `lots` (`id`),
  CONSTRAINT `lot_identifiers_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `lot_identifiers_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3300 DEFAULT CHARSET=latin1;

CREATE TABLE `lots` (
  `id` int(11) NOT NULL auto_increment,
  `otu_id` int(11) NOT NULL,
  `key_specimens` int(11) NOT NULL default '0',
  `value_specimens` int(11) NOT NULL default '0',
  `ce_id` int(11) default NULL,
  `ce_labels` text COMMENT 'ultimately a ce_id, presently stores verbatim collecting event labels',
  `rarity` varchar(16) default NULL COMMENT '0-10 or whatever scale you want',
  `source_quality` varchar(16) default NULL COMMENT 'place to note the possible quality of the sample',
  `notes` text,
  `repository_id` int(11) default NULL,
  `dna_usable` tinyint(1) default '1' COMMENT 'the value specimens can be used for sequencing',
  `mixed_lot` tinyint(1) default '0' COMMENT 'more than 1 species',
  `sex` varchar(64) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `stage` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `repository_id_ind` (`repository_id`),
  KEY `otu_id_ind` (`otu_id`),
  CONSTRAINT `lots_ibfk_1` FOREIGN KEY (`repository_id`) REFERENCES `repositories` (`id`),
  CONSTRAINT `lots_ibfk_2` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `lots_ibfk_3` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `lots_ibfk_4` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `lots_ibfk_5` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=4923 DEFAULT CHARSET=latin1;

CREATE TABLE `measurements` (
  `id` int(11) NOT NULL auto_increment,
  `specimen_id` int(11) NOT NULL,
  `measurement` double default NULL,
  `standard_view_id` int(11) NOT NULL,
  `units` varchar(12) default NULL,
  `conversion_factor` double default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `specimen_id` (`specimen_id`),
  KEY `standard_view_id` (`standard_view_id`),
  CONSTRAINT `measurements_ibfk_1` FOREIGN KEY (`specimen_id`) REFERENCES `specimens` (`id`),
  CONSTRAINT `measurements_ibfk_2` FOREIGN KEY (`standard_view_id`) REFERENCES `standard_views` (`id`),
  CONSTRAINT `measurements_ibfk_3` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `measurements_ibfk_4` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `measurements_ibfk_5` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=latin1;

CREATE TABLE `mxes` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) default NULL,
  `revision_history` text,
  `notes` text,
  `web_description` text,
  `is_multikey` tinyint(1) default '0',
  `is_public` tinyint(1) default '0',
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `mxes_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `mxes_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `mxes_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=206 DEFAULT CHARSET=latin1;

CREATE TABLE `mxes_minus_chrs` (
  `chr_id` int(11) NOT NULL,
  `mx_id` int(11) NOT NULL,
  `id` int(11) NOT NULL auto_increment,
  PRIMARY KEY  (`id`),
  KEY `chr_id_ind` (`chr_id`),
  KEY `mx_id_ind` (`mx_id`),
  CONSTRAINT `mxes_minus_chrs_ibfk_1` FOREIGN KEY (`chr_id`) REFERENCES `chrs` (`id`),
  CONSTRAINT `mxes_minus_chrs_ibfk_2` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=149 DEFAULT CHARSET=latin1;

CREATE TABLE `mxes_minus_otus` (
  `id` int(11) NOT NULL auto_increment,
  `mx_id` int(4) NOT NULL,
  `otu_id` int(4) NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `mx_id` (`mx_id`),
  KEY `otu_id` (`otu_id`),
  CONSTRAINT `mxes_minus_otus_ibfk_1` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`),
  CONSTRAINT `mxes_minus_otus_ibfk_2` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=140 DEFAULT CHARSET=latin1;

CREATE TABLE `mxes_otu_groups` (
  `mx_id` int(4) NOT NULL,
  `otu_group_id` int(4) NOT NULL,
  KEY `mx_id` (`mx_id`),
  KEY `otu_group_id` (`otu_group_id`),
  CONSTRAINT `mxes_otu_groups_ibfk_1` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`),
  CONSTRAINT `mxes_otu_groups_ibfk_2` FOREIGN KEY (`otu_group_id`) REFERENCES `otu_groups` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `mxes_otus` (
  `mx_id` int(11) NOT NULL,
  `otu_id` int(11) NOT NULL,
  `notes` text,
  `position` int(11) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `id` int(11) NOT NULL auto_increment,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `mx_id` (`mx_id`,`otu_id`),
  KEY `mx_id_ind` (`mx_id`),
  KEY `otu_id_ind` (`otu_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `mxes_otus_ibfk_1` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `mxes_otus_ibfk_2` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `mxes_otus_ibfk_3` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`),
  CONSTRAINT `mxes_otus_ibfk_4` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=6025 DEFAULT CHARSET=latin1;

CREATE TABLE `mxes_plus_chrs` (
  `chr_id` int(11) NOT NULL,
  `mx_id` int(11) NOT NULL,
  `id` int(11) NOT NULL auto_increment,
  PRIMARY KEY  (`id`),
  KEY `mx_id_ind` (`mx_id`),
  KEY `chr_id_ind` (`chr_id`),
  CONSTRAINT `mxes_plus_chrs_ibfk_3` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`),
  CONSTRAINT `mxes_plus_chrs_ibfk_4` FOREIGN KEY (`chr_id`) REFERENCES `chrs` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=740 DEFAULT CHARSET=latin1;

CREATE TABLE `mxes_plus_otus` (
  `id` int(11) NOT NULL auto_increment,
  `mx_id` int(4) NOT NULL,
  `otu_id` int(4) NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `mx_id` (`mx_id`),
  KEY `otu_id` (`otu_id`),
  CONSTRAINT `mxes_plus_otus_ibfk_1` FOREIGN KEY (`mx_id`) REFERENCES `mxes` (`id`),
  CONSTRAINT `mxes_plus_otus_ibfk_2` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3895 DEFAULT CHARSET=latin1;

CREATE TABLE `namespaces` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL,
  `owner` varchar(255) default NULL,
  `notes` text,
  `url_access` text,
  `short_name` varchar(6) default NULL,
  `last_loaded_on` datetime default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `namespaces_ibfk_1` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `namespaces_ibfk_2` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=32 DEFAULT CHARSET=latin1;

CREATE TABLE `news` (
  `id` int(11) NOT NULL auto_increment,
  `news_type` varchar(255) default NULL,
  `body` text,
  `expires_on` date default NULL,
  `proj_id` int(11) default NULL,
  `title` varchar(255) default NULL,
  `is_public` tinyint(1) NOT NULL default '0',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `proj_id` (`proj_id`),
  CONSTRAINT `news_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `news_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `news_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=53 DEFAULT CHARSET=latin1;

CREATE TABLE `ontologies` (
  `id` int(11) NOT NULL auto_increment,
  `part1_id` int(11) NOT NULL,
  `part2_id` int(11) NOT NULL,
  `isa_id` int(11) NOT NULL,
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `part1_id` (`part1_id`,`part2_id`,`isa_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `part1_id_2` (`part1_id`),
  KEY `part2_id` (`part2_id`),
  KEY `isa_id` (`isa_id`),
  CONSTRAINT `ontologies_ibfk_1` FOREIGN KEY (`part1_id`) REFERENCES `parts` (`id`),
  CONSTRAINT `ontologies_ibfk_2` FOREIGN KEY (`part2_id`) REFERENCES `parts` (`id`),
  CONSTRAINT `ontologies_ibfk_3` FOREIGN KEY (`isa_id`) REFERENCES `isas` (`id`),
  CONSTRAINT `ontologies_ibfk_4` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `ontologies_ibfk_5` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `ontologies_ibfk_6` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=4727 DEFAULT CHARSET=latin1;

CREATE TABLE `otu_groups` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(64) default NULL,
  `is_public` tinyint(1) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `otu_groups_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `otu_groups_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `otu_groups_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=260 DEFAULT CHARSET=latin1;

CREATE TABLE `otu_groups_otus` (
  `otu_group_id` int(11) NOT NULL,
  `otu_id` int(11) NOT NULL,
  `position` int(11) default NULL,
  `id` int(11) NOT NULL auto_increment,
  PRIMARY KEY  (`id`),
  KEY `otu_group_id_ind` (`otu_group_id`),
  KEY `otu_id_ind` (`otu_id`),
  CONSTRAINT `otu_groups_otus_ibfk_1` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `otu_groups_otus_ibfk_2` FOREIGN KEY (`otu_group_id`) REFERENCES `otu_groups` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=6745 DEFAULT CHARSET=latin1;

CREATE TABLE `otus` (
  `id` int(11) NOT NULL auto_increment,
  `taxon_name_id` int(11) default NULL,
  `is_child` tinyint(1) default '0',
  `name` varchar(255) default NULL,
  `manuscript_name` varchar(255) default NULL,
  `matrix_name` varchar(64) default NULL COMMENT 'when present overrides name for matrix output',
  `parent_otu_id` int(11) default NULL,
  `as_cited_in` int(11) default NULL,
  `revision_history` text,
  `iczn_group` varchar(32) default NULL,
  `syn_with_otu_id` int(11) default NULL COMMENT 'references another OTU',
  `sensu` varchar(255) default NULL,
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `taxon_name_id_ind` (`taxon_name_id`),
  KEY `as_cited_in_ind` (`as_cited_in`),
  KEY `syn_with_otu_id` (`syn_with_otu_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `otus_ibfk_1` FOREIGN KEY (`as_cited_in`) REFERENCES `refs` (`id`),
  CONSTRAINT `otus_ibfk_2` FOREIGN KEY (`syn_with_otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `otus_ibfk_3` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `otus_ibfk_4` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `otus_ibfk_5` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=11829 DEFAULT CHARSET=latin1;

CREATE TABLE `otus_statements` (
  `otu_id` int(11) NOT NULL,
  `statement_id` int(11) NOT NULL,
  `qualifier` varchar(255) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`otu_id`,`statement_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `creator_id_ind` (`creator_id`),
  KEY `statement_id_ind` (`statement_id`),
  KEY `otu_id_ind` (`otu_id`),
  CONSTRAINT `otus_statements_ibfk_1` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `otus_statements_ibfk_2` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `otus_statements_ibfk_3` FOREIGN KEY (`statement_id`) REFERENCES `statements` (`id`),
  CONSTRAINT `otus_statements_ibfk_4` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `parts` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(128) NOT NULL COMMENT 'name of the morphological part',
  `abbrev` varchar(24) default NULL,
  `description` text,
  `ref_id` int(11) default NULL COMMENT 'a ref_id wherin the name was *originally* proposed',
  `ref_page` varchar(25) default NULL COMMENT 'page within reference',
  `notes` text,
  `taxon_name_id` int(11) default NULL,
  `language_id` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `obo_dbxref` varchar(255) default NULL,
  `is_public` tinyint(1) default NULL,
  `is_acronym` tinyint(1) default NULL,
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `ref_id` (`ref_id`),
  KEY `taxon_name_id` (`taxon_name_id`),
  KEY `language_id` (`language_id`),
  CONSTRAINT `parts_ibfk_1` FOREIGN KEY (`taxon_name_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `parts_ibfk_2` FOREIGN KEY (`language_id`) REFERENCES `languages` (`id`),
  CONSTRAINT `parts_ibfk_3` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `parts_ibfk_4` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `parts_ibfk_5` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `parts_ibfk_6` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=4621 DEFAULT CHARSET=latin1;

CREATE TABLE `parts_refs` (
  `id` int(11) NOT NULL auto_increment,
  `ref_id` int(4) NOT NULL,
  `part_id` int(4) NOT NULL,
  `total` int(4) NOT NULL default '0',
  `created_on` datetime default NULL,
  `updated_on` datetime default NULL,
  PRIMARY KEY  (`id`),
  KEY `ref_id` (`ref_id`),
  KEY `index_parts_refs_on_part_id_and_ref_id` (`part_id`,`ref_id`),
  CONSTRAINT `parts_refs_ibfk_1` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `parts_refs_ibfk_2` FOREIGN KEY (`part_id`) REFERENCES `parts` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `pcrs` (
  `id` int(11) NOT NULL auto_increment,
  `extract_id` int(11) default NULL,
  `fwd_primer_id` int(11) default NULL,
  `rev_primer_id` int(11) default NULL,
  `protocol_id` int(11) default NULL,
  `gel_image_id` int(11) default NULL,
  `lane` tinyint(4) default NULL,
  `done_by` varchar(255) default NULL,
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `confidence_id` int(11) default NULL,
  PRIMARY KEY  (`id`),
  KEY `gel_image_id` (`gel_image_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `fwd_primer_id_ind` (`fwd_primer_id`),
  KEY `rev_primer_id_ind` (`rev_primer_id`),
  KEY `extract_id_ind` (`extract_id`),
  KEY `index_pcrs_on_confidence_id` (`confidence_id`),
  CONSTRAINT `pcrs_ibfk_1` FOREIGN KEY (`gel_image_id`) REFERENCES `gel_images` (`id`),
  CONSTRAINT `pcrs_ibfk_2` FOREIGN KEY (`extract_id`) REFERENCES `extracts` (`id`),
  CONSTRAINT `pcrs_ibfk_3` FOREIGN KEY (`fwd_primer_id`) REFERENCES `primers` (`id`),
  CONSTRAINT `pcrs_ibfk_4` FOREIGN KEY (`rev_primer_id`) REFERENCES `primers` (`id`),
  CONSTRAINT `pcrs_ibfk_5` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `pcrs_ibfk_6` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `pcrs_ibfk_7` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `pcrs_ibfk_8` FOREIGN KEY (`confidence_id`) REFERENCES `confidences` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1047 DEFAULT CHARSET=latin1;

CREATE TABLE `pdfs` (
  `id` int(11) NOT NULL auto_increment,
  `parent_id` int(11) default NULL,
  `content_type` varchar(255) default NULL,
  `filename` varchar(1024) default NULL,
  `size` int(11) default NULL,
  `is_ocred` tinyint(1) default NULL,
  PRIMARY KEY  (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=215 DEFAULT CHARSET=latin1;

CREATE TABLE `people` (
  `id` int(11) NOT NULL auto_increment,
  `last_name` varchar(255) NOT NULL default '',
  `first_name` varchar(100) NOT NULL default '',
  `middle_name` varchar(100) default NULL,
  `login` varchar(32) default NULL,
  `password` varchar(40) default NULL,
  `is_admin` tinyint(1) NOT NULL default '0',
  `creates_projects` tinyint(1) NOT NULL default '0',
  `email` varchar(255) default NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `pref_mx_display_width` smallint(3) default '20',
  `pref_mx_display_height` smallint(3) default '10',
  `pref_creator_html_color` varchar(6) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `last_name` (`last_name`,`first_name`,`middle_name`),
  UNIQUE KEY `login` (`login`),
  KEY `login_ind` (`login`),
  KEY `password_ind` (`password`),
  KEY `is_admin_ind` (`is_admin`)
) ENGINE=InnoDB AUTO_INCREMENT=135 DEFAULT CHARSET=latin1;

CREATE TABLE `people_projs` (
  `person_id` int(11) NOT NULL,
  `proj_id` int(11) NOT NULL,
  PRIMARY KEY  (`person_id`,`proj_id`),
  KEY `person_id_ind` (`person_id`),
  KEY `proj_id_ind` (`proj_id`),
  CONSTRAINT `people_projs_ibfk_1` FOREIGN KEY (`person_id`) REFERENCES `people` (`id`),
  CONSTRAINT `people_projs_ibfk_2` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `people_taxon_names` (
  `person_id` int(11) NOT NULL,
  `taxon_name_id` int(11) NOT NULL,
  PRIMARY KEY  (`person_id`,`taxon_name_id`),
  KEY `taxon_name_id` (`taxon_name_id`),
  CONSTRAINT `people_taxon_names_ibfk_1` FOREIGN KEY (`person_id`) REFERENCES `people` (`id`),
  CONSTRAINT `people_taxon_names_ibfk_2` FOREIGN KEY (`taxon_name_id`) REFERENCES `taxon_names` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `primers` (
  `id` int(11) NOT NULL auto_increment,
  `gene_id` int(11) default NULL,
  `name` varchar(64) default NULL,
  `sequence` varchar(255) default NULL,
  `regex` varchar(255) default NULL,
  `ref_id` int(11) default NULL,
  `protocol_id` int(11) default NULL,
  `notes` text,
  `designed_by` varchar(255) default NULL,
  `target_otu_id` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `sequence` (`sequence`,`gene_id`,`proj_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `ref_id` (`ref_id`),
  KEY `protocol_id` (`protocol_id`),
  KEY `gene_id_ind` (`gene_id`),
  KEY `target_otu_id` (`target_otu_id`),
  CONSTRAINT `primers_ibfk_1` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `primers_ibfk_2` FOREIGN KEY (`protocol_id`) REFERENCES `protocols` (`id`),
  CONSTRAINT `primers_ibfk_3` FOREIGN KEY (`target_otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `primers_ibfk_4` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `primers_ibfk_5` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `primers_ibfk_6` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `primers_ibfk_7` FOREIGN KEY (`gene_id`) REFERENCES `genes` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=184 DEFAULT CHARSET=latin1;

CREATE TABLE `projs` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL,
  `hidden_tabs` text,
  `public_server_name` varchar(255) default NULL,
  `unix_name` varchar(32) default NULL,
  `public_controllers` text,
  `public_tn_criteria` varchar(32) default NULL,
  `repository_id` int(11) default NULL,
  `starting_tab` varchar(32) default 'otu',
  `default_ontology_id` int(11) default NULL,
  `default_content_template_id` int(11) default NULL,
  `gmaps_API_key` varchar(90) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `ontology_namespace` varchar(32) default NULL,
  `default_ontology_term_id` int(11) default NULL,
  `obo_remark` text,
  `ontology_inclusion_keyword_id` int(11) default NULL,
  `ontology_exclusion_keyword_id` int(11) default NULL,
  PRIMARY KEY  (`id`),
  KEY `public_server_name` (`public_server_name`),
  KEY `repository_id` (`repository_id`),
  KEY `default_ontology_id` (`default_ontology_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `default_content_template_id` (`default_content_template_id`),
  KEY `index_projs_on_default_ontology_term_id` (`default_ontology_term_id`),
  KEY `ontology_inclusion_keyword_id` (`ontology_inclusion_keyword_id`),
  KEY `ontology_exclusion_keyword_id` (`ontology_exclusion_keyword_id`),
  CONSTRAINT `projs_ibfk_1` FOREIGN KEY (`repository_id`) REFERENCES `repositories` (`id`),
  CONSTRAINT `projs_ibfk_2` FOREIGN KEY (`default_ontology_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `projs_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `projs_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `projs_ibfk_5` FOREIGN KEY (`default_content_template_id`) REFERENCES `content_templates` (`id`),
  CONSTRAINT `projs_ibfk_6` FOREIGN KEY (`default_ontology_term_id`) REFERENCES `parts` (`id`),
  CONSTRAINT `projs_ibfk_8` FOREIGN KEY (`ontology_inclusion_keyword_id`) REFERENCES `keywords` (`id`),
  CONSTRAINT `projs_ibfk_9` FOREIGN KEY (`ontology_exclusion_keyword_id`) REFERENCES `keywords` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=72 DEFAULT CHARSET=latin1;

CREATE TABLE `projs_refs` (
  `proj_id` int(11) NOT NULL,
  `ref_id` int(11) NOT NULL,
  `id` int(11) NOT NULL auto_increment,
  `position` int(11) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `proj_id` (`proj_id`,`ref_id`),
  UNIQUE KEY `projs_refs_index` (`proj_id`,`ref_id`),
  KEY `proj_id_2` (`proj_id`),
  KEY `ref_id` (`ref_id`)
) ENGINE=InnoDB AUTO_INCREMENT=46577 DEFAULT CHARSET=latin1;

CREATE TABLE `projs_taxon_names` (
  `id` int(11) NOT NULL auto_increment,
  `proj_id` int(11) NOT NULL,
  `taxon_name_id` int(11) NOT NULL,
  `is_public` tinyint(1) NOT NULL default '0',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `taxon_name_id` (`taxon_name_id`),
  KEY `proj_id_2` (`proj_id`,`taxon_name_id`),
  CONSTRAINT `projs_taxon_names_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `projs_taxon_names_ibfk_2` FOREIGN KEY (`taxon_name_id`) REFERENCES `taxon_names` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=76 DEFAULT CHARSET=latin1;

CREATE TABLE `protocol_steps` (
  `id` int(11) NOT NULL auto_increment,
  `protocol_id` int(11) NOT NULL,
  `description` text,
  `reagent` varchar(255) default NULL,
  `reagent_quanitity` varchar(64) default NULL,
  `step_time` varchar(64) default NULL,
  `step_order` int(11) default NULL,
  `step_temp` double default NULL,
  `step_cycles` int(11) default NULL,
  PRIMARY KEY  (`id`),
  KEY `protocol_id_ind` (`protocol_id`),
  CONSTRAINT `protocol_steps_ibfk_1` FOREIGN KEY (`protocol_id`) REFERENCES `protocols` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=latin1;

CREATE TABLE `protocols` (
  `id` int(11) NOT NULL auto_increment,
  `kind` varchar(10) default NULL,
  `description` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `protocols_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `protocols_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `protocols_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=32 DEFAULT CHARSET=latin1;

CREATE TABLE `refs` (
  `id` int(11) NOT NULL auto_increment,
  `namespace_id` int(11) default NULL,
  `external_id` int(11) default NULL,
  `serial_id` int(11) default NULL,
  `valid_ref_id` int(11) default NULL,
  `language_id` int(11) default NULL,
  `pdf_id` int(11) default NULL,
  `year` smallint(6) default NULL,
  `year_letter` varchar(255) default NULL,
  `ref_type` varchar(50) default NULL,
  `title` text,
  `volume` varchar(255) default NULL,
  `issue` varchar(255) default NULL,
  `pages` varchar(255) default NULL,
  `pg_start` varchar(8) default NULL,
  `pg_end` varchar(8) default NULL,
  `book_title` text,
  `city` varchar(255) default NULL,
  `publisher` varchar(255) default NULL,
  `institution` varchar(255) default NULL,
  `date` varchar(255) default NULL,
  `language_OLD` varchar(255) default NULL,
  `notes` text,
  `ISBN` varchar(14) default NULL,
  `DOI` varchar(255) default NULL,
  `is_public` tinyint(1) default '0',
  `pub_med_url` text,
  `other_url` text,
  `full_citation` text,
  `temp_citation` text,
  `cached_display_name` varchar(2047) default NULL,
  `short_citation` varchar(255) default NULL,
  `author` varchar(255) default NULL,
  `journal` varchar(255) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `ocr_text` text,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `namespace_id_3` (`namespace_id`,`external_id`),
  KEY `language_id` (`language_id`),
  KEY `pdf_id` (`pdf_id`),
  KEY `namespace_id` (`namespace_id`),
  KEY `author_ind` (`author`),
  KEY `display_name_ind` (`cached_display_name`(10)),
  KEY `year_ind` (`year`),
  KEY `external_id` (`external_id`),
  KEY `namespace_id_2` (`namespace_id`),
  KEY `serial_id` (`serial_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `refs_ibfk_1` FOREIGN KEY (`pdf_id`) REFERENCES `pdfs` (`id`),
  CONSTRAINT `refs_ibfk_2` FOREIGN KEY (`language_id`) REFERENCES `languages` (`id`),
  CONSTRAINT `refs_ibfk_3` FOREIGN KEY (`serial_id`) REFERENCES `serials` (`id`),
  CONSTRAINT `refs_ibfk_4` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `refs_ibfk_5` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `refs_ibfk_6` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=68181 DEFAULT CHARSET=utf8;

CREATE TABLE `repositories` (
  `id` int(11) NOT NULL auto_increment,
  `name` text NOT NULL,
  `coden` varchar(12) default NULL,
  `url` text,
  `synonymous_with_id` int(11) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `codon` (`coden`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `synonymous_with_id` (`synonymous_with_id`),
  CONSTRAINT `repositories_ibfk_1` FOREIGN KEY (`synonymous_with_id`) REFERENCES `repositories` (`id`),
  CONSTRAINT `repositories_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `repositories_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1481 DEFAULT CHARSET=latin1;

CREATE TABLE `schema_migrations` (
  `version` varchar(255) NOT NULL,
  UNIQUE KEY `unique_schema_migrations` (`version`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `seqs` (
  `id` int(11) NOT NULL auto_increment,
  `gene_id` int(11) default NULL,
  `specimen_id` int(11) default NULL,
  `type_of_voucher` varchar(32) default NULL,
  `otu_id` int(11) default NULL,
  `genbank_identifier` varchar(24) default NULL,
  `ref_id` int(11) default NULL,
  `sequence` text,
  `attempt_complete` tinyint(1) default '0',
  `assigned_to` varchar(64) default NULL,
  `notes` text,
  `status` varchar(32) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `pcr_id` int(11) default NULL,
  PRIMARY KEY  (`id`),
  KEY `specimen_id` (`specimen_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `gene_id_ind` (`gene_id`),
  KEY `out_id_ind` (`otu_id`),
  KEY `index_seqs_on_pcr_id` (`pcr_id`),
  CONSTRAINT `seqs_ibfk_1` FOREIGN KEY (`specimen_id`) REFERENCES `specimens` (`id`),
  CONSTRAINT `seqs_ibfk_2` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `seqs_ibfk_3` FOREIGN KEY (`gene_id`) REFERENCES `genes` (`id`),
  CONSTRAINT `seqs_ibfk_4` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `seqs_ibfk_5` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `seqs_ibfk_6` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `seqs_ibfk_7` FOREIGN KEY (`pcr_id`) REFERENCES `pcrs` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1451 DEFAULT CHARSET=latin1;

CREATE TABLE `serials` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(1024) default NULL,
  `city` varchar(255) default NULL,
  `atTAMU` varchar(12) default NULL,
  `notes` text,
  `URL` text,
  `call_num` varchar(255) default NULL,
  `abbreviation` varchar(255) default NULL,
  `synonymous_with_id` int(11) default NULL,
  `language_id` int(11) default NULL,
  `namespace_id` int(11) default NULL,
  `external_id` int(11) default NULL,
  `issn_print` varchar(10) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `issn_digital` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `namespace_id` (`namespace_id`,`external_id`),
  KEY `language_id` (`language_id`),
  KEY `namespace_ind` (`namespace_id`),
  KEY `synonymous_with_id` (`synonymous_with_id`),
  KEY `name` (`name`(100)),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `serials_ibfk_1` FOREIGN KEY (`language_id`) REFERENCES `languages` (`id`),
  CONSTRAINT `serials_ibfk_2` FOREIGN KEY (`synonymous_with_id`) REFERENCES `serials` (`id`),
  CONSTRAINT `serials_ibfk_3` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `serials_ibfk_4` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `serials_ibfk_5` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=11161 DEFAULT CHARSET=latin1;

CREATE TABLE `specimen_determinations` (
  `id` int(11) NOT NULL auto_increment,
  `specimen_id` int(11) default NULL,
  `otu_id` int(11) default NULL,
  `current_det` tinyint(1) default '1',
  `determiner` varchar(255) default NULL,
  `name` varchar(255) default NULL,
  `det_year` varchar(4) default NULL,
  `confidence_id` int(11) default NULL,
  `determination_basis` varchar(255) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `specimen_id_ind` (`specimen_id`),
  KEY `otu_id_ind` (`otu_id`),
  KEY `confidence_id_ind` (`confidence_id`),
  CONSTRAINT `specimen_determinations_ibfk_1` FOREIGN KEY (`confidence_id`) REFERENCES `confidences` (`id`),
  CONSTRAINT `specimen_determinations_ibfk_2` FOREIGN KEY (`specimen_id`) REFERENCES `specimens` (`id`),
  CONSTRAINT `specimen_determinations_ibfk_3` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`),
  CONSTRAINT `specimen_determinations_ibfk_4` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `specimen_determinations_ibfk_5` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3793 DEFAULT CHARSET=latin1;

CREATE TABLE `specimen_identifiers` (
  `id` int(11) NOT NULL auto_increment,
  `specimen_id` int(11) NOT NULL,
  `identifier` varchar(64) NOT NULL,
  `namespace_id` int(11) NOT NULL COMMENT 'filled if identifier comes from an alternate source',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `display_name` varchar(255) default NULL,
  `position` int(11) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `identifier` (`identifier`,`specimen_id`,`namespace_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `namespace_ind` (`namespace_id`),
  KEY `specimen_id_ind` (`specimen_id`),
  CONSTRAINT `specimen_identifiers_ibfk_1` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `specimen_identifiers_ibfk_2` FOREIGN KEY (`specimen_id`) REFERENCES `specimens` (`id`),
  CONSTRAINT `specimen_identifiers_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `specimen_identifiers_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3699 DEFAULT CHARSET=latin1;

CREATE TABLE `specimens` (
  `id` int(11) NOT NULL auto_increment,
  `ce_id` int(11) default NULL,
  `temp_ce` text,
  `parent_specimen_id` int(11) default NULL,
  `repository_id` int(11) default NULL,
  `dna_usable` tinyint(1) default '0',
  `notes` text,
  `sex` varchar(64) default NULL,
  `stage` varchar(64) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `preparations` text,
  `disposition` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `repository_id_ind` (`repository_id`),
  KEY `parent_specimen_id_ind` (`parent_specimen_id`),
  KEY `ce_id_ind` (`ce_id`),
  CONSTRAINT `specimens_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `specimens_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `specimens_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `specimens_ibfk_4` FOREIGN KEY (`repository_id`) REFERENCES `repositories` (`id`),
  CONSTRAINT `specimens_ibfk_5` FOREIGN KEY (`parent_specimen_id`) REFERENCES `specimens` (`id`),
  CONSTRAINT `specimens_ibfk_6` FOREIGN KEY (`ce_id`) REFERENCES `ces` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=5211 DEFAULT CHARSET=latin1;

CREATE TABLE `standard_view_groups` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) default NULL,
  `notes` text,
  `other_identifier` varchar(32) default NULL,
  `proj_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `name` (`name`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `standard_view_groups_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `standard_view_groups_ibfk_2` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `standard_view_groups_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=12 DEFAULT CHARSET=latin1;

CREATE TABLE `standard_view_groups_standard_views` (
  `standard_view_id` int(11) NOT NULL,
  `standard_view_group_id` int(11) NOT NULL,
  `proj_id` int(11) NOT NULL,
  `sort` int(11) default NULL,
  PRIMARY KEY  (`standard_view_id`,`standard_view_group_id`),
  UNIQUE KEY `proj_id` (`proj_id`,`standard_view_group_id`,`standard_view_id`),
  KEY `proj_id_2` (`proj_id`),
  KEY `standard_view_id` (`standard_view_id`),
  KEY `standard_view_group_id` (`standard_view_group_id`),
  CONSTRAINT `standard_view_groups_standard_views_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `standard_view_groups_standard_views_ibfk_2` FOREIGN KEY (`standard_view_id`) REFERENCES `standard_views` (`id`),
  CONSTRAINT `standard_view_groups_standard_views_ibfk_3` FOREIGN KEY (`standard_view_group_id`) REFERENCES `standard_view_groups` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `standard_views` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(64) default NULL,
  `part_id` int(11) NOT NULL,
  `image_view_id` int(11) NOT NULL,
  `stage` varchar(32) default NULL,
  `sex` varchar(32) default NULL,
  `notes` text,
  `proj_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `identifier` varchar(255) default NULL,
  `namespace_id` int(11) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `name` (`name`,`proj_id`),
  KEY `proj_id_2` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `standard_views_namespace_fk` (`namespace_id`),
  CONSTRAINT `standard_views_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `standard_views_ibfk_2` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `standard_views_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `standard_views_ibfk_4` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `standard_views_namespace_fk` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=14 DEFAULT CHARSET=latin1;

CREATE TABLE `statements` (
  `id` int(11) NOT NULL auto_increment,
  `as_cited_in` int(11) default NULL,
  `content_id` int(11) default NULL,
  `attributed_to` int(11) default NULL COMMENT 'who made this statement (if not cited)?',
  `pages` varchar(64) default NULL,
  `figures` varchar(64) default NULL,
  `revision_history` text,
  `temp_citation` text,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `as_cited_in_ind` (`as_cited_in`),
  KEY `content_id_ind` (`content_id`),
  KEY `attributed_to_ind` (`attributed_to`),
  CONSTRAINT `statements_ibfk_1` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `statements_ibfk_2` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `statements_ibfk_3` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `statements_ibfk_4` FOREIGN KEY (`as_cited_in`) REFERENCES `refs` (`id`),
  CONSTRAINT `statements_ibfk_5` FOREIGN KEY (`content_id`) REFERENCES `contents` (`id`),
  CONSTRAINT `statements_ibfk_6` FOREIGN KEY (`attributed_to`) REFERENCES `people` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE `tags` (
  `id` int(11) NOT NULL auto_increment,
  `keyword_id` int(11) default NULL,
  `addressable_id` int(11) default NULL,
  `addressable_type` varchar(64) default NULL,
  `notes` text,
  `ref_id` int(11) default NULL,
  `pages` varchar(255) default NULL,
  `pg_start` varchar(8) default NULL,
  `pg_end` varchar(8) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  `referenced_object` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `keyword_id` (`keyword_id`,`addressable_id`,`addressable_type`,`ref_id`),
  KEY `keyword_id_2` (`keyword_id`),
  KEY `addressable_id` (`addressable_id`),
  KEY `addressable_type` (`addressable_type`(4)),
  KEY `ref_id` (`ref_id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `index_tags_on_referenced_object` (`referenced_object`)
) ENGINE=InnoDB AUTO_INCREMENT=61809 DEFAULT CHARSET=latin1;

CREATE TABLE `taxon_hists` (
  `id` int(11) NOT NULL auto_increment,
  `taxon_name_id` int(11) NOT NULL,
  `higher_id` int(11) default NULL,
  `genus_id` int(11) default NULL,
  `subgenus_id` int(11) default NULL,
  `species_id` int(11) default NULL,
  `subspecies_id` int(11) default NULL,
  `author` varchar(255) default NULL,
  `year` varchar(6) default NULL,
  `varietal_id` int(11) default NULL,
  `varietal_usage` varchar(24) default NULL,
  `ref_id` int(11) default NULL,
  `ref_page` varchar(64) default NULL,
  `taxon_name_status_id` int(11) default NULL,
  `notes` text,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `higher_id` (`higher_id`),
  KEY `taxon_name_id` (`taxon_name_id`),
  KEY `genus_id` (`genus_id`),
  KEY `subgenus_id` (`subgenus_id`),
  KEY `species_id` (`species_id`),
  KEY `subspecies_id` (`subspecies_id`),
  KEY `varietal_id` (`varietal_id`),
  KEY `ref_id` (`ref_id`),
  KEY `taxon_name_status_id` (`taxon_name_status_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `taxon_name_id_2` (`taxon_name_id`),
  CONSTRAINT `taxon_hists_ibfk_1` FOREIGN KEY (`taxon_name_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_hists_ibfk_10` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `taxon_hists_ibfk_11` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `taxon_hists_ibfk_2` FOREIGN KEY (`higher_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_hists_ibfk_3` FOREIGN KEY (`genus_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_hists_ibfk_4` FOREIGN KEY (`subgenus_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_hists_ibfk_5` FOREIGN KEY (`species_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_hists_ibfk_6` FOREIGN KEY (`subspecies_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_hists_ibfk_7` FOREIGN KEY (`varietal_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_hists_ibfk_8` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `taxon_hists_ibfk_9` FOREIGN KEY (`taxon_name_status_id`) REFERENCES `taxon_name_status` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3061 DEFAULT CHARSET=latin1;

CREATE TABLE `taxon_name_status` (
  `id` int(11) NOT NULL auto_increment,
  `status` varchar(128) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `status` (`status`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `taxon_name_status_ibfk_1` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `taxon_name_status_ibfk_2` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1054 DEFAULT CHARSET=latin1;

CREATE TABLE `taxon_names` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL,
  `author` varchar(128) default NULL,
  `year` varchar(4) default NULL,
  `nominotypical_subgenus` tinyint(1) default NULL,
  `parent_id` int(11) default NULL,
  `valid_name_id` int(11) default NULL,
  `namespace_id` int(11) default NULL,
  `external_id` int(11) default NULL,
  `taxon_name_status_id` int(11) default NULL,
  `l` int(11) default NULL,
  `r` int(11) default NULL,
  `orig_genus_id` int(11) default NULL,
  `orig_subgenus_id` int(11) default NULL,
  `orig_species_id` int(11) default NULL,
  `iczn_group` varchar(8) default NULL,
  `type_type` varchar(255) default NULL,
  `type_count` int(11) default NULL,
  `type_sex` varchar(255) default NULL,
  `type_repository_id` int(11) default NULL,
  `type_repository_notes` varchar(255) default NULL,
  `type_geog_id` int(11) default NULL,
  `type_locality` text,
  `type_notes` varchar(255) default NULL COMMENT 'e.g. lost, number of males and females, etc',
  `type_taxon_id` int(11) default NULL,
  `type_by` varchar(64) default NULL,
  `type_lost` tinyint(1) default NULL,
  `ref_id` int(11) default NULL,
  `page_validated_on` int(11) default NULL,
  `page_first_appearance` int(11) default NULL,
  `notes` text,
  `import_notes` text,
  `cached_display_name` varchar(255) default NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `namespace_id` (`namespace_id`,`external_id`),
  KEY `name` (`name`),
  KEY `namespace_id_2` (`namespace_id`),
  KEY `external_id` (`external_id`),
  KEY `parent_id` (`parent_id`),
  KEY `l` (`l`),
  KEY `r` (`r`),
  KEY `valid_name_id` (`valid_name_id`),
  KEY `taxon_name_status_id` (`taxon_name_status_id`),
  KEY `type_taxon_id` (`type_taxon_id`),
  KEY `ref_id` (`ref_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  CONSTRAINT `taxon_names_ibfk_1` FOREIGN KEY (`parent_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_names_ibfk_2` FOREIGN KEY (`valid_name_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_names_ibfk_3` FOREIGN KEY (`taxon_name_status_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_names_ibfk_4` FOREIGN KEY (`type_taxon_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `taxon_names_ibfk_5` FOREIGN KEY (`namespace_id`) REFERENCES `namespaces` (`id`),
  CONSTRAINT `taxon_names_ibfk_6` FOREIGN KEY (`ref_id`) REFERENCES `refs` (`id`),
  CONSTRAINT `taxon_names_ibfk_7` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `taxon_names_ibfk_8` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=33937 DEFAULT CHARSET=latin1;

CREATE TABLE `tree_nodes` (
  `id` int(11) NOT NULL auto_increment,
  `parent_id` int(11) default NULL,
  `tree_id` int(11) NOT NULL,
  `label` varchar(255) default NULL,
  `branch_length` double default NULL,
  `cumulative_branch_length` double default NULL,
  `otu_id` int(11) default NULL,
  `depth` int(11) default NULL,
  `lft` int(11) default NULL,
  `rgt` int(11) default NULL,
  PRIMARY KEY  (`id`),
  KEY `tree_id` (`tree_id`),
  KEY `otu_id` (`otu_id`),
  KEY `parent_id` (`parent_id`),
  KEY `lft` (`lft`),
  KEY `rgt` (`rgt`),
  KEY `lft_2` (`lft`,`rgt`),
  CONSTRAINT `tree_nodes_ibfk_1` FOREIGN KEY (`otu_id`) REFERENCES `otus` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=206 DEFAULT CHARSET=latin1;

CREATE TABLE `trees` (
  `id` int(11) NOT NULL auto_increment,
  `tree_string` longtext,
  `name` varchar(255) default NULL,
  `data_source_id` int(11) default NULL,
  `notes` text,
  `max_depth` int(11) default NULL,
  `proj_id` int(11) NOT NULL,
  `creator_id` int(11) NOT NULL,
  `updator_id` int(11) NOT NULL,
  `updated_on` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `created_on` timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`id`),
  KEY `proj_id` (`proj_id`),
  KEY `creator_id` (`creator_id`),
  KEY `updator_id` (`updator_id`),
  KEY `data_source_id` (`data_source_id`),
  CONSTRAINT `trees_ibfk_1` FOREIGN KEY (`data_source_id`) REFERENCES `data_sources` (`id`),
  CONSTRAINT `trees_ibfk_2` FOREIGN KEY (`proj_id`) REFERENCES `projs` (`id`),
  CONSTRAINT `trees_ibfk_3` FOREIGN KEY (`creator_id`) REFERENCES `people` (`id`),
  CONSTRAINT `trees_ibfk_4` FOREIGN KEY (`updator_id`) REFERENCES `people` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=7 DEFAULT CHARSET=latin1;

CREATE TABLE `type_specimens` (
  `id` int(11) NOT NULL auto_increment,
  `specimen_id` int(11) default NULL,
  `taxon_name_id` int(11) NOT NULL,
  `type_type` varchar(24) NOT NULL,
  `notes` text,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `specimen_id_2` (`specimen_id`,`taxon_name_id`),
  KEY `specimen_id` (`specimen_id`),
  KEY `taxon_name_id` (`taxon_name_id`),
  CONSTRAINT `type_specimens_ibfk_1` FOREIGN KEY (`taxon_name_id`) REFERENCES `taxon_names` (`id`),
  CONSTRAINT `type_specimens_ibfk_2` FOREIGN KEY (`specimen_id`) REFERENCES `specimens` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=707 DEFAULT CHARSET=utf8;

CREATE TABLE `xyl_hosts` (
  `id` int(11) NOT NULL auto_increment,
  `taxon_name_id` int(11) NOT NULL,
  `name` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  KEY `taxon_name_id` (`taxon_name_id`),
  CONSTRAINT `xyl_hosts_ibfk_1` FOREIGN KEY (`taxon_name_id`) REFERENCES `taxon_names` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3601 DEFAULT CHARSET=latin1;

CREATE TABLE `xyl_syn_errors` (
  `tax_id` int(11) NOT NULL,
  `external_id` int(11) NOT NULL,
  `name` varchar(255) default NULL,
  `genus` varchar(255) default NULL
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

INSERT INTO schema_migrations (version) VALUES ('1');

INSERT INTO schema_migrations (version) VALUES ('20080721011559');

INSERT INTO schema_migrations (version) VALUES ('20080822234357');

INSERT INTO schema_migrations (version) VALUES ('20080824161118');

INSERT INTO schema_migrations (version) VALUES ('20081203035354');

INSERT INTO schema_migrations (version) VALUES ('20081208030634');

INSERT INTO schema_migrations (version) VALUES ('20090308210205');

INSERT INTO schema_migrations (version) VALUES ('20090317124307');

INSERT INTO schema_migrations (version) VALUES ('20090722192800');

INSERT INTO schema_migrations (version) VALUES ('20090730190113');

INSERT INTO schema_migrations (version) VALUES ('20090731123736');

INSERT INTO schema_migrations (version) VALUES ('20090804155702');

INSERT INTO schema_migrations (version) VALUES ('20090806160057');

INSERT INTO schema_migrations (version) VALUES ('20090807131513');

INSERT INTO schema_migrations (version) VALUES ('20090807144546');

INSERT INTO schema_migrations (version) VALUES ('20090909182434');

INSERT INTO schema_migrations (version) VALUES ('20090909201436');

INSERT INTO schema_migrations (version) VALUES ('20090911132952');

INSERT INTO schema_migrations (version) VALUES ('20090911135054');

INSERT INTO schema_migrations (version) VALUES ('20090915013539');

INSERT INTO schema_migrations (version) VALUES ('20090915165049');

INSERT INTO schema_migrations (version) VALUES ('20090916162632');

INSERT INTO schema_migrations (version) VALUES ('20090917131116');

INSERT INTO schema_migrations (version) VALUES ('20090917165634');

INSERT INTO schema_migrations (version) VALUES ('20090918190245');

INSERT INTO schema_migrations (version) VALUES ('20090920181928');

INSERT INTO schema_migrations (version) VALUES ('20090921181907');

INSERT INTO schema_migrations (version) VALUES ('20090930163041');

INSERT INTO schema_migrations (version) VALUES ('20091008140345');

INSERT INTO schema_migrations (version) VALUES ('20091009211243');

INSERT INTO schema_migrations (version) VALUES ('20091012182110');

INSERT INTO schema_migrations (version) VALUES ('20091020140705');

INSERT INTO schema_migrations (version) VALUES ('20091102195131');

INSERT INTO schema_migrations (version) VALUES ('20091108151715');

INSERT INTO schema_migrations (version) VALUES ('20091117224036');

INSERT INTO schema_migrations (version) VALUES ('20091123150322');

INSERT INTO schema_migrations (version) VALUES ('20091123204050');

INSERT INTO schema_migrations (version) VALUES ('20091129001740');