
# a class which organizes Objects with a taxon_name_id into sets for use in display
# does *NOT* take into account visibility at present, assumes the whole heirarchy is present
# this is essentially the intersection of two  trees

# we have 1:1, 1:many, and many:many

class ByTnDisplay
  attr_accessor :sections, :unplaced_items

  # takes a TaxonName and an arry of objects that have a taxon_name_id
  def initialize(taxon_name, items = [])
    @tn = taxon_name # the root name
    @items = items
    
    @unplaced_items = [] 
    @sections = []

    # grab the unplaced items
    @items.each do |i|
      if i.taxon_name_id == nil
        @unplaced_items << i
      end
    end
   
    @unplaced_items.each do |i|    # have to do this outside the loop or wonkiness ensues
      @items.delete(i)
    end

    @tn_set = @tn.full_set # grab the set so we can speedup 
    s = {} # internal tracking for building the sections, tn.id => a @section index
    
    @tn.full_set.each do |t| # we have to outer loop for order
      j = 0
      @items[j..@items.size-1].each do |i| 
        if i.taxon_name_id == t.id
          # if the section exists add it there
          if s.keys.include?(t.id)
            @sections[s[t.id]].items << i
          else
            sn = ByTnDisplay::Section.new(t)     # make a new section
            sn.items << i # add the initial object to it
            s.update(t.id => (@sections.size - 1)) # create an internal reference by taxon name
            @sections << sn
          end
        end
      end
      j += 1
    end
    @sections.reverse!
  end

  class Section
    attr_reader :header, :items
    # header is a taxon_name
    def initialize(header)
      @header = header
      @items = []
    end
  end

end
