module Ontology::Obo2mx 

  # Maps an OBO file to a full ontology in mx, instantiating new Parts, Tags, and Ontologies where necessary
  # Returns a array of arrays, each of which contains a hash  
  # Array index maps 1:1 with order of terms in incoming :file, it looks like this:
  # [ [ :label => Label, :tags => [ Tag1, ...Tagn], :ontology_classes => [OntologyClass1...OntologyClassn], ... [] ]
  # Could lib/class this out, but likely a waste of time.
  def self.terms_from_obo(options = {})
    opt = {
      :file => nil,
      :proj_id => nil
    }.merge!(options.symbolize_keys)

    return false if (opt[:file].blank? || opt[:file].size == 0 )
    proj = Proj.find(opt[:proj_id])
    of = parse_obo_file(opt[:file])
    result = {}
    obo_map  = {}

    # handle the Labels, Classes 
    of.terms.each_with_index do |term, i|
      obo_map[i] = {:label => nil, :tags => Array, :ontology_classes => Array}

      # there are multiple classes possible within mx
      oc = OntologyClass.new(:definition => (term.tags["def"].blank? ? nil : term.tags["def"].first))
      
      if l = Label.find(:first, :conditions => {:name => term.name, :proj_id => opt[:proj_id]}) 
        obo_map[i] = {:label => l, :ontology_classes => t.ontology_classes}
        obo_map[i][:ontology_classes].unshift(oc)  # store the OBO files "class" as the first in the list, there will always be one
      else 
        obo_map[i] = { :label =>  Label.new(:name => term.name),
        :ontology_classes => [oc] }
      end

      # handle the Tags
      # TODO: assume they are attached to classes, not labels
      # Tags need to have their addressable_id set post OntologyClass#save
      tags = []
      tags += [Tag.new(:addressable_type => 'OntologyClass', :keyword => nil, :referenced_object => term.id)]

      # additional xrefs (Tags in mx)
      if !term.tags["xref"].blank?
        tags += term.tags["xref"].collect{|x| Tag.new(:addressable_type => 'OntologyClass', :keyword => nil, :referenced_object => x) }
      end 
      obo_map[i].merge!(:tags => tags)
    end 
   
    # do a second pass to derive the OntologyRelationships 
    # properties, only handles is a, part of at this point
    isa = proj.object_relationships.by_interaction("is_a").first
    part_of = proj.object_relationships.by_interaction("part_of").first


    if !isa.blank? && !part_of.blank?
     
      obo_map.keys.each do |i|
        obo_map[i].merge!(:ontology_relationships => []) # why can't this be initiated above ... weird
        ["is_a", "part_of"].each do |rel|
          if !of.terms[i].tags[rel].blank?
            os = []  
             of.terms[i].tags[rel].each do |id|
             
               # part1 is self
               # part2 is index
                os << {:ontology_relationships => OntologyRelationship.new(:ontology_class1_id => nil, :ontology_class2_id => nil, :object_relationship_id => (rel == "is_a" ? isa.id : part_of.id)),
                       :ontology_class2_index => self._index_of_term_from_obo_map_for_term(of, id) }
             end
            obo_map[i][:ontology_relationships] += os 
          end
        end
      end
    end

    # <OboFile::OboFile::Term:0x4d7daa8 @name="cell", @id="CL:0000000", @tags={"xref"=>["FMA:68646"], "def"=>["\"Anatomical structure that has as its parts a maximally connected cell compartment surrounded by a plasma membrane.\" [CARO:mah]"]}>
    result = obo_map
  end

  def self._index_of_term_from_obo_map_for_term(parsed_obo_file, xref)
    parsed_obo_file.terms.each_with_index do |t,i|
      return i if xref == t.id 
    end
    false
  end




end


